/*---------------------------------------------------------------------------
 *
 *      Title:          Farsi - Extend Subsystem (Arexx)
 *
 *      File:           extend.c
 *
 *      Date:           May 1995
 *
 *      Description:
 *          This module implements the extension interface, which on the
 *      amiga is via Arexx. Currently for future expansion...
 *
 *      Rev     By              Description                            Date
 *      1.0     R.J.Harrison    Implement                              2-5-95
 *
 *      Copyright  1995 Zaretto Ltd.
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "zpl.h"

#include	<exec/types.h>

#include	<proto/exec.h>
#include	<proto/intuition.h>

#include	<rexx/storage.h>
#include	<rexx/rxslib.h>


#include	"Sim_Rexx.h"

#define NUMBER_OF_COMMANDS 55

static AREXXCONTEXT RexxStuff;
static ULONG signal_mask;

typedef struct PB
{
    TY_int to, new;
    char *result;
    char *error_text;
    TY_int error_code;
}
PB;

static PB pb;
static TOK_ROOT cdl_root;
static int ex_debug = 0, exit_request = 0;

static void ex_help_command (char *command)
{
    TY_int ci, cp;
    TOK *ctok, *tok;

    tok = cdl_root.table;
    ci = find_command (tok, command, cdl_root.num_commands);
    if (ci)
    {
	ctok = &tok[ci];

printf ("Command '%s' ",ctok->command);

if(ctok->number_of_parameters)
{
printf ("  - parameters as follows:\n",command);

printf ("\n\t%-20s  %-20s  %s\n", "Parameter", "Min Abbreviation", "Type");
	for (cp = 0; cp < ctok->number_of_parameters; cp++)
	{
	    PARAM *p = ctok->parameters[cp];

	    printf ("\t/%-20s /%-20s %s\n",p->parameter, p->min_abb_p, CDL_describe_var (p));
	}
}
else
printf ("  - No parameters:\n",command);
printf ("\n\n");
}
}

void ex_help_all (UI_ARGS)
{
    TY_int c;
    TOK *ctok;
    TOK *tok;
    char line[200];

    *line = 0;
    tok = cdl_root.table;
    for (c = 0; c < cdl_root.num_commands; c++)
    {
	ctok = &tok[c];
	ex_help_command (ctok->command);
    }
}

static void ex_quit (UI_ARGS)
{
exit_request = TRUE;
}

static TY_int ex_init_parse (void)
{
    TOK *tok, *tok_storage;

    tok_storage = calloc (NUMBER_OF_COMMANDS, sizeof (TOK));

    if (!tok_storage)
	return 0;

    init_cdl (tok_storage, NUMBER_OF_COMMANDS);

    tok = add_command ("exit", ex_quit);

/*
 * Debugging commands--
 */
{
    extern TY_int re_debug;
    extern long interactive_mode;

    tok = add_command ("set", 0);
    add_parameter (tok, "re_debug", (void *) &re_debug, INTEGER, NULL);
    add_parameter (tok, "interactive", (void *) &interactive_mode, INTEGER, NULL);

    tok = add_command ("check_memory", MY_ov_check_memory);
    tok = add_command ("dump_memory", MY_dump_memory);
}

    abbreviate_commands ();
    store_cdl (&cdl_root);

    return (0);
}

TY_int EX_init (void)
{
    RexxStuff = InitARexx ("FARSI", "EDIT");

    signal_mask = ARexxSignal (RexxStuff);
    ex_init_parse ();
    return 1;
}

char *EX_describe (void)
{
    static char buf[80];
    if (RexxStuff)
	sprintf (buf, "ARexx port name %s", ARexxName (RexxStuff));
    else
	sprintf (buf, "Arexx not installed");

    return buf;
}

void EX_finish (void)
{
    FreeARexx (RexxStuff);
}

ULONG EX_get_signal_mask (void)
{
    return signal_mask;
}


static TY_int ex_parseline (char *in_line)
{
    char *ch, *line = skip_sp (in_line);

    if (*line)
    {
	set_cdl (&cdl_root);
	set_cdl_interactive (FALSE);

	memset (&pb, 0, sizeof (pb));

	if (ch = check_command (line, 0))
	{
	    pb.error_text = ch;
	    pb.error_code = 1;
	    return (1);
	}

	set_cdl_interactive (TRUE);
    }
    return (0);
}

void EX_process_message (ULONG mask)
{
    struct RexxMsg *rmsg;

    if (!(mask & signal_mask))
	return;

exit_request = FALSE;

    /*
     * Process the ARexx messages...
     */

    while (rmsg = GetARexxMsg (RexxStuff))
    {
	ex_parseline (ARG0 (rmsg));

	if (pb.error_code)
	{
	    SetARexxLastError (RexxStuff, rmsg, pb.error_text);
	}
if(ex_debug)
{
if (pb.result)
printf ("Result %s\n",pb.result);
else
printf ("No Result\n");
}
	ReplyARexxMsg (RexxStuff, rmsg, pb.result, pb.error_code);
    }
    if(exit_request)
        terminate (0);
}
