/*---------------------------------------------------------------------------
 *
 *      Title:          Farsi Text Editor - Preferences module
 *
 *      File:           prefs.c
 *
 *      Date:           May 1995
 *
 *      Description:
 *          Provides interface to preference items, ie. those which are set
 *      by the user, *or* maintained to aid the user.
 *
 *      Rev     By              Description                            Date
 *      1.0     R.J.Harrison    Implement                              2-5-95
 *
 *      Copyright  1995 Zaretto Ltd.
 *---------------------------------------------------------------------------*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>

#include "zpl.h"

static GENERIC_PREF *cp_p=0, *sc_p=0, *gp_p=0;

static char *PR_default (char *str, char *def)
{
    if (!*str)
	strcpy (str, def);
    return (str);
}

static void sc_default (void *p1)
{
SCREEN_PREFS * cp=p1;

    strcpy (cp->name, "Screen");
    cp->depth = 2;
    cp->width = 640;
    cp->height = 256;
    cp->use_custom_screen = TRUE;
    cp->display_id = HIRES_KEY;
    cp->lace = FALSE;
    cp->hires = TRUE;
    cp->wb_version = 37;
    strcpy (cp->public_screen_to_use, "Workbench");
}

static void PR_gp_default (void *p1)
{
GENERAL_PREFS * gp_p=p1;

memset(gp_p,0,sizeof(*gp_p));

    gp_p->caching = FALSE;
    gp_p->granularity = 20;
    gp_p->batch_scroll = 2;
    gp_p->right_margin = 76;
    gp_p->ask_before_quit = TRUE;
    gp_p->scrollback_size = 200;
    gp_p->scrollback_remove = 20;
    gp_p->do_dynamic_updates = TRUE;

    strcpy(gp_p->re_font.name,"topaz.font");
    gp_p->re_font.y_size = 8;
    gp_p->re_font.flags  = 0x60;

    strcpy(gp_p->win_font.name,"TT.font");
    gp_p->win_font.y_size = 10;
    gp_p->re_font.flags  = 0x60;
}

static void cp_default (void *p1)
{
COLOUR_PREFS * cp=p1;

    cp->prompt = 1;
    cp->foreground = 1;
    cp->background = 1;
    cp->header = 1;
    cp->body = 1;
    cp->error = 2;
}

static TY_int palette_default[] =
{0xa98, 0x0, 0xec9, 0xf00, 0xf, 0xf0f, 0xff, 0xff};


SCREEN_PREFS *PR_get_screen_prefs (void)
{
return (sc_p->pe->data);
}

TY_int PR_default_colour (TY_int a)
{
    if (!a)
	return (1);
    return (a);
}

GENERAL_PREFS *PR_get_general_prefs (void)
{
if(gp_p)
return (gp_p->pe->data);
else
return 0;
}

COLOUR_PREFS *PR_get_colour_prefs (void)
{
return (cp_p->pe->data);
}

TY_int PR_get_colour (TY_int a)
{
    COLOUR_PREFS *cp = PR_get_colour_prefs ();
#ifdef AMIGA
    TY_int num_colours = 1 << get_screen_depth ();
#else
    TY_int num_colours = 8;
#endif

    switch (a)
    {
    case MESSAGE_HEADER:
	return min (num_colours - 1, cp->header);
    case MESSAGE_HEADER_INTERESTING:
	return min (num_colours - 1, cp->interesting);
    case MESSAGE_BODY:
	return min (num_colours - 1, cp->body);
    case ERROR_COLOUR:
	return min (num_colours - 1, cp->error);
    default:
	return min (num_colours - 1, cp->foreground);
    }
}

static TY_int check_changes (GENERIC_PREF *gp)
{
return (gp && gp->pe->data && gp->original && memcmp (gp->original, gp->pe->data, gp->pe->size));
}

TY_int PR_check_changes ()
{
if (check_changes(gp_p) ||
check_changes(cp_p) ||
check_changes(sc_p))
	return (TRUE);
    else
	return (FALSE);
}

static void reset_changes (GENERIC_PREF *gp)
{
if (gp && gp->pe->data && gp->original)
memcpy (gp->original, gp->pe->data, gp->pe->size);
}

void PR_reset_changes ()
{
reset_changes(gp_p);
reset_changes(cp_p);
reset_changes(sc_p);
}

void init_prefs (void)
{
    char op_line[240];

    *op_line = 0;
    sc_p = PR_init_load (sizeof(SCREEN_PREFS),  "Screen.Preferences", sc_default, op_line);
    cp_p = PR_init_load (sizeof(COLOUR_PREFS),  "Colour.Preferences", cp_default, op_line);
    gp_p = PR_init_load (sizeof(GENERAL_PREFS), "General.Preferences", PR_gp_default, op_line);

    if (*op_line)
    {
    PR_save_database (PR_get_prefs ());
    PR_reset_changes ();
    }
}

TY_int PR_get_granularity ()
{
    GENERAL_PREFS *gp_p = PR_get_general_prefs ();
    return (max (1, gp_p->granularity));

}

char *PR_get_printer_device ()
{
    GENERAL_PREFS *gp=PR_get_general_prefs();
    return (gp->printer_device);
}

#ifdef M_AMIGA
struct TextAttr *PR_get_re_font (void)
{
    GENERAL_PREFS *gp_p = PR_get_general_prefs ();
    static struct TextAttr tf;
    convert_font_to_attr (&gp_p->re_font, &tf);
    return (&tf);
}

struct TextAttr *PR_get_win_font (void)
{
    GENERAL_PREFS *gp_p = PR_get_general_prefs ();
    static struct TextAttr tf;
    convert_font_to_attr (&gp_p->win_font, &tf);
    return (&tf);
}

struct TextAttr *PR_get_fw_font (void)
{
    GENERAL_PREFS *gp_p = PR_get_general_prefs ();
    static struct TextAttr tf;
    convert_font_to_attr (&gp_p->fw_font, &tf);
    return (&tf);
}
#else
struct TextAttr *PR_get_re_font ()
{
    GENERAL_PREFS *gp_p = PR_get_general_prefs ();
    return (&gp_p->re_font);
}

struct TextAttr *PR_get_fw_font ()
{
    GENERAL_PREFS *gp_p = PR_get_general_prefs ();
    return (&gp_p->fw_font);
}
#endif

/*
 * Non-zero means abort exit
 */
TY_int PR_ask_before_quit ()
{
    GENERAL_PREFS *gp_p = PR_get_general_prefs ();

    if (gp_p->ask_before_quit)
    {
        if(!do_it ("Really Exit", 0))
            return(1);
    }
    return PR_finish();
}

char *PR_get_default_path ()
{
    GENERAL_PREFS *pn_p = PR_get_general_prefs ();

    return (pn_p->default_path);
}

TY_int PR_add_formfeed ()
{
    GENERAL_PREFS *gp = PR_get_general_prefs ();
    return (gp->add_formfeed);
}
