#ifndef __WIN_INCLUDE
#define __WIN_INCLUDE
/*---------------------------------------------------------------------------
 *
 *      Title:          Main window definitinos
 *
 *      File:           win.h
 *
 *      Date:           October 1991
 *
 *      Description:
 *              This header defines the structures used by the windows and
 *      menus, together with the window classes.
 *
 *      Rev: 1.1
 *      1.0     R.J.Harrison    Add header                              March 1993
 *      1.1                     Add scrollback and window class defs    March 1993
 *      Copyright  1993 Zaretto LTD.
 *---------------------------------------------------------------------------*/

#define RESOURCE_WINDOW_ID  4
#define INFO_WINDOW_ID      5
#define ERROR_WINDOW_ID     7

extern char *resource_database;
extern char *preference_database;

#ifdef M_AMIGA
#include <exec/lists.h>
#include <intuition/intuition.h>
#define MD_MENU struct Menu
#define MD_MENU_ITEM struct MenuItem *
#define MD_GADGET_P struct Gadget *
#define MD_GADGET struct Gadget
#define MD_NEWGADGET_P struct NewGadget *
#define MD_NEWGADGET struct NewGadget
#define MD_WINDOW struct Window
#define MD_WINDOW_P struct Window *
#define GET_IO_REQUEST(a) (struct IORequest *)(a)
#endif

#ifdef WINDOWS
#include "ms_win.h"
typedef struct
{
    HMENU menu;
    char *title;
}
MD_MENU;
//#define WIN_BUF // SMART_REFRESH WINDOWS PLEASE...

#define MD_MENU_ITEM HMENU
#define MD_GADGET HWND
#define MD_NEWGADGET_P struct NewGadget *
#define MD_NEWGADGET struct NewGadget
#define MD_GADGET_P HWND
#define MD_WINDOW HWND
#define MD_WINDOW_P HWND
#endif

#ifdef VMS
typedef struct
  {
      void *menu_bar, *pulldown, *cascade; /* Menu Bar Widgets */
      char *title;
  }
MD_MENU;

typedef struct
  {
      MD_MENU *menu_root;
      void *mw; /* Menu Widget */
  }
MD_MENU_ITEM_ST;

typedef struct
  {
      void *fw;
      void (*func) (TY_int, void *);
  }
MD_CALLBACK_ITEM;

#define MD_MENU_ITEM MD_MENU_ITEM_ST *

/*
 * The following aren't right....~~
 */
#define MD_GADGET_P struct Gadget *
#define MD_GADGET struct Gadget
#define MD_NEWGADGET_P struct NewGadget *
#define MD_NEWGADGET struct NewGadget
#define MD_WINDOW int
#define MD_WINDOW_P int
#endif

#include "types.h"

#define MAX_COLOURS 16

#define FW_TYPE_INTERACTIVE     ((long)1 <<  1)
#define FW_TYPE_OVERVIEW        ((long)1 <<  2)

#define FW_DISALLOW_CLOSE       ((long)1 <<  3) 
#define FW_TYPE___            ((long)1 <<  4)
#define FW_TYPE____            ((long)1 <<  5)

#define FW_TYPE_GADTOOLS        ((long)1 <<  6) 
#define FW_TYPE_IO              ((long)1 <<  7) 
#define FW_TYPE_OUTPUT          ((long)1 <<  8) 
#define FW_PROCESSING           ((long)1 <<  9) 
#define FW_TYPE_MAIL            ((long)1 <<  10)
#define FW_LINE_RECALL_DISABLED ((long)1 <<  11)
#define FW_SCROLLBACK           ((long)1 <<  12)
#define FW_CURSOR               ((long)1 <<  13) 
#define FW_WAITING_FOR_MORE     ((long)1 <<  14)
#define FW_NO_SCROLL            ((long)1 <<  15)

#define WINDOW_HAS_MENU (FW_TYPE_INTERACTIVE|FW_TYPE_OVERVIEW)
#define HISTORY_SIZE 20
#define INPUT_LINE_SIZE 132
#define STR_SIZE 200
#define SCROLLBACK_LENGTH STR_SIZE

#define FW_EV_SIZEVERIFY        IDCMP_SIZEVERIFY
#define FW_EV_NEWSIZE           IDCMP_NEWSIZE
#define FW_EV_REFRESHWINDOW     IDCMP_REFRESHWINDOW
#define FW_EV_MOUSEBUTTONS      IDCMP_MOUSEBUTTONS
#define FW_EV_MOUSEMOVE         IDCMP_MOUSEMOVE
#define FW_EV_GADGETDOWN        IDCMP_GADGETDOWN
#define FW_EV_GADGETUP          IDCMP_GADGETUP
#define FW_EV_REQSET            IDCMP_REQSET
#define FW_EV_MENUPICK          IDCMP_MENUPICK
#define FW_EV_CLOSEWINDOW       IDCMP_CLOSEWINDOW
#define FW_EV_RAWKEY            IDCMP_RAWKEY
#define FW_EV_REQVERIFY         IDCMP_REQVERIFY
#define FW_EV_REQCLEAR          IDCMP_REQCLEAR
#define FW_EV_MENUVERIFY        IDCMP_MENUVERIFY
#define FW_EV_NEWPREFS          IDCMP_NEWPREFS
#define FW_EV_DISKINSERTED      IDCMP_DISKINSERTED
#define FW_EV_DISKREMOVED       IDCMP_DISKREMOVED
#define FW_EV_ACTIVEWINDOW      IDCMP_ACTIVEWINDOW
#define FW_EV_INACTIVEWINDOW    IDCMP_INACTIVEWINDOW
#define FW_EV_DELTAMOVE         IDCMP_DELTAMOVE
#define FW_EV_VANILLAKEY        IDCMP_VANILLAKEY
#define FW_EV_INTUITICK         IDCMP_INTUITICK

#define NO_CURSOR -1

typedef TY_int TEXT_COORDINATE;

#define STRIP_BIT(a,b) (a &= ~b)
#define SET_BIT(a,b)   (a |=  b)

#define LAYOUTABLE ((long)1 <<  16)
#define WINDOW_ID_MASK 0xff

#define UI_ARGS                 TY_int code, ZPL_WINDOW * fw
#define UI_ARGS_PASS            code, fw
#define UI_ARGS_PASS_WIN        0,fw    /*
					 * For sending only a window 
					 */
/*
 * General states
 */
#define ACTIVE        ((long)1 <<  0)
#define TOUCHED       ((long)1 <<  1)

typedef enum ACTION
{
    WM_GET_WINDOW_TYPE, WM_DRAW, WM_UPDATE,
    WM_GET_WINDOW_PREFS, WM_SET_WINDOW_PREFS, WM_MAKE_WINDOW_PREFS, WM_PROCESS_INPUT,
    WM_WINDOW_FINISH, WM_WINDOW_OPEN, WM_EVENT,
    WM_ADD_TOOLBOX, WM_TOOLBOX_REMOVED,
    WM_CONTENT_CHANGED, WM_ADD_GADGETS, WM_WINDOW_CLOSE,
    WM_REPLY_CHANGED, WM_TODO_CHANGED, WM_AB_CHANGED,
    WM_PL_CHANGED, WM_IM_CHANGED,
    WM_UNREAD_CHANGED, WM_NEW_CONFS, WM_MENU_GROUPS
} ACTION;

typedef struct MENUITEMS
{
    MD_MENU_ITEM item;
    TY_int group;
    void (*func) (TY_int, struct ZPL_WINDOW *);
    TY_int *read, last_value;
    struct MENUITEMS *next;
} MENUITEMS;

typedef struct
{
    TY_int colour, size;
    char output;                /*
				 * Used to indicate if the output is new...
				 */
#ifdef STATIC_SB
    char line[SCROLLBACK_LENGTH];
#else
    char *line;
#endif
}
SCROLLBACK_ITEM;

/*
 * The scrollback is stored using the GRAN_ITEM structure (to avoid frequent
 * malloc/free'ing. The header contains the following information.
 * number           - number of lines currently stored (upto the maximum)
 * displayed_upto   - more? prompt buffering
 * display_position - Position where the display should be
 * top              - position where the scrollback has put the display (or
 *                    the same value as display_position otherwise..)
 * scrollbar        - Resource entity for the scrollbar
 * new_output       - Set whenever there is output to display (FW_sb_add, cleared after FW_output_sb)
 */
typedef struct
{
    TY_int displayed_upto, display_position, top;
    QDB_GRAN_ITEM list;         /*
				 * The actual scrollback list 
				 */
    void *scrollbar;
    char new_output;
    TY_int more_upto;
}
SCROLLBACK;

/*
 * This is the basic class for windows.
 * However the linkage between this and the subclasses is not really
 * as I'd like it to be, ideally this class should be a self descripting entity,
 * from which the subclasses inherit the properties. Conceptually the subclasses
 * do inherit, however the implementation of the inheritance is via the item
 */
typedef struct ZPL_WINDOW
{
#ifdef AMIGA
    struct Window *w;
    struct RastPort *rp;
    struct ViewPort *vp;
#endif
#ifdef WINDOWS
    HWND w;
HDC dc;
HBITMAP bm;
#endif
#ifdef MOTIF
void *w;
void *top_widget;
#endif
    struct ZPL_WINDOW *next, *parent, *menu_from;     /*
							 * The window to which this is
							 * * attached 
							 */
    struct Gadget *input_gadget;
    TY_int type;

    TEXT_COORDINATE current_text_y, current_text_x, input_len, input_pos,
      input_x, input_y, cursor_x, cursor_y, scroll_y;

    TY_int gx, gy;              /*
				 * Graphics X&Y Position 
				 */
    TY_int scroll_y_pix, height, width, t_height, t_width;

    TY_int font_x_size, font_y_size;

    TY_int colour, gcolour;

    char input_line[INPUT_LINE_SIZE], *previous_lines[HISTORY_SIZE];
    TY_int highest_pl, current_pl, end_pl;
    struct GENERAL_WINDOW *sub_window;
    struct TextAttr *current_font;
    MENUITEMS *menulist;
#ifdef WINDOWS
    HMENU first_menu;
    HFONT font;
    RECT scroll_rect;
#else
    MD_MENU *first_menu;
#endif
    void *cdl_root;

    struct IOStdReq *write_req; /*
				 * I/O request block pointer 
				 */
    struct MsgPort *write_port; /*
				 * replyport for writes      
				 */
    struct IOStdReq *read_req;  /*
				 * I/O request block pointer 
				 */
    struct MsgPort *read_port;  /*
				 * replyport for reads       
				 */

    char *prompt;
    struct RE_resource *resources;

    void (*process_complete) (struct ZPL_WINDOW *, void *);
    TY_int (*process) (struct ZPL_WINDOW *, UWORD mclass, void *);
				/*
				 * Used when doing something.. (flags
				 * * & PROCESSING) 
				 */
    void *process_ptr, *process_ptr1;   /*
					 * pointer for above
					 * * procedure 
					 */

    char toolbox_added;
    TY_int lines_printed, more_disabled, window_id;
    struct ALLOC_TRACK *alloc_track;
    FILE *opfile;               /*
				 * File to which lines are output 
				 */
    char word_wrapping;
    SCROLLBACK scrollback;
    char *title;
    BOOL *active_groups;
    TY_int busy_count;
}
ZPL_WINDOW;

/*
 * Define the structure which is used to pass events around within the system
 */

#ifdef AMIGA
typedef struct IntuiMessage EVENT_MESSAGE;
#endif

#ifdef WINDOWS
typedef struct IntuiMessage {
HWND window;
UINT message;
WPARAM param_w;
LPARAM param_l;
char keycode;
int gadget_id;
TY_int id;
int event;
} EVENT_MESSAGE;
#endif

#ifdef MOTIF
typedef struct IntuiMessage {
void *window;
void *message;
char keycode;
int gadget_id;
TY_int id;
int event;
} EVENT_MESSAGE;
#endif

typedef struct
{
    TY_int age;
    ZPL_WINDOW *fw;
}

STD_TOPIC;

typedef struct
{
    TY_int action;
    ZPL_WINDOW *fw;
}

SS_SEARCH;

/*
 * Now define each of the window classes.
 */

/*
 * Windows.
 *     Our basic window class is a ZPL_WINDOW,
 *     however there are subclasses, as below.
 *
 */

typedef struct STD_WINDOW_HEADER
{
/*
 * First the 'backwards' link to the window
 * ~I'm not sure if we need this, as a true object should be self contained
 */
    struct ZPL_WINDOW *win;
      TY_int (*process) (void *, ACTION, TY_int, TY_int);
    TY_int type;
    char *class;
}
STD_WINDOW_HEADER;

typedef struct GENERAL_WINDOW
{
    STD_WINDOW_HEADER hdr;
} GENERAL_WINDOW;

typedef struct RESOURCE_WINDOW
{
    STD_WINDOW_HEADER hdr;
    void (*create) (struct RESOURCE_WINDOW *, void *);
    char *title;
    struct List lv_lis, lv_lis_1;
    void *ptr, *ptr1, *ptr2;
    ZPL_WINDOW *parent;
    void (*user_process) (struct RESOURCE_WINDOW *, ACTION, TY_int, TY_int);
    TY_int selected_item;
    char status[80];
} RESOURCE_WINDOW;
#endif
