/*---------------------------------------------------------------------------
 *
 *      Title:          Farsi Text Editor - User Interface
 *
 *      File:           ui.c
 *
 *      Date:           May 1995
 *
 *      Description:
 *          This module implements the user interface.
 *
 *      Rev     By              Description                            Date
 *      1.0     R.J.Harrison    Implement                             12-5-95
 *      1.1     R.J.Harrison    Fix clear page                        20-5-95
 *      1.2     R.J.Harrison    Fix tails being lost during save      12-6-95
 *
 *      Copyright  1995 Zaretto Ltd.
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>

#include "zpl.h"

static void do_update(RESOURCE_WINDOW *rw)
{
}

static char *make_pagename (int pn)
{
    static char  fname [200];
    char page_name[20];
    GENERAL_PREFS *gp = PR_get_general_prefs ();

    sprintf (page_name, "P%03d",pn);
    strcpy (fname, gp->default_path);
    add_part (fname, page_name, 200);

    return fname;
}


void do_clear (ZPL_WINDOW *fw)
{
    TEXT_PAGE      *page          = TEXT_PAGE_PTR(fw->sub_window);
    int i;

    fw->input_y = 1;
    fw->input_pos = 0;
    fw->input_len = 0;
    fw->input_x = 39;
    page->modified =0;
    memset (fw->input_line, 0, sizeof(fw->input_line));

        for (i=0;i<PAGELEN;i++)
        {
           memset(page->text[i], 0, MAXLEN);
        }
        display_page (fw);
}

static void text_clear (RE_resource_entry * re, UWORD id)
{
    ZPL_WINDOW *fw = re->root->window;

    if (fw)
    {
	if (do_it ("Really clear all text ?", NULL))
	{
            TEXT_PAGE      *page          = TEXT_PAGE_PTR(fw->sub_window);
            char  *fname = make_pagename(page->page_number);

            do_clear(fw);
            page->modified =1;
            set_status(fw,"Text cleared",0);
	}
	else
            set_status(fw,"Text clear cancelled",0);
    }
}

static int do_save_page (ZPL_WINDOW *fw)
{
    char           top [INPUT_LINE_SIZE];
    TEXT_PAGE      *page          = TEXT_PAGE_PTR(fw->sub_window);
    CHAR_TYPE      *ct_line = CHARTYPE_PTR (fw->sub_window);
    char  *fname = make_pagename(page->page_number);
    FILE *f;
int i;

    if (f=fopen(fname,"w"))
    {
int first_time = 1;
page->modified =0;
        for (i=0;i<PAGELEN;i++)
        {
           setup_line (page->text[i], ct_line, 0);
           convert_to_output (fw, top, page->text[i]);
if(first_time)
{
first_time=0;
fwrite (top+1, MAXLEN-1,1,f); 
}
else
          fwrite (top, MAXLEN,1,f); 
if (i < PAGELEN-1)
      fputc (' ',f);
        }
        fclose(f);
set_status(fw,"Page saved",0);
        return 1;
    }
return 0;
}

int do_load_page(ZPL_WINDOW *fw, int pn)
{
    TEXT_PAGE      *page          = TEXT_PAGE_PTR(fw->sub_window);
    char  *fname = make_pagename(pn);
    FILE *f;
    int i;

    page->page_number = pn;

    do_clear(fw);

    if (f=fopen(fname,"r"))
    {
        for (i=0;i<PAGELEN;i++)
        {
           memset(page->text[i], 0, MAXLEN);
           page->text[i][0] = ' ';
           fread (page->text[i]+1, MAXLEN+1,1,f); 
           convert_from_input (page->text[i]);
        }
        load_line(fw);
        fclose(f);
        display_page (fw);
set_status(fw,"Page %d loaded",pn);
        return 1;
    }
    return 0;
}

static void ask_to_save(ZPL_WINDOW *fw)
{
    TEXT_PAGE      *page          = TEXT_PAGE_PTR(fw->sub_window);
if (page->modified && do_it("Save changes to current page first ?",0))
do_save_page(fw);
}

static void load_page (RE_resource_entry * re, UWORD id)
{
    ZPL_WINDOW *fw = re->root->window;
    static int pn;

ask_to_save(fw);

    if (!ranged_numinput (fw, &pn, "Page number to load", 100, 899))
    {
        set_status (fw,"Page load cancelled",0);
        return;
    }

    if (!do_load_page (fw,pn))
        tell_em ("Page not found",0);
}

static void goto_page (RE_resource_entry * re, UWORD id)
{
    ZPL_WINDOW *fw = re->root->window;
    TEXT_PAGE      *page          = TEXT_PAGE_PTR(fw->sub_window);
    static int pn;

ask_to_save(fw);

    if (!ranged_numinput (fw, &pn, "Page number to goto", 100, 899))
    {
        set_status (fw,"Page goto cancelled",0);
        return;
    }

    page->page_number = pn;
    do_clear(fw);

set_status(fw,"Page %d selected",pn);
}

static void next_page (RE_resource_entry * re, UWORD id)
{
    ZPL_WINDOW *fw = re->root->window;
    TEXT_PAGE      *page          = TEXT_PAGE_PTR(fw->sub_window);

if (page->page_number > 998)
{
set_status(fw,"Already at end",0);
return;
}

ask_to_save(fw);

page->page_number ++;

if(!do_load_page (fw,page->page_number))
set_status(fw,"Page %d created",page->page_number);

}

static void prev_page (RE_resource_entry * re, UWORD id)
{
    ZPL_WINDOW *fw = re->root->window;
    TEXT_PAGE      *page          = TEXT_PAGE_PTR(fw->sub_window);

if (page->page_number < 101)
{
set_status(fw,"Already at beginning",0);
return;
}

ask_to_save(fw);

page->page_number --;

if(!do_load_page (fw,page->page_number))
set_status(fw,"Page %d created",page->page_number);

}


static void save_page (RE_resource_entry * re, UWORD id)
{
    ZPL_WINDOW *fw = re->root->window;

if (!do_save_page (fw))
       tell_em ("Couldn't save page",0);
}

static void set_dir (RE_resource_entry * re, UWORD id)
{
    GENERAL_PREFS *gp = PR_get_general_prefs ();
    char working_directory[200];

    if (dir_request ("Directory for loading and saving", working_directory))
		    {
			strcpy (gp->default_path, working_directory);
			tell_em ("Directory now %s",gp->default_path);
		    }
		    else
			tell_em ("Directory for loading and saving not changed",0);
}

static void quit (RE_resource_entry * re, UWORD id)
{
    ZPL_WINDOW *fw = re->root->window;
    TEXT_PAGE      *page          = TEXT_PAGE_PTR(fw->sub_window);

if (page->modified && do_it("Save changes to current page before exiting ?",0))
do_save_page(fw);

    terminate (0);
}

/*
 * Open a new edit window.
 */
void UI_edit (UI_ARGS)
{
    RE_resource *re;
    RESOURCE_WINDOW *rw;
    ZPL_WINDOW *re_fw;

    re_fw = open_window (make_edit_window ());

    if (re_fw)
    {

        re_fw->input_y = 1;
        re_fw->input_x = 39;
        re_fw->t_width = 40;
        edit_cursor_move(re_fw);
        do_load_page(re_fw,100);

	re = re_fw->resources;

	rw = (RESOURCE_WINDOW *) re_fw->sub_window;

	RE_set_process (RE_find_resource ("clear_all", re), text_clear, (void *) NULL);
	RE_set_process (RE_find_resource ("exit", re), quit, (void *) NULL);

RE_set_process (RE_find_resource ("save", re), save_page, 0);
RE_set_process (RE_find_resource ("load", re), load_page, 0);
RE_set_process (RE_find_resource ("next", re), next_page, 0);
RE_set_process (RE_find_resource ("prev", re), prev_page, 0);
RE_set_process (RE_find_resource ("goto", re), goto_page, 0);
RE_set_process (RE_find_resource ("dir", re),  set_dir, 0);

    }
}

void UI_edit_init (RE_resource *re)
{
}
