/*---------------------------------------------------------------------------
 *
 *      Title:          NICOLA Memory Allocation tracking and checking
 *
 *      File:           alloc_check.c
 *
 *      Date:           August 1992
 *
 *      Description:
 *
 *      Data sources:
 *
 *      Rev: 1.0
 *      1.0     R.J.Harrison    Implement
 *      1.1     R.J.Harrison    Modifications for speed          February 1993
 *
 *      Copyright  1993 ZARGsoft / The exploding software company
 *---------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "OS_def.h"

#ifdef calloc
#undef calloc
#endif
#ifdef free
#undef free
#endif
#ifdef malloc
#undef malloc
#endif
#ifdef savestr
#undef savestr
#endif

void error_log (char *,...);

#define DO_CHECK
/*
 * #define CREATE_ALLOC_LOG_FILE
 */

static long check_key = 0Xfeaee1e1, check_key_long[] =
{0xaabbccdd, 0x11223344};
int do_scan_check = 0;

typedef struct CHECK
{
    long pre[2];
    void *data;
    long index;
    char *checkword;
}
CHECK;

typedef struct CHECK_SHORT
{
    void *data;
    long line, size;
    char *name;
}
CHECK_SHORT;

typedef enum
{
    C_ADD, C_REMOVE, C_SIZE, C_USAGE, C_USAGE_ALL, C_FIND, C_FIND_I, C_CHECK, C_DUMP, C_REMOVE_I
} CHECK_ACTION;

/*
 * Smarter version of realloc which will work with a null pointer 
 * signifying to create the list...
 */
void *sm_realloc (void *ptr, size_t ne, size_t sz)
{
    if (!ptr)
	return calloc (ne, sz);
    else
	return realloc (ptr, ne * sz);
}

static long check_memory_item (void *data, CHECK_SHORT * cs)
{
if(data)
{
    char *td = data;
    CHECK *ck;

    td = td - sizeof (CHECK);
    ck = (CHECK *) td;

    if (memcmp (ck->checkword, &check_key, sizeof (check_key)))
    {
	error_log ("memory %x, %s/%ld overwrite past end.\n", cs->data, cs->name, cs->line);
	printf ("memory %x, %s/%ld overwrite past end.\n", cs->data, cs->name, cs->line);
	return (1);
    }

    if (memcmp (ck->pre, check_key_long, sizeof (ck->pre)))
    {
	error_log ("memory %x, %s/%ld overwrite at start.\n", cs->data, cs->name, cs->line);
	printf ("memory %x, %s/%ld overwrite at start end.\n", cs->data, cs->name, cs->line);
	return (1);
    }
}
    return (0);
}

static long do_check_list (CHECK_ACTION action, void *ptr, CHECK_SHORT * cs)
{
    static CHECK_SHORT *check_list = 0;
    static long *first_free, list_size = 0, list_max_size = 0, first_free_pos = -1,
      list_inc = 700;

    switch (action)
    {
    case C_ADD:
	{
	    long pos = 0;

	    if (first_free_pos >= 0)
		pos = first_free[first_free_pos--];

	    if (!pos)
	    {
		if (list_size >= list_max_size)
		{
		    list_max_size += list_inc;
		    check_list = sm_realloc (check_list, list_max_size, sizeof (CHECK_SHORT));
		    first_free = sm_realloc (first_free, list_max_size, sizeof (*first_free));
		}
		pos = list_size++;
	    }
	    if (!check_list)
		return (0);

	    check_list[pos] = *cs;
	    check_list[pos].data = ptr;
	    return (pos);

	    break;
	}

    case C_REMOVE:
	ptr = (void *) do_check_list (C_FIND_I, ptr, 0);

    case C_REMOVE_I:
	{
	    long remove_index = (long) ptr;

	    if ((remove_index < 0) || (remove_index > list_size))
		return -1;

	    if (remove_index > -1)
	    {
		first_free[++first_free_pos] = remove_index;
/*
 * you will not use memory which has been freed ...
 */
		memset (check_list[remove_index].data, 0, check_list[remove_index].size);
		memset (&check_list[remove_index], 0, sizeof (CHECK_SHORT));
	    }
	    break;
	}

    case C_FIND_I:
	{
	    long i;

	    for (i = 0; i < list_size; i++)
		if (check_list[i].data == ptr)
		    return i;

	    return -1;
	    break;
	}

    case C_FIND:
	{
	    long i = do_check_list (C_FIND_I, ptr, 0);

	    if (i < 0)
	    {
		memset (cs, 0, sizeof (*cs));
		cs->name = "Unknown";
	    }
	    else
		*cs = check_list[i];

	    break;
	}

    case C_SIZE:
	{
	    return list_size;
	    break;
	}

    case C_USAGE:
	{
	    return (long) (list_size * (sizeof (CHECK_SHORT) + sizeof (long)));
	    break;
	}

    case C_USAGE_ALL:
	{
	    long usage = 0, i;

	    for (i = 0; i < list_size; i++)
		usage += check_list[i].size;

            return (usage);
	    break;
	}

    case C_CHECK:
	{
	    long failed = 0, i;

	    for (i = 0; i < list_size; i++)
	    {
		failed += check_memory_item (check_list[i].data, &check_list[i]);
	    }
	    if (failed)
		    printf ("%ld memory corruptions found\n", failed);

	    return failed;
	    break;
	}

    case C_DUMP:
	{
	    long i;
	    FILE *op = fopen ("memory.usage", "w");

	    if (!op)
		return 1;

	    fprintf (op, "Detailed dynamic memory usage\n");
	    for (i = 0; i < list_size; i++)
	    {
                if(check_list[i].data)
  	     	    fprintf (op, "%-20s :%4ld size %ld\n", 
                             check_list[i].name, check_list[i].line, check_list[i].size);
	    }
	    fclose (op);
	    return 0;
	    break;
	}
    }
}

static CHECK *start;
static long memory_used = 0;

static char *get_checkptr (void *ptr, long sz)
{
    char *p = ptr;
    return (p + sz);
}

/*
 * 'Remember' a given chunk of memory.
 * ****NOTE*** the CHECK portion must have been zero'd already.
 */
static void MY_remeber (CHECK * cptr, char *module, long line, long sz)
{
    if (cptr)
    {
	void *ptr = (char *) cptr + sizeof (CHECK);
	char *cp = get_checkptr (ptr, sz);
	CHECK *ck = cptr;
	CHECK_SHORT cs;

	memcpy (cp, &check_key, sizeof (check_key));

	if (ck)
	{
#ifdef CREATE_ALLOC_LOG_FILE
	    static FILE *mem_log = 0;
	    if (!mem_log)
		mem_log = fopen ("t:nic_mem.log", "w");
	    if (mem_log)
		fprintf (mem_log, "%8x: %8x %-12s %4d\n", ptr, ck, module, line);
#endif

	    memcpy (ck->pre, &check_key_long, sizeof (ck->pre));
	    cs.name = module;
	    cs.line = line;
	    cs.size = sz;

	    ck->data = ptr;
	    ck->checkword = cp;
	    ck->index = do_check_list (C_ADD, ptr, &cs);

#ifdef KEEP_LIST
	    if (!start)
		start = ck;
	    else
	    {
		ck->next = start;
		start->prev = ck;
		start = ck;
	    }
#endif
	}
    }
}


static long MY_remove (void *data, char *module, long line)
{
    CHECK *ck;
    char *td = data;

    td = td - sizeof (CHECK);
    ck = (CHECK *) td;

    if (!data || !ck->data)
	return (0);

    if (ck->data == data)
    {
	if (memcmp (ck->checkword, &check_key, sizeof (check_key)))
	{
	    CHECK_SHORT cs;
	    do_check_list (C_FIND, ck->data, &cs);

	    error_log ("Memory check %s (line %ld) failed\noverwrite with %x",
		       cs.name, cs.line, *ck->checkword);
	}

	if (!do_check_list (C_REMOVE_I, (void *) ck->index, 0))
	    return (0);

	ck->data = 0;
	free (ck);
	return (1);
    }
    else
    {
	CHECK_SHORT cs;
	do_check_list (C_FIND, ck->data, &cs);

	error_log ("Memory list corrupted %s (line %ld)",
		   cs.name, cs.line, *ck->checkword);
    }
    return (0);
}

static char *oph (char *ptr, char *opbuf, long sz)
{
    static char c[10];
    *opbuf = 0;
    while (sz--)
    {
	sprintf (c, "%02x", *ptr++);
	strcat (opbuf, c);
    }
    return (opbuf);
}


void MY_ov_check_memory (void)
{
    do_check_list (C_CHECK, 0, 0);
}

void MY_scan_check_memory (void)
{
    static int t = 0;
    if (!do_scan_check)
	return;
    if (!t || t <= time (0))
	do_check_list (C_CHECK, 0, 0);
    t = time (0) + 5;
}


long MY_get_dynamic_memory_usage (void)
{
#ifdef AMIGA
    extern long bss_size;
#else
    long bss_size = 62122;
#endif

#ifdef DO_CHECK
    long sz = do_check_list (C_USAGE, 0, 0);

    return do_check_list (C_USAGE_ALL, 0, 0) + bss_size;
#else
    return (memory_used + bss_size);
#endif
}

void MY_dump_memory (void)
{
    if (do_check_list (C_DUMP, 0, 0))
	error_log ("Unable to open memory dump file");
}

/*
 * Alloc sanity checking routines
 */
void *MY_malloc (TY_unsigned sz, char *module, long line)
{
#ifdef DO_CHECK
    void *ptr, *aptr;
    if (sz && sz > 0)
    {
	long size = sizeof (CHECK) + sz + sizeof (long);
	ptr = malloc (size);
	if (ptr)
	{
	    if (ptr)
		memset (ptr, 0, size);
	    aptr = ptr;
	    ptr = (char *) ptr + sizeof (CHECK);
	    MY_remeber (aptr, module, line, sz);
	    return (ptr);
	}
    }
    return (NULL);
#else
    return (malloc (sz));
#endif
}


void *MY_calloc (TY_unsigned ne, TY_unsigned sz, char *module, long line)
{
#ifdef DO_CHECK
    void *ptr = MY_malloc (ne * sz, module, line);
    return (ptr);
#else
      return (calloc (ne, sz));
#endif
}

void *MY_savestr (char *p, char *module, long line)
{
    char *c = MY_malloc (strlen (p) + 3, module, line);
    strcpy (c, p);
    return (c);
}

void MY_free (void *a, char *module, long line)
{
#ifdef DO_CHECK
    if (!MY_remove (a, module, line))
	error_log ("free(): Module %s (line %ld) failed\nattempt to free non-alloc memory %8x", module, line, a);
#else
    free (a);
#endif
}
