/*---------------------------------------------------------------------------
 *
 *      Title:          NICOLA main event/window control
 *
 *      File:           control.c
 *
 *      Date:           14 October 1991
 *
 *      Description:
 *              This module is responsible for controlling things to do with the
 *      screens/windows/events.
 *
 *
 *      Data sources:
 *
 *      Rev: 1.0
 *      1.0     R.J.Harrison    Release                              1 Sept 92
 *
 *      Copyright  1990 ZARGsoft / The exploding software company
 *
 *  $Log$
 *
 *-------------------------------------------------------------------
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#ifdef AMIGA
#include <exec/types.h>
#include <exec/nodes.h>
#include <exec/lists.h>
#include <exec/io.h>
#include <exec/memory.h>

#include <intuition/intuition.h>
#include <intuition/intuitionbase.h>

#include <dos/dos.h>
#include <dos/dosextens.h>
#include <libraries/reqtools.h>
#include <libraries/dos.h>

#include <graphics/text.h>

#include <proto/exec.h>
#include <proto/graphics.h>
#include <proto/gadtools.h>
#include <proto/intuition.h>
#endif

#include "zpl.h"

#ifdef AMIGA
#include "intuisup.h"
#endif


#define PUBLIC_SCREEN_NAME get_public_screen_name()

#define GADGET_IDCMP (IDCMP_MOUSEBUTTONS | IDCMP_MOUSEMOVE | IDCMP_GADGETDOWN | IDCMP_GADGETUP)

#define RESOURCE_DATABASE resource_database
#define PREFERENCE_DATABASE preference_database

#define MAX_SAFE_CLOSE 10
static ZPL_WINDOW *safe_close_windows[MAX_SAFE_CLOSE];
static TY_int safe_close_ptr = 0, opened_on_public = 0;

static ZPL_WINDOW *first_window = NULL;

struct Screen *find_workbench_screen (void);
void close_all_windows (TY_int do_safely);

TY_int event ();
static int window_font_changed = FALSE;
static TY_int checking_for_events = TRUE;
static struct Screen *custom_screen = NULL;
static char *screen_title = NULL;
#define SCREEN_TITLE_SIZE 120

#ifdef AMIGA
/*
 * The workbench compatible pens we'll use for that New Look.
 */
static UWORD dri_Pens[] = {0, 1, 1, 2, 1, 3, 1, 0, 3, (UWORD)~0};
static struct Process *proc = NULL;
static APTR old_pr_win_ptr = NULL;

/*
 * The following two lines are used to translate the
 * viewport into the actual screen coordinates. However
 * there is probably a more efficient way of doing this using
 * intuition that I haven't discovered yet.
 */

struct Library *GadToolsBase = NULL;
struct Library *IntuiSupBase = NULL;
struct Library *DiskfontBase = NULL;

static struct TextFont *window_font;

//struct ReqToolsLib *ReqToolsBase = NULL;
struct Library *ReqToolsBase = NULL;
struct ConsoleDevice *ConsoleDevice = NULL;

struct IOStdReq console_ioreq;
struct Task *task;

static ULONG wait_signal_mask = 0;

TY_int os_version_id;		/*
				 * 2.04 (ish) loaded flag
				 */
TY_int ignore_v36;

/************************ Window Defines ***********************************/
struct NewScreen new_screen =
{
    0, 0,			/*
				 * start position
				 */
#ifdef USE_LACE
    640, 512, 4,		/*
				 * width, height, depth
				 */
    0, 1,			/*
				 * detail pen, block pen
				 */
    HIRES | LACE,		/*
				 * HIRES, INTERLACE, SPRITES, DUAL, HAM
				 */
#else
    640, 256, 3,		/*
				 * width, height, depth
				 */
    0, 1,			/*
				 * detail pen, block pen
				 */
    NULL,			/*
				 * HIRES, INTERLACE, SPRITES, DUAL, HAM
				 */
#endif
    CUSTOMSCREEN,		/*
				 * screen type
				 */
    NULL,			/*
				 * font to use
				 */
    NULL,			/*
				 * default title for screen
				 */
    NULL,			/*
				 * pointer to extra gadgets
				 */
    NULL			/*
				 * No CustomBitMap
				 */
};

/**
 *
 * initialize output window.
 *
 **/
static struct IntuiText abort_text[] =
{
    {1, 2, JAM2, 20, 5, NULL, "Aborted", NULL},
    {0, 1, JAM2, 5, 4, NULL, "OK", NULL},
    {0, 1, JAM2, 6, 4, NULL, "OK", NULL}
};

long interactive_mode = TRUE;

void set_interactive_mode(long turned_on)
{
interactive_mode = turned_on;
}

long is_interactive(void)
{
return interactive_mode;
}

void fatal_error (char *str)
{
    char buf[120];
    sprintf (buf, "Fatal Error: %s", str);
    abort_text[0].IText = buf;
    AutoRequest (NULL, &abort_text[0], &abort_text[1], &abort_text[2], NULL, 0, 500, 50);
    exit (0);
}

TY_int init_window ()
{
    extern TY_int _stack_size;

    GfxBase = (struct GfxBase *) OpenLibrary ("graphics.library", 0);
    if (GfxBase == NULL)
	return (FALSE);

    IntuitionBase = (struct IntuitionBase *) OpenLibrary ("intuition.library", 0);
    if (IntuitionBase == NULL)
    {
	CloseLibrary ((struct Library *) GfxBase);
	return (FALSE);
    }

    if (!(ReqToolsBase = OpenLibrary (REQTOOLSNAME, REQTOOLSVERSION)))
    {
	CloseLibrary ((struct Library *) GfxBase);
	CloseLibrary ((struct Library *) IntuitionBase);
	fatal_error ("cannot open reqtools.library");
	return (FALSE);
    }

    if (!(DiskfontBase = OpenLibrary ("diskfont.library", 33L)))
    {
	CloseLibrary ((struct Library *) GfxBase);
	CloseLibrary ((struct Library *) IntuitionBase);
	CloseLibrary ((struct Library *) ReqToolsBase);
	fatal_error ("cannot open diskfont.library");
	return (FALSE);
    }

    if (ignore_v36 || !(GadToolsBase = OpenLibrary ("gadtools.library", 36L)))
    {
	os_version_id = 10;

	if (!(IntuiSupBase = OpenLibrary (IntuiSupName, IntuiSupVersion)))
	{
	    tell_em ("Either WB2.04+ or intuisup.library is required",0);
	    return (FALSE);
	}
    }
    else
	os_version_id = 36;

    /*
     * must have the console.device opened to use RawKeyConvert()
     */

    if (OpenDevice ("console.device", -1L, (struct IORequest *) &console_ioreq, 0L))
    {
	CloseLibrary ((struct Library *) GfxBase);
	CloseLibrary ((struct Library *) IntuitionBase);
	CloseLibrary ((struct Library *) ReqToolsBase);
	CloseLibrary (DiskfontBase);
	tell_em ("cannot open console.device",0);
	return (FALSE);
    }
    ConsoleDevice = (struct ConsoleDevice *) console_ioreq.io_Device;


    if (_stack_size < 15000)
    {
	tell_em ("Minimum stack is 15k, current stack %sk\nPlease increase this and restart", op_ltoa (_stack_size / 1024));
	terminate (0);
    }

    task = FindTask (0);
/*
 * Open the resource database..This should really look harder...maybe later~
 */
    if (!RE_init_resource (RESOURCE_DATABASE))
    {
	tell_em ("Cannot open resource database ", RESOURCE_DATABASE);
	return (FALSE);
    }
    PR_open (PREFERENCE_DATABASE);
EX_init();
    return (TRUE);
}

TY_int using_v36(void)
{
    return (os_version_id > 34);
}

TY_int using_v38(void)
{
     return (GadToolsBase->lib_Version > 37);
}

TY_int get_gadtools_version (void)
{
     return (GadToolsBase->lib_Version);
}

struct Screen *get_custom_screen ()
{
    return (custom_screen);
}

TY_int get_intensity (TY_int col_word)
{
    TY_int r, g, b;
    r = (col_word & 0xf00) >> 8;
    g = (col_word & 0xf0) >> 4;
    b = (col_word & 0xf);
    return (r + g + b) / 3;
}

TY_int brightest_colour ()
{
    static TY_int br_colour = -1;
    TY_int i, num_colours;
    if (br_colour < 0)
    {
	struct Screen *scr = get_custom_screen ();

	if (!scr)
	    scr = find_workbench_screen ();

	if (scr)
	{
	    TY_int f, f1 = 0, fi = 1;
	    num_colours = 1 << scr->BitMap.Depth;
	    for (i = 0; i < num_colours; i++)
	    {
		f = get_intensity (GetRGB4 (scr->ViewPort.ColorMap, i));
		if (f > f1)
		{
		    f1 = f;
		    fi = i;
		}
	    }
	    br_colour = fi;
	}
	else
	    br_colour = 0;
    }
    return (br_colour);
}

TY_int darkest_colour ()
{
    static TY_int br_colour = -1;
    TY_int i, num_colours;
    if (br_colour < 0)
    {
	struct Screen *scr = get_custom_screen ();

	if (!scr)
	    scr = find_workbench_screen ();

	if (scr)
	{
	    TY_int f, f1 = 0xfffffff, fi = 0;
	    num_colours = 1 << scr->BitMap.Depth;
	    for (i = 0; i < num_colours; i++)
	    {
		f = get_intensity (GetRGB4 (scr->ViewPort.ColorMap, i));
		if (f < f1)
		{
		    f1 = f;
		    fi = i;
		}
	    }
	    br_colour = fi;
	}
	else
	    br_colour = 1;
    }
    return (br_colour);
}

TY_int menu_background_not_black ()
{
    struct Screen *scr = get_custom_screen ();

    if (!scr)
	scr = find_workbench_screen ();

    if (scr)
	if (get_intensity (GetRGB4 (scr->ViewPort.ColorMap, 1)) > 8)
	    return (TRUE);
    return (0);
}

void save_palette ()
{
    TY_int i, num_colours;
    struct Screen *scr = get_custom_screen ();

    if (scr)
    {
	TY_int *carray;
	num_colours = 1 << scr->BitMap.Depth;
	carray = calloc (num_colours, sizeof (TY_int));
	if (carray)
	{
	    for (i = 0; i < num_colours; i++)
	    {
		carray[i] = GetRGB4 (scr->ViewPort.ColorMap, i);
	    }
	    PR_add_item (PR_get_prefs (), "Colour.Palette", carray, sizeof (TY_int) * num_colours);
	    free (carray);
	}
    }
}

void load_palette ()
{
    TY_int i, num_colours;
    struct Screen *scr = get_custom_screen ();

    if (scr)
    {
	TY_int *carray;
	PREFERENCE_ENTRY *pe = PR_get_item (PR_get_prefs (), "Colour.Palette");
	if (pe)
	{
	    carray = pe->data;
	    if (carray)
	    {
		num_colours = min (pe->size / sizeof (TY_int), 1 << scr->BitMap.Depth);
		for (i = 0; i < num_colours; i++)
		{
		    TY_int r, g, b;
		    r = (carray[i] & 0xf00) >> 8;
		    g = (carray[i] & 0xf0) >> 4;
		    b = (carray[i] & 0xf);
		    SetRGB4 (&scr->ViewPort, i, r, g, b);
		}
	    }
	}
    }
}

/*
 * This finds the workbench screen.
 */
struct Screen *find_workbench_screen (void)
{
    static struct Screen wb_screen, *scr;
   if (using_v36 ())
    {
	scr = LockPubScreen (NULL);
	UnlockPubScreen (NULL, scr);
	return (scr);
    }
    else
    {

	GetScreenData ((BYTE *) & wb_screen, sizeof (wb_screen), WBENCHSCREEN, NULL);
	return (&wb_screen);
    }
}


struct Screen *get_screen ()
{
    struct Screen *scr = custom_screen;
    if (!scr)
	return (find_workbench_screen ());
    else
	return (scr);
}

short get_screen_width ()
{
    struct Screen *scr = get_screen ();
    return (scr->Width);
}

short get_screen_height ()
{
    struct Screen *scr = get_screen ();
    return (scr->Height);
}

UBYTE get_screen_depth ()
{
    struct Screen *scr = get_screen ();
    return (scr->BitMap.Depth);
}
#endif

void set_window_font_changed()
{
    window_font_changed = TRUE;
}

void enable_event_checking ()
{
    checking_for_events = TRUE;
}

void disable_event_checking ()
{
    checking_for_events = FALSE;
}

TY_int events_enabled ()
{
    return (checking_for_events);
}

void do_global_action (ACTION action, TY_int param, TY_int id)
{
    ZPL_WINDOW *fw = first_window;
    while (fw)
    {
	(*fw->sub_window->hdr.process) (fw->sub_window, action, param, id);
	fw = fw->next;
    }
}

void *do_all_windows (void *(*function) (ZPL_WINDOW *, void *), void *param)
{
    ZPL_WINDOW *fw = first_window;
    void *result_code = 0;	/* Non-zero indicates function failed */

    while (fw && !result_code)
    {
	result_code = (*function) (fw, param);
	fw = fw->next;
    }
    return (result_code);
}

void add_window_to_list(ZPL_WINDOW *fw)
{
    if (!first_window)
	first_window = fw;
    else
    {
	ZPL_WINDOW *zfw = first_window, *lfw = NULL;
	while (zfw)
	{
	    lfw = zfw;
	    zfw = zfw->next;
	}
	if (lfw)
	    lfw->next = fw;
    }
    fw->next = NULL;
}

void window_finish (ZPL_WINDOW * fw)
{
    (*fw->sub_window->hdr.process) (fw->sub_window, WM_WINDOW_FINISH, 0, 0);
    FW_close (fw);
}

void remove_window_from_list (ZPL_WINDOW * fw_to_remove)
{
    ZPL_WINDOW *fw = first_window, *lfw = NULL, *tfw;

    while (fw)
    {
	if (fw == fw_to_remove)
	{
	    tfw = fw->next;
	    if (lfw)
		lfw->next = tfw;
	    else
		first_window = tfw;

/*
 * As a fudge.... only free if this window isn't a 'main' window, otherwise
 * if the memory gets overwritten then we have all sorts of hassles....
 */
	    if (!(fw->sub_window->hdr.type & LAYOUTABLE))
{
	free (fw->sub_window);
	free (fw);
}
	    fw = NULL;
	}
	else
	{
	    lfw = fw;
	    fw = fw->next;
	}
    }
}

void lock_window (ZPL_WINDOW * fw)
{
    fw->busy_count ++;
    SET_BIT (fw->type, FW_PROCESSING);
    SET_BIT(fw->type, FW_DISALLOW_CLOSE);
    FW_set_busy_pointer (fw);
}

void unlock_window (ZPL_WINDOW * fw)
{
    fw->busy_count--;
    if(fw->busy_count <= 0)
    {
        STRIP_BIT (fw->type, FW_PROCESSING);
        STRIP_BIT(fw->type, FW_DISALLOW_CLOSE);
        FW_clear_busy_pointer (fw);
        fw->busy_count = 0;
        dump_events(fw);
    }
}

TY_int window_locked (ZPL_WINDOW * fw)
{
    if(fw)
        return BIT_SET (fw->type, FW_PROCESSING);
    else
        return TRUE;
}

void set_close_allow (ZPL_WINDOW * fw, TY_int allow)
{
if(allow)
STRIP_BIT(fw->type, FW_DISALLOW_CLOSE);
else
SET_BIT(fw->type, FW_DISALLOW_CLOSE);
}

void close_window (ZPL_WINDOW * fw)
{
/*
 * Firstly close any windows attached to this...
 */
    ZPL_WINDOW *sub_fw = first_window;

    if(!fw)
        return;

    if (window_locked (fw))
    {
	FW_clear_busy_pointer (fw);
	return;
    }
    if(BIT_SET(fw->type, FW_DISALLOW_CLOSE))
        return;

    if (fw == first_window && (get_number_of_windows() < 2))
    {
	if (PR_ask_before_quit ())
	{
	    FW_clear_busy_pointer (fw);
	    return;
	}

close_all_windows(FALSE);
	terminate (0);
    }
else
{
    while (sub_fw)
    {
	if (sub_fw->parent == fw)
	{
/*
 * Sever the link before closing...
 */
	    sub_fw->parent = NULL;
	    close_window (sub_fw);
	}
	else
	{
	    sub_fw = sub_fw->next;
	}
    }

    window_finish (fw);		/*
				 * This does a FW_close(fw)
				 */

    FW_free_sb (fw);		/*
				 * Lose the scrolback
				 */
    RE_dispose (fw->resources);
    FW_free_all (fw);

    remove_window_from_list (fw);
#ifdef AMIGA
if(proc)
proc->pr_WindowPtr = first_window->w;
#endif
}
}

static void do_close_safely (void)
{
    TY_int i;
    static TY_int in_progress = FALSE;

    if (!in_progress)
    {
	in_progress = TRUE;
	for (i = 0; i < safe_close_ptr; i++)
	    close_window (safe_close_windows[i]);
	safe_close_ptr = 0;
	memset (safe_close_windows, 0, sizeof (safe_close_windows));
	in_progress = FALSE;
    }
}

void FW_close_safely (ZPL_WINDOW * fw)
{
    TY_int i;

    if (window_locked (fw))
	return;

    for (i = 0; i < safe_close_ptr; i++)
	if (safe_close_windows[i] == fw)
	    return;

    FW_set_busy_pointer (fw);
    if (safe_close_ptr < MAX_SAFE_CLOSE)
	safe_close_windows[safe_close_ptr++] = fw;
}

void prefs_make_all_default (void)
{
    do_global_action (WM_MAKE_WINDOW_PREFS, -1, -1);
}

void send_content_changed (ZPL_WINDOW * fw)
{
    do_global_action (WM_CONTENT_CHANGED, (TY_int) fw, -1);
}

void send_content_changed_param (ZPL_WINDOW * fw, TY_int param)
{
    do_global_action (WM_CONTENT_CHANGED, (TY_int) fw, param);
}

ZPL_WINDOW *find_window (MD_WINDOW_P w)
{
    ZPL_WINDOW *zfw = first_window;
    while (zfw)
    {
	if (w == zfw->w)
	    return (zfw);
	zfw = zfw->next;
    }
    return (0);
}

ZPL_WINDOW *find_window_type (TY_int type)
{
    ZPL_WINDOW *zfw = first_window;
    while (zfw)
    {
	if ((*zfw->sub_window->hdr.process) (zfw->sub_window, WM_GET_WINDOW_TYPE, 0, 0) & type)
	    return (zfw);
	zfw = zfw->next;
    }
    return (0);
}

static MD_WINDOW_P open_sized_window (struct Screen *scr, struct NewWindow *new_win)
{
    struct NewWindow nw;

#ifdef AMIGA
    if (!scr)
	scr = find_workbench_screen ();
#endif

    if (new_win)
    {
	nw = *new_win;
#ifdef AMIGA
	/*
	 * Scale new window dimension
	 */
	if (nw.Width > scr->Width)
	{
	    nw.LeftEdge = 0;
	    nw.Width = scr->Width;
	}
	else
	{
	    if ((nw.LeftEdge + nw.Width) > scr->Width)
	    {
		nw.LeftEdge = scr->Width - nw.Width;
	    }
	}
	if (nw.Height > scr->Height)
	{
	    nw.TopEdge = 0;
	    nw.Height = scr->Height;
	}
	else
	{
	    if ((nw.TopEdge + nw.Height) > scr->Height)
	    {
		nw.TopEdge = scr->Height - nw.Height;
	    }
	}
	if (using_v36 ())
        {
            static WORD zoom_defaults[] = {0,0,200,20};
	    return (OpenWindowTags (&nw,
				    WA_InnerHeight, nw.Height,
				    WA_AutoAdjust, TRUE,
                                    WA_MinWidth,200, WA_MinHeight,50,  WA_MaxWidth,~0, WA_MaxHeight,~0,
                                    WA_Zoom, zoom_defaults,
				    WFLG_NEWLOOKMENUS, TRUE,
				    TAG_DONE));
        }
	else
        {
            struct Window *w=OpenWindow (&nw);
            if(w)
                WindowLimits(w,200,50,~0,~0);
            return (w);
        }
#endif
#ifdef WINDOWS
	return (OpenWindow (&nw));
#endif
#ifdef VMS
	return (OpenWindow (&nw));
#endif
    }
    return (NULL);
}
#ifdef NEED_FLOG
/*

 * Write message to logfile
 */
void flog(char *msg)
{
static FILE *f=NULL;

#include <sys/timeb.h>

if(!f)
f=fopen("/nic.log","a");
if(f)
{
struct timeb t;
char *tm;

if(msg)
{
   ftime(&t);
tm=ctime(&t.time);
tm[19]=0;
   fprintf(f,"%s.%03d: %s\n", tm, t.millitm,msg);
}
else
{
fprintf(f,"\n");
fclose(f);
f=0;
}
}
}
#endif
/*
 * This function adds a console device to a window. If the window already has a console
 * then nothing will happen, otherwise it will get one.
 */
TY_int add_console_to_window (ZPL_WINDOW * fw)
{
#ifdef AMIGA
    BYTE error;

    if (!(fw->type & FW_TYPE_IO))
    {
	/*
	 * Create reply port and io block for writing to console
	 */
	if (!(fw->write_port = CreatePort ("ZPL_write", 0)))
	    return (FALSE);

	if (!(fw->write_req = (struct IOStdReq *)
	      CreateExtIO (fw->write_port, (LONG) sizeof (struct IOStdReq))))
	      return (FALSE);

	/*
	 * Create reply port and io block for reading from console
	 */
	if (!(fw->read_port = CreatePort ("ZPL_read", 0)))
	    return (FALSE);

	if (!(fw->read_req = (struct IOStdReq *)
	      CreateExtIO (fw->read_port, (LONG) sizeof (struct IOStdReq))))
	      return (FALSE);

/*
 * Attach console device to an open Intuition window.
 * This function returns a value of 0 if the console
 * device opened correctly and a nonzero value (the error
 * returned from OpenDevice) if there was an error.
 */
	fw->write_req->io_Data = (APTR) fw->w;
	fw->write_req->io_Length = sizeof (*fw->w);
	error = OpenDevice ("console.device", 0, (struct IORequest *) fw->write_req, 0);
	fw->read_req->io_Device = fw->write_req->io_Device;	/*
								 * clone required parts
								 */
	fw->read_req->io_Unit = fw->write_req->io_Unit;

	if (error)
	    return (FALSE);

	fw->type |= FW_TYPE_IO;
    }
#endif
#ifdef WINDOWS
#ifdef WINDOWS
#ifdef  WIN_BUF
{
HDC hdc;
int xs,ys;
xs  = GetSystemMetrics(SM_CXSCREEN);
ys  = GetSystemMetrics(SM_CYSCREEN);
hdc=GetDC(fw->w);
fw->dc = CreateCompatibleDC(hdc);
fw->bm = CreateCompatibleBitmap(hdc,xs,ys);
SelectObject(fw->dc,fw->bm);
/* SelectObject(fw->dc,GetStockObject(LTGRAY_BRUSH)); */
SelectObject(fw->dc,(COLOR_WINDOW + 1));
PatBlt(fw->dc,0,0,xs,ys,PATCOPY);
ReleaseDC(fw->w,hdc);
}
#endif
#endif
    fw->type |= FW_TYPE_IO;
#endif
}

struct TextAttr *get_window_font ()
{
#ifdef AMIGA
    struct Screen *scr = get_screen ();
    return (scr->Font);
#else
      return (0);
#endif
}


struct TextAttr *init_font (struct TextAttr *textattr, struct TextFont **op_f)
{
#ifdef AMIGA
    struct TextFont *font;

    if ((font = (struct TextFont *) OpenFont (textattr)))
    {
	SCset (font->tf_XSize, font->tf_YSize);
	SC_reset_offsets;
if(op_f)
*op_f = font;
else
	CloseFont (font);
    }
    else
    {
	if (!font && (font = (struct TextFont *) OpenDiskFont (textattr)))
	{
	    SCset (font->tf_XSize, font->tf_YSize);
	    SC_reset_offsets;
if(op_f)
*op_f = font;
else
	    CloseFont (font);
	}
	else
	    tell_em ("Failed to open font %s",textattr->ta_Name);
    }
    if (!font)
    {
	textattr = get_window_font ();

	if ((font = (struct TextFont *) OpenFont (textattr)))
	{
	    SCset (font->tf_XSize, font->tf_YSize);
	    SC_reset_offsets;
if(op_f)
*op_f = font;
else
	    CloseFont (font);
	}
    }
#endif
    return textattr;
}

void set_window_font(ZPL_WINDOW *fw)
{
#ifdef AMIGA

if (window_font_changed)
{
if(window_font)
CloseFont(window_font);
window_font = 0;
}

if(!window_font)
{
    static struct TextAttr *ffa;
    ffa = PR_get_win_font ();

    if (ffa && ffa->ta_Name && *ffa->ta_Name)
    {
init_font (ffa, &window_font);
/*	fw->w->IFont = window_font;*/
SetFont(fw->w->RPort, window_font);
    }
}
#endif
}
ZPL_WINDOW *open_window_at (GENERAL_WINDOW * sw, TY_int x, TY_int y, TY_int xs, TY_int ys)
{
    ZPL_WINDOW *fw = (ZPL_WINDOW *) malloc (sizeof (ZPL_WINDOW));
    SCREEN_PREFS *sp = PR_get_screen_prefs ();
    struct NewWindow nw;
    struct NewWindow *pref_nw;
    static TY_int first_time = TRUE;
    TY_int redirect_custom;
    static TY_int current_id = 1;

    if (!fw || !sw)
	return (NULL);
    memset (fw, 0, sizeof (*fw));

    sw->hdr.win = fw;

    if (pref_nw = (struct NewWindow *) (*sw->hdr.process) (sw, WM_GET_WINDOW_PREFS, 0, 0))
	nw = *pref_nw;

    if (x != -1)
	nw.LeftEdge = x;
    if (y != -1)
	nw.TopEdge = y;
    if (xs != -1)
	nw.Width = xs;
    if (ys != -1)
	nw.Height = ys;

#ifdef AMIGA
    redirect_custom = FALSE;

    if (sp->use_custom_screen && first_time)
    {
	    if (!custom_screen)
	    {
screen_title = AllocMem(SCREEN_TITLE_SIZE,MEMF_CLEAR);
if(!screen_title)
{
tell_em("Failed to get memory for screen title",0);
return 0;
}

		sprintf (screen_title, get_screen_title(),  get_version ());

		if (sp->wb_version >= 36)
		{
		    if (sp->use_custom_screen == 2 && *sp->public_screen_to_use)
		    {
			custom_screen = LockPubScreen (sp->public_screen_to_use);
			if (custom_screen)
			{
			    opened_on_public = TRUE;
			    UnlockPubScreen (NULL, custom_screen);
			}
		    }

		    if (!custom_screen)
		    {
			custom_screen = LockPubScreen (PUBLIC_SCREEN_NAME);
			if (custom_screen)
			{
			    UnlockPubScreen (NULL, custom_screen);
			}
			else
			{
			    custom_screen = OpenScreenTags (NULL,
					       SA_DisplayID, sp->display_id,
							    SA_Title, screen_title,
							SA_Depth, sp->depth,
							    SA_SysFont, 1,
							SA_Width, sp->width,
						      SA_Height, sp->height,
							  SA_Pens, dri_Pens,
						    SA_Overscan, OSCAN_TEXT,
							SA_AutoScroll, TRUE,
						       SA_Interleaved, TRUE,
				     SA_PubName, (ULONG) PUBLIC_SCREEN_NAME,
							    TAG_END);
			    PubScreenStatus (custom_screen, 0);
			}
		    }
		}
		else
		{
		    if (sp->lace)
		    {
			new_screen.ViewModes |= LACE;
			new_screen.Height = min (sp->height, 512);
		    }
		    else
			new_screen.Height = min (sp->height, 256);
		    if (sp->hires)
		    {
			new_screen.ViewModes |= HIRES;
			new_screen.Width = min (sp->width, 640);
		    }
		    else
			new_screen.Width = min (sp->width, 320);
		    new_screen.Depth = max (sp->depth + 1, 1);
		    new_screen.DefaultTitle = screen_title;
		    custom_screen = OpenScreen (&new_screen);
		}

		if (!custom_screen)
		{
		    SCREEN_PREFS *sp = PR_get_screen_prefs ();
		    sp->use_custom_screen = FALSE;
		    tell_em ("Cannot Open Specifed Custom Screen, Will use workbench", NULL);
		}
		else
		{
		    load_palette ();
		    redirect_custom = TRUE;
/*
 * open_window(-2);
 */
		}
	    }
    }

    if (custom_screen)
    {
	nw.Type = CUSTOMSCREEN;
	nw.Screen = custom_screen;
    }
    else
    {
	nw.Type = WBENCHSCREEN;
    }
#endif

#ifdef A
    if (use_custom == -2)
    {
	nw.Flags |= BORDERLESS | BACKDROP;
    }
#endif

    if (!(fw->w = open_sized_window (custom_screen, &nw)))
    {
	char buf[200];

	sprintf (buf, "Unable to open window %s, (%d,%d) (%d,%d)",
		 nw.Title, nw.LeftEdge, nw.TopEdge, nw.Width, nw.Height);
/*
 * If the window is the error window, or when the user wants it
 * we will try to make it fit on the screen.
 */
	if (sw->hdr.type == ERROR_WINDOW_ID ||
	    do_it ("Attempt to make window %s fit on screen ?", nw.Title))
	{

#ifdef AAA_OLD_SIZER
	    struct Screen *scr;
	    if (nw.Type == CUSTOMSCREEN)
		scr = custom_screen;
	    else
		scr = find_workbench_screen ();

	    /*
	     * Tweak the size to fit (hopefully...)
	     */

	    if ((nw.LeftEdge + nw.Width) >= scr->Width)
	    {
		nw.Width -= ((nw.LeftEdge + nw.Width) - scr->Width) + 10;
		printf ("Width changed to %d\n", nw.Width);
	    }

	    if ((nw.TopEdge + nw.Height) >= scr->Height)
	    {
		nw.Height -= ((nw.TopEdge + nw.Height) - scr->Height) + 10;
		printf ("Height changed to %d\n", nw.Height);
	    }

	    if (!(fw->w = OpenWindow (&nw)))
	    {

		/*
		 * Ok, play really safe...
		 */
		nw.LeftEdge = nw.TopEdge = 10;
		nw.Width = 320;
		nw.Height = 140;
		printf ("Still can't open, will use the smallest size possible...\n");
		if (!(fw->w = OpenWindow (&nw)))
		{
		    do_it ("Still can't open the window, giving up", NULL);
		    return (FALSE);
		}

	    }
#else
/*
 * TY_int             ox = nw.Width, oy = nw.Height;
 */

	    /*
	     * Open a window which *will* fit, and then resize it.
	     */
	    nw.LeftEdge = nw.TopEdge = 10;
	    nw.Width = 320;
	    nw.Height = 50;

	    if (!(fw->w = OpenWindow (&nw)))
	    {
		do_it ("Still can't open the window, giving up", NULL);
		return (FALSE);
	    }
#endif
	    if (!fw->w)
	    {

		/*
		 * Only log the error message to the error window, when the
		 * window trying to be opened isn't the error window,
		 * otherwise the code becomes a tad recursive, and very
		 * dull...
		 */
		if (sw->hdr.type != ERROR_WINDOW_ID)
		    error_log (buf);
		else
		    tell_em (buf, NULL);

		return (FALSE);
	    }
	}
	else
	    return (FALSE);
    }

    first_time = FALSE;
    FW_set_busy_pointer (fw);

#ifdef AMIGA
    if (redirect_custom)
    {
	proc = (struct Process *) FindTask (NULL);
	old_pr_win_ptr = proc->pr_WindowPtr;
	proc->pr_WindowPtr = fw->w;
	redirect_custom = FALSE;
    }
    fw->rp = fw->w->RPort;	/*
				 * Get the raster port pointer
				 */
    fw->vp = &fw->w->WScreen->ViewPort;		/*
						 * Get the view port pointer
						 */
#ifdef AMIGA
set_window_font(fw);
#endif
    fw->current_font = fw->w->WScreen->Font;

    (*sw->hdr.process) (sw, WM_ADD_GADGETS, 0, 0);

    SetAPen (fw->rp, 3);	/*
				 * Set foreground pen to white
				 */
    SetDrMd (fw->rp, JAM1);	/*
				 * Draw with foreground pen
				 */
#endif

    fw->sub_window = sw;
    fw->window_id = current_id++;

    if (((*sw->hdr.process) (sw, WM_GET_WINDOW_TYPE, 0, 0) & FW_TYPE_OUTPUT) ||
	((*sw->hdr.process) (sw, WM_GET_WINDOW_TYPE, 0, 0) & FW_TYPE_IO))
    {
	add_console_to_window (fw);
	if ((*sw->hdr.process) (sw, WM_GET_WINDOW_TYPE, 0, 0) & FW_TYPE_OUTPUT)
	    fw->type |= FW_TYPE_OUTPUT;
    }
#ifdef AMIGA
    if (!(fw->w->IDCMPFlags & RAWKEY))
	fw->more_disabled = TRUE;

    if (fw->w->UserPort)
	wait_signal_mask |= 1L << fw->w->UserPort->mp_SigBit;

    if (fw->read_port)
	wait_signal_mask |= 1L << fw->read_port->mp_SigBit;
#endif
/*
 * DMR_init(fw->w)
 */

add_window_to_list(fw);
    fw->cursor_x = NO_CURSOR;

#ifdef AWSD
    if (use_custom == -2)
	nw.Flags &= ~(BORDERLESS | BACKDROP);
#endif

#ifdef WINDOWS
    ShowWindow (fw->w, SW_SHOWNORMAL);
    UpdateWindow (fw->w);
#endif
    change_window_size (fw);
    (*sw->hdr.process) (sw, WM_WINDOW_OPEN, 0, 0);
    change_window_size (fw);

    FW_init_window (fw);
    fw->active_groups = calloc(sizeof(*fw->active_groups),get_last_group());
#ifdef WINDOWS
{
void set_current_window(ZPL_WINDOW *);
set_current_window(fw);
}
#endif
    return (fw);
}

ZPL_WINDOW *open_window (GENERAL_WINDOW * sw)
{
    return (open_window_at (sw, -1, -1, -1, -1));
}

void FW_close (ZPL_WINDOW * fw)
{
/*
 * Tell the window that it is about to be closed.
 */
    (*fw->sub_window->hdr.process) (fw->sub_window, WM_WINDOW_CLOSE, 0, 0);

#ifdef AMIGA
    if (fw->w && fw->type & FW_TYPE_IO)
    {
	/*
	 * We always have an outstanding queued read request so we must abort
	 * it if it hasn't completed, and we must remove it.
	 */
	if (!(CheckIO ((struct IORequest *) fw->read_req)))
	    AbortIO ((struct IORequest *) fw->read_req);
	WaitIO ((struct IORequest *) fw->read_req);	/*
				 * clear it from our replyport
				 */

	CloseDevice ((struct IORequest *) fw->write_req);

	if (fw->read_req)
	    DeleteExtIO ((struct IORequest *) fw->read_req);
	if (fw->read_port)
	    DeletePort (fw->read_port);
	if (fw->write_req)
	    DeleteExtIO ((struct IORequest *) fw->write_req);
	if (fw->write_port)
	    DeletePort (fw->write_port);
    }
#endif

    if (fw->w)
    {
#ifdef AMIGA
	ClearMenuStrip (fw->w);
	CloseWindow (fw->w);
#endif
#ifdef WINDOWS
#ifdef NO_MDI
	DestroyWindow (fw->w);
#else
    SendMessage(get_mdi_client(), WM_MDIDESTROY, (DWORD) fw->w, 0L);
#endif
#endif
        free (fw->active_groups);
	fw->w = NULL;
    }
}

void close_all_windows (TY_int do_safely)
{
    ZPL_WINDOW *fw = first_window, *fw1;
    while (fw)
    {
	fw1 = fw->next;
	unlock_window (fw);

	if (do_safely)
	    FW_close_safely (fw);
	else
	    FW_close (fw);

	free (fw);
	fw = fw1;
    }
    first_window = NULL;
}

void close_all_children (void)
{
    ZPL_WINDOW *fw = first_window, *fw1;

fw=fw->next; /* Skip the MDI window */

while (fw)
    {
	fw1 = fw->next;
        FW_close (fw);
	fw = fw1;
    }
}

void do_child_windows (ZPL_WINDOW * pfw,
		       TY_int (*do_it) (ZPL_WINDOW *, void *), void *param)
{
    ZPL_WINDOW *fw = first_window;
    while (fw)
    {
	if (fw->parent == pfw)
	    (*do_it) (fw, param);
	fw = fw->next;
    }
}

void close_system ()
{
#ifdef AMIGA
    close_all_windows (FALSE);

EX_finish();
    if (old_pr_win_ptr && proc)
	proc->pr_WindowPtr = old_pr_win_ptr;

if(window_font)
CloseFont(window_font);
    if (GfxBase)
	CloseLibrary ((struct Library *) GfxBase);

    if (IntuitionBase)
	CloseLibrary ((struct Library *) IntuitionBase);

    if (ReqToolsBase)
	CloseLibrary (ReqToolsBase);

    if (GadToolsBase)
    {
	CloseLibrary (GadToolsBase);
    }
    else if (IntuiSupBase)
    {
	CloseLibrary (IntuiSupBase);
    }

    if (DiskfontBase)
	CloseLibrary (DiskfontBase);

    if (ConsoleDevice)
	CloseDevice ((struct IORequest *) &console_ioreq);

    if (custom_screen && !opened_on_public)
    {
	if(!using_v36() || CloseScreen (custom_screen))
FreeMem(screen_title, SCREEN_TITLE_SIZE);

	custom_screen = NULL;
    }
#endif
}

void update_windows ()
{
    ZPL_WINDOW *fw = first_window;

    while (fw)
    {
	if (fw->resources)
	    RE_update (fw->resources);
	FW_output_sb (fw, FALSE);
	fw = fw->next;
    }
}

void check_close_safely (void)
{
    if (safe_close_ptr)
	do_close_safely ();
}

void do_events ()
{
#ifdef AMIGA
    ZPL_WINDOW *fw = first_window;
    TY_int up = 0;
    while (fw)
    {
	if (check_events (fw))
	    up = TRUE;
	if (fw->process)
            dump_events(fw);
	fw = fw->next;
    }
    if (up)
	update_windows ();
MY_scan_check_memory ();
#endif
#ifdef WINDOWS
    MSG msg;

while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
    {
	if (!TranslateMDISysAccel(get_mdi_client(), &msg))
/*~~accell	if (!TranslateAccelerator (msg.hwnd, hAccelTable, &msg)) */
	{
	    TranslateMessage (&msg);
	    DispatchMessage (&msg);
	}
    }
#endif
}


void wait_for_event ()
{
#ifdef AMIGA

    /*
     * This waits for a reportable event to ocurr. When this event occurs,
     * then we'll check what it is
     */
    ULONG a;
/*
 * smask = 1L << w->UserPort->mp_SigBit;
 */
    a = Wait (wait_signal_mask | SIGBREAKF_CTRL_C | EX_get_signal_mask());

    if (!checking_for_events)
	return;

    if (a & wait_signal_mask)
    {
	do_events ();
    }

/*
 * See if the extend subsystem (i.e. Arexx) claims this...
 */
EX_process_message(a);

    if (a & SIGBREAKF_CTRL_C)
	CXBRK ();

#endif
#ifdef WINDOWS
    MSG msg;

    if (GetMessage (&msg,	/*
				 * message structure
				 */
		       NULL,	/*
				 * handle of window receiving the message
				 */
		       0,	/*
				 * lowest message to examine
				 */
		       0))	/*
				 * highest message to examine
				 */
    {
	if (!TranslateMDISysAccel(get_mdi_client(), &msg))
/* 	if (!TranslateAccelerator (msg.hwnd, hAccelTable, &msg)) */
	{
	    TranslateMessage (&msg);	/*
					 * Translates virtual key codes
					 */
	    DispatchMessage (&msg);
	}
    }
#endif
    check_close_safely ();
}

#ifdef AMIGA
ULONG get_signal_mask ()
{
    return (wait_signal_mask);
}

ULONG just_wait_for_event ()
{
    return (Wait (wait_signal_mask));
}
#endif


void do_received_events ()
{
    if (checking_for_events)
#ifdef AMIGA
	if (task->tc_SigRecvd & (wait_signal_mask | EX_get_signal_mask()))
	    wait_for_event ();
#endif
#ifdef WINDOWS
do_events();
#endif
}

/*
 * Send a fake message to a window. Usually used to get it to
 * update/check something which otherwise it would miss.
 */
void fake_message (ZPL_WINDOW * fw)
{
#ifdef AMIGA
    static struct IntuiMessage message;
    message.Class = NULL;
    message.Code = NULL;
    PutMsg (fw->w->UserPort, (struct Message *) &message);
#endif
/*
 * ~
 */
}


void global_disable_group (TY_int group)
{
    ZPL_WINDOW *fw = first_window;

    while (fw)
    {
        disable_group (fw, group);
	fw = fw->next;
    }
}

void global_enable_group (TY_int group)
{
    ZPL_WINDOW *fw = first_window;

    while (fw)
    {
        enable_group (fw, group);
	fw = fw->next;
    }
}

void resize_window (MD_WINDOW_P w, TY_int dw, TY_int dh)
{
#ifdef AAA
    ULONG a, smask = 1L << w->UserPort->mp_SigBit;
    TY_int complete = FALSE;
#endif

#ifdef AMIGA
    SizeWindow (w, dw, dh);
    Delay (5);
#endif
#ifdef WINDOWS
    MoveWindow (w, 0, 0, dw, dh, TRUE);
#endif
}

TY_int get_window_title_height ()
{
    if (first_window)
	return (FW_get_border_top (first_window));
    else
	return (10);
}

struct TextFont *get_console_font ()
{
#ifdef AMIGA
    if (first_window)
	return (first_window->w->IFont);
    else
#endif
	return (0);
}

TY_int FW_system (char *str)
{
#ifdef AMIGA
    extern TY_int from_wb;
    if (from_wb)
    {
	char *fsp, *cmdline = malloc (strlen (str) + 40);
	TY_int i;
	fsp = strchr (str, ' ');
	if (fsp)
	{
	    char *p1 = " ", *p2 = " ";
	    *fsp++ = 0;
	    if (!strchr (fsp, '>'))
		p1 = ">nil:";
	    if (!strchr (fsp, '<'))
		p2 = "<nil:";

	    sprintf (cmdline, "%s %s %s %s", str, p1, p2, fsp);
	}
	else
	    sprintf (cmdline, "%s <nil: >nil:", str);
	i = system (cmdline);
	free (cmdline);
	return (i);
    }
    else
#endif
	return system (str);
}

void quit_window (UI_ARGS)
{
    FW_close_safely (fw);
}

/*
 * Message decoding routines....
 */
TY_int get_message_code (struct IntuiMessage *msg)
{
#ifdef AMIGA
    return (msg->Code);
#endif
#ifdef WINDOWS
#endif
}

unsigned short get_message_key (void *p1)
{
#ifdef AMIGA
    struct IntuiMessage *msg = p1;
    return (msg->Code);
#endif
#ifdef WINDOWS
#endif
}

unsigned long get_message_class (void *p1)
{
#ifdef AMIGA
    struct IntuiMessage *msg = p1;
    return (msg->Class);
#endif
#ifdef WINDOWS
#endif
}

#ifdef AMIGA
struct IntuiMessage *get_blank_message (void)
{
    static struct IntuiMessage message;
    memset (&message, 0, sizeof (message));
    return (&message);
}
#endif

/*
 * Return the number of 'root' windows open. This does not include child
 * windows or non input windows, which are considered to be part of the parent.
 */
TY_int get_number_of_windows (void)
{
    ZPL_WINDOW *fw = first_window;
    TY_int wc = 0;

    while (fw)
    {
#ifdef AMIGA
        if(!fw->parent && fw->w->UserPort)
#else
        if(!fw->parent)
#endif
	   wc++;
	fw = fw->next;
    }
    return (wc);
}

#ifdef WINDOWS
void chkabort ()
{
}

TY_int using_v36 ()
{
    return (0);
}
#endif

void all_windows_busy(TY_int set_busy)
{
    ZPL_WINDOW *fw = first_window;

    while (fw)
    {
	if (fw->resources)
	    RE_update (fw->resources);

	if (set_busy)
	{
            FW_set_busy_pointer (fw);
	}
	else
	{
            if(!fw->busy_count)
                FW_clear_busy_pointer (fw);
            dump_events(fw);
	}
	fw = fw->next;
    }
}

