/*---------------------------------------------------------------------------
 *
 *	Title:		ZPL main event handling (amiga)
 *
 *	File:		event.c
 *
 *	Date:		14 October 1991
 *
 *	Description:
 *
 *	Rev: 1.0
 *	1.0	R.J.Harrison	Release			             1 Sept 92
 *
 *	Copyright  1990 ZARGsoft / The exploding software company
 *
 *  $Log$
 *
 *-------------------------------------------------------------------
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#ifdef AMIGA
#include <exec/types.h>
#include <exec/nodes.h>
#include <exec/lists.h>
#include <exec/io.h>
#include <exec/memory.h>

#include <intuition/intuition.h>
#include <intuition/intuitionbase.h>

#include <dos/dos.h>
#include <dos/dosextens.h>
#include <libraries/reqtools.h>
#include <libraries/dos.h>

#include <graphics/text.h>

#include <proto/exec.h>
#include <proto/graphics.h>
#include <proto/gadtools.h>
#include <proto/intuition.h>
#endif

#include "zpl.h"

#ifdef AMIGA
#include "intuisup.h"
#endif


#define BUFSIZE 40

extern struct Library *IntuiSupBase;

#define GADGET_IDCMP (IDCMP_MOUSEBUTTONS | IDCMP_MOUSEMOVE | IDCMP_GADGETDOWN | IDCMP_GADGETUP)

TY_int check_events (ZPL_WINDOW * fw)
{
    USHORT class;		/* Intu event class */
    USHORT code;		/* Intu event code */
    struct IntuiMessage *i_message, t_msg, *message = &t_msg;
    TY_int update_required = FALSE;

    /*
     * Only check events on windows which have an IDCMP (replyport)
     */
    if (!fw || !fw->w || !fw->w->UserPort)
	return (0);

    if (!find_window (fw->w))
	return (0);

    if (fw->type & FW_TYPE_GADTOOLS)
    {
	while (fw->w && (i_message =
			 (using_v36() ? GT_GetIMsg (fw->w->UserPort) :
			  IGetMsg (fw->w->UserPort))))
	{
	    *message = *i_message;
	    class = message->Class;
	    code = message->Code;

	    if (using_v36())
		GT_ReplyIMsg ((struct IntuiMessage *) i_message);
	    else
		IReplyMsg (i_message);

if(!BIT_SET(fw->type, FW_PROCESSING))
{
	    if (using_v36() && class == GADGETUP)
		FW_remove_cursor (fw);

	    if (using_v36() && class == REFRESHWINDOW)
	    {

		GT_BeginRefresh (fw->w);
		GT_EndRefresh (fw->w, TRUE);
	    }

	    if (class == MENUPICK)
	    {
		while (code && (code != MENUNULL))
		{
		    code = process_menu (code, fw);
		}
	    }
	    if (class != INTUITICKS)
		update_required = TRUE;
	    if (event (message, fw))
		break;
}
	}
    }
    else
    {
	while (fw->w &&
	     (i_message = (struct IntuiMessage *) GetMsg (fw->w->UserPort)))
	{
	    *message = *i_message;

	    class = message->Class;
	    code = message->Code;
	    ReplyMsg ((struct Message *) i_message);

if(!BIT_SET(fw->type, FW_PROCESSING))
{
	    if (class == MENUPICK)
	    {
		while (code != MENUNULL)
		{
		    code = process_menu (code, fw);
		}
	    }
	    if (class != INTUITICKS)
		update_required = TRUE;
	    if (event (message, fw))
		break;
}
	}
    }
    return (update_required);
}

TY_int dump_events (ZPL_WINDOW * fw)
{
    TY_int old_type = fw->type;

    SET_BIT (fw->type, FW_PROCESSING);
    check_events(fw);
    fw->type = old_type;
    return (0);
}


/*
 * Convert RAWKEYS into VANILLAKEYS
 * returns the number of characters placed in the buffer, or a -ve number
 * indicating failuer
 */

TY_int rawkey_convert (struct IntuiMessage *msg, char *kbuffer, TY_int kbsize, struct KeyMap *kmap)
{
    static struct InputEvent ievent =
    {NULL, IECLASS_RAWKEY, 0, 0, 0};

    if (msg->Class != RAWKEY)
	return (-2);

    memset (kbuffer, 0, kbsize);
    ievent.ie_Code = msg->Code;
    ievent.ie_Qualifier = msg->Qualifier;
    ievent.ie_position.ie_addr = *((APTR *) msg->IAddress);
    return (RawKeyConvert (&ievent, kbuffer, kbsize, kmap));
}

static void *keymap;

void set_event_keymap (void *km)
{
keymap = km;
}
/*
 * Process all of the keys which have been pressed
 */

VOID
DoKeys (struct IntuiMessage * msg, ZPL_WINDOW * fw)
{
    TY_int i, numchars;
    static char buffer[BUFSIZE];

    numchars = rawkey_convert (msg, buffer, BUFSIZE, keymap);

    /*
     * numchars now contains the number of characters placed within the
     * buffer.  If this was a message notifying us of a key release, then it
     * shows zero.  Special keys (like HELP, the cursor keys, FKeys, etc)
     * return multiple characters that have to then be parsed.
     */

    if (numchars > 0 && numchars < sizeof (buffer))
    {
	for (i = 0; i < numchars; i++)
	{
	    window_input (fw, buffer[i]);
	}
    }
    else if (numchars > 0)
	error_log ("DoKeys(): Suspicious number of characters received %d", numchars);
}

TY_int event (struct IntuiMessage *msg, ZPL_WINDOW * fw)
{
/*
 * Each window can have a single process to which it's events are diverted. This
 * is usually used to lock a window whilst a request is made, without locking the
 * system (eg. add/edit/cancel)
msg->Class != CLOSEWINDOW &&
 */
    if (fw->process && msg->Class != NEWSIZE)
    {
	TY_int action;
	if (action = (*fw->process) (fw, msg->Class, msg))
	{
	    if (fw->process_complete)
		(*fw->process_complete) (fw, action);
	}

	if (msg->Class == RAWKEY && fw->process)
	{
	    static char buffer[BUFSIZE];

	    if (rawkey_convert (msg, buffer, BUFSIZE, 0L) == 1)
	    {
		msg->Class = VANILLAKEY;
		msg->Code = buffer[0];
		(*fw->process) (fw, msg->Class, msg);
	    }
	}
    }
    else
    {
	if (fw->type & FW_TYPE_GADTOOLS && msg->Class != INTUITICKS)
	{
	    struct Gadget *gad;

	    if (using_v36())
	    {
		if (msg->Class == RAWKEY && fw->resources)
		{
		    RE_resource_entry *re = RE_find_shortcut (fw->resources, RE_scroll_lv_id);
		    if (re)
		    {
			switch (msg->Code)
			{
			case KE_left:
			    RE_page_scroll_lv (re, TRUE);
			    break;
			case KE_right:
			    RE_page_scroll_lv (re, FALSE);
			    break;
			case KE_up:
			    RE_scroll_lv (re, TRUE);
			    break;
			case KE_down:
			    RE_scroll_lv (re, FALSE);
			    break;
			}
		    }
		}
		if (msg->Class == VANILLAKEY && fw->resources)
		{
		    RE_resource_entry *re = RE_find_shortcut (fw->resources, msg->Code);
		    if (re)
			RE_process_shortcut (re, msg, fw->w);
		    else
		    {
			if (msg->Code == 27)
			    FW_close_safely (fw);
			else
			{
			    if (msg->Code == '9' || msg->Code == '3' ||
				msg->Code == ' ' || msg->Code == '8' || msg->Code == '2')
			    {
				RE_resource_entry *re = RE_find_shortcut (fw->resources, RE_scroll_lv_id);
				if (re)
				{
				    switch (msg->Code)
				    {
				    case KE_left:
					break;

				    case '9':
					RE_page_scroll_lv (re, TRUE);
					break;
				    case '3':
					RE_page_scroll_lv (re, FALSE);
					break;
				    case '8':
					RE_scroll_lv (re, TRUE);
					break;
				    case '2':
					RE_scroll_lv (re, FALSE);
					break;
				    case ' ':
					msg->Code = re->cur_pos;
					RE_process_shortcut (re, msg, fw->w);
					break;
				    }
				}
			    }
			}
		    }
		}

		if ((TY_int) msg & 1)
		    error_log ("Odd msg addres %d", msg);
		else if (msg->Class & GADGET_IDCMP)
		{
		    TY_int valid = TRUE;
		    gad = (struct Gadget *) msg->IAddress;

		    if ((TY_int) gad < 100)
			valid = FALSE;

		    if (valid &&/*msg->Class == IDCMP_MOUSEMOVE &&*/
			!RE_is_valid_gadget (fw->resources, gad))
			valid = FALSE;

		    if (valid && (TY_int) gad & 1)
		    {
			valid = FALSE;
			error_log ("Odd gad addres %d", msg);
		    }
		    if (valid && (TY_int) gad->UserData & 1)
		    {
			valid = FALSE;
			error_log ("Odd gadud addres %d", msg);
		    }
		    if (valid && (TY_int) &gad->UserData & 1)
		    {
			valid = FALSE;
			error_log ("Odd *gadud addres %d", msg);
		    }

		    if (valid)
		    {
/* If this is a valid gadget then process it.. */
			if (gad && gad->UserData)
			    RE_process_event (gad->UserData, msg);
		    }
		}
	    }
	    else
	    {
/*
 * With IS, the code is the offset into the array.~
 */
		if (fw->resources && msg->Class == ISUP_ID)
		{
		    RE_resource_entry *r = RE_find_is (fw->resources, msg->Code);
		    RE_process_event (r, msg);
		}
	    }
	    FW_clear_more (fw);
	}
	switch (msg->Class)
	{
	case CLOSEWINDOW:
if(!BIT_SET(fw->type, FW_DISALLOW_CLOSE))
{
	    FW_set_busy_pointer (fw);
	    if (get_number_of_windows() > 1)
		FW_close_safely (fw);
	    else
		close_window (fw);
}
	    return (1);

	case RAWKEY:
            if (fw->type & FW_TYPE_IO)
                DoKeys (msg, fw);
	    break;

	case NEWSIZE:
	    change_window_size (fw);
	    break;

	case ACTIVEWINDOW:
	    return (0);

	case INACTIVEWINDOW:
	    return (0);

	}
    }
    return (0);
}
