#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>


#include "zpl.h"

#ifdef M_AMIGA
#include <exec/memory.h>
#include <libraries/dos.h>
#include <dos/dos.h>
#include <libraries/dosextens.h>
#endif

#ifndef SAVESTR_CHECK
/*
 * Save a copy of the string given into dynamic memory. Free'ing of this
 * is the responsibility of the acquiring process
 */
char *savestr (char *p)
{
    char *c = malloc (strlen (p) + 3);
if(c)
    strcpy (c, p);
    return (c);
}

#endif

char *skip_spaces (char *p)
{
    while (*p && isspace (*p))
	p++;
    return (p);
}

TY_int filesize (FILE * f)
{
    TY_int cp, sz;
    if (!f)
	return (0);
    cp = ftell (f);
    fseek (f, 0, 2);
    sz = ftell (f);
    fseek (f, cp, 0);
    return (sz);
}

void write_int (FILE * f, TY_int i)
{
    i = SWAP_LONG(i);
    fwrite ((char *) &i, sizeof (TY_int), 1, f);
}

void write_text (FILE * f, char *text)
{
    if (!text)
    {
	write_int (f, 0);
	return;
    }
    write_int (f, strlen (text));
    fwrite (text, strlen (text), 1, f);
}

TY_int read_int (FILE * f)
{
    TY_int i, res;
    if (!feof (f))
    {
	res = fread ((char *) &i, sizeof (TY_int), 1, f);
	if (res < 1)
	    return (0);
	else
            return SWAP_LONG(i);
    }
    else
	return (0);
}
/*
 * Read a text string from a file, and set an error code.
 */
char *read_text_ec (FILE * f, TY_int *ec)
{
    TY_int len = read_int (f), res;
    char *text;

*ec = 0;
/*
 * To catch memory gobbling file corruptions ensure that the amount
 * of string specified to read actually exists within this file...
 */
    if ((ftell (f) + len <= filesize (f)) || feof (f))
    {
	if (feof (f))
	    return (NULL);
	if (len)
	{
            text = malloc (len + 1);
	    res = fread (text, 1, len, f);
	    if (res < len - 3)
	    {
		error_log ("Error: attempt to read string at end of file");
		free (text);
*ec = 1;
		return (0);
	    }
   	    text[len] = 0;
            return (text);
	}
    }
    else
    {
	if (feof (f))
{
	    error_log ("Error: attempt to read string at end of file");
*ec = 1;
}
	else
{
	    error_log ("Error: Text string too large to read %ld", len);
*ec = 1;
}
    }
    return (NULL);
}

char *read_text (FILE * f)
{
    TY_int ec;
    return (read_text_ec(f,&ec));
}

char *strindex (char *s, char *wanted)
{
    register char *scan;
    register len;
    register char firstc;

    /*
     * The odd placement of the two tests is so "" is findable. Also, we
     * inline the first char for speed. The ++ on scan has been moved down
     * for optimization.
     */
    firstc = *wanted;
    len = strlen (wanted);
    for (scan = s; *scan != firstc || strncmp (scan, wanted, len) != 0;)
	if (*scan++ == '\0')
	    return (0);
    return (scan);
}


/*
 * Split a string up into an array of pointers to the delimited parts.
 */
char **getnamelist (char *ptr)
{
    char *p = ptr, *e;
    char **names;
    TY_int num = 10, cur_offset = 0;


    if (!(names = calloc (num, sizeof (*names))))
	return (NULL);

    while (p && *p)
    {
	p = skip_spaces (p);
	if (*p == '"')
	{
	    e = strchr (++p, '"');
	    if (e)
		*e++ = 0;
	}
	else
	{
	    e = strchr (p, ',');
	    if (e)
		*e = 0;
	}
	names[cur_offset] = malloc (strlen (p) + 1);
	strcpy (names[cur_offset++], p);

/*
 * When we run out of space in our current allocated lump, get
 * another chunk.
 */
	if (cur_offset >= num)
	{
	    TY_int old_num = num;
	    char **old_names;

	    if (!(old_names = calloc (num, sizeof (*names))))
	    {
		error_log ("getnamelist(): failed to get more memory for old");
		return (NULL);
	    }
	    memcpy (old_names, names, sizeof (*names) * num);
	    free (names);
	    num += PR_get_granularity ();
	    if (!(names = calloc (num, sizeof (*names))))
	    {
		error_log ("getnamelist(): failed to get more memory for new");
		return (NULL);
	    }
	    memcpy (old_names, names, sizeof (*names) * old_num);
	    free (old_names);
	}
	if (e)
	    p = ++e;
	else
	    p = NULL;
    }
    if (!cur_offset)
    {
	error_log ("getnamelist(): Error creating namelist");
	free (names);
	return (NULL);
    }
    else
	return (names);
}


char *op_ltoa (TY_int n)
{
    static char nb[20];
    sprintf (nb, "%ld", n);
    return (nb);
}

char *decrud (char *str)
{
    char *ostr = str;
    while (str && *str)
    {
	if (*str < ' ')
	    *str = ' ';
	str++;
    }

    return (ostr);
}

void change_case_upper (char *c)
{
    if (c)
	while (*c)
	{
	    if(islower(*c))
	       *c = toupper (*c);
	    c++;
	}
}

void change_case_lower (char *c)
{
    if (c)
	while (*c)
	{
	    if(isupper(*c))
  	        *c = tolower (*c);
	    c++;
	}
}
void nchange_case_upper (char *c, TY_int n)
{
    if (c)
	while (*c && --n > 0)
	{
	    if(islower(*c))
		    *c = toupper (*c);
	    c++;
	}
}

void nchange_case_lower (char *c, TY_int n)
{
    if (c)
	while (*c && --n > 0)
	{
	    if(isupper(*c))
	        *c = tolower (*c);
	    c++;
	}
}

void cr_expand (char *s)
{
    char *o = s;
    while (*s)
    {
	if (*s == '\\')
	{
	    switch (*++s)
	    {
	    case '\\':
		*o++ = *s;
		break;
	    case 'n':
		*o++ = '\n';
		break;
	    case 't':
		*o++ = '\t';
		break;
	    case '0':
		{
		    TY_int i = atoi (s);
		    *o++ = i;
		    while (*s && isdigit (*s))
			s++;
		    s--;
		    break;
		}
	    default:
		error_log (stderr, "cr_expand(): Strange formatting char [%c]\n", s[1]);
	    }
	    s++;
	}
	else
	    *o++ = *s++;
    }
    *o = 0;
}

char *file_part (char *path)
{
    if (path)
    {
	char *a = strchr (path, ':'), *b = strrchr (path, '/'), *c = strrchr(path,'\\');
	if (b)
	    return (++b);
	if (c)
	    return (++c);
	if (a)
	    return (++a);
    }
    return (path);
}

char *directory_part (char *i_path)
{
    if (i_path && *i_path)
    {
	static char path[200];
	strcpy (path, i_path);
	if (*path)
	{
	    char *a = file_part (path);
	    if (a)
		*a = 0;
	    return (path);
	}
    }
    return (i_path);
}


char *add_part (char *dirname, char *filename, TY_int maxlen)
{
    if (dirname && filename && maxlen)
    {
	if (dirname && *dirname)
	{
	    char *fp = file_part (filename), *dn = file_part (dirname),
	     *a = strchr (dirname, ':');
	    char *eos = &dirname[strlen (dirname) - 1];
/*
 * If the file_part starts at a different place than the given 'dirname' or if the
 * last character in the string isn't a ':' then add a '/'
 */
	    if (dn != dirname && (a && !*dn && !a[1]))
		*dn = 0;
	    else if (*eos != '/')
		strcat (dirname, "/");
	    strcat (dirname, fp);
	}
	else
	    strcpy (dirname, filename);
    }
    return (dirname);
}

/*
 * remove carriage returns & linefeeds
 */
void decr (char *s)
{
    if (s)
    {
	while (*s)
	{
	    if (*s == '\n' || *s == 13)
		*s = 32;
	    s++;
	}
	s--;
	while (*s && isspace (*s))
	{
	    *s-- = 0;
	}
    }
}

void remove_cr (char *s)
{
    if (s)
    {
	while (*s)
	{
	    if (*s == 13)
		*s = 32;
	    s++;
	}
	s--;
	while (*s && isspace (*s))
	{
	    *s-- = 0;
	}
    }
}

TY_int read_line (char *l, TY_int sz, FILE * f)
{
    char *li = fgets (l, sz, f);
    if (li)
    {
	l[sz - 1] = 0;
	decr (l);
	return (TRUE);
    }
    return (NULL);
}


/*
 * Insert a character into a string.
 */

/*
 * Given a chunk of text, reformat it to fit within the margin.
 */
char *str_insert (char *p1, TY_int sz)
{
    char *eos = &p1[strlen (p1)];
    if (eos)
    {
	eos += sz;
	while (eos > p1)
	{
	    *eos = eos[-sz];
	    eos--;
	}
	memset (p1, 32, sz);
	return (p1);
    }
    else
	return (NULL);
}

/*
 * Start off at the beggining.
 * Find the first new line.
 * Calculate the line length.
 * If greater than the right margin then start to wrap
 * wrap by inserting a NL, then removing the real NL.
 * Continue this until an end of paragraph marker is found.
 */

void WW_wraptext (char *text, TY_int rm)
{
    char *cp, *eol_p, *sol;
    TY_int changed, filled, quote_count;
    char quoted = FALSE, *quote_finish = FALSE;

    cp = text;

    while (cp)
    {
	eol_p = strchr (cp, '\n');

	/*
	 * Does it need to wrap ?
	 */
	quote_finish = 0;
	if (eol_p && eol_p - cp > rm)
	{
	    TY_int done = 0;

	    sol = cp;
	    if (sol)
	    {
		if (*sol == '>')
		{
		    quote_finish = sol - 1;
		    while (quote_finish && quote_finish[1] == '>')
		    {
			quote_finish = strchr (quote_finish + 1, '\n');
		    }
		}
	    }

	    if (*cp == '>')
	    {
		TY_int i = 0;
		quoted = *cp;
		quote_count = 0;
		while (cp[i] && cp[i++] == quoted)
		    quote_count++;

	    }
	    if (eol_p[1] == '>')
	    {
		TY_int i = 0;
		while (eol_p[++i] && eol_p[i] == quoted) ;
		strcpy (eol_p + 1, eol_p + i);
	    }


/*
 * This next bit is repeated until the whole paragraph has been filled
 */

	    while (!done)
	    {
		char *p1 = cp + rm;

/*
 * Start at the Right Margin, and work backwards until a space is found
 */
		while (p1 > cp && !isspace (*p1))
		    p1--;

		if (p1 > cp)
		{
		    *p1 = '\n';
		}
		else
		{
		    p1 = cp + rm;
		    *p1 = '\n';
		}

/*
 * If the line was originally quoted, then quote the continuation too
 */
		if (quoted)
		{
		    str_insert (p1 + 1, quote_count);
		    memset (p1 + 1, quoted, quote_count);
		}
		cp = ++p1;
		filled = 0;

/*
 *
 */
		while (!filled)
		{

		    eol_p = strchr (cp, '\n');
		    if (!eol_p || eol_p[1] == '\n')
			done = 1;

		    if (quote_finish && eol_p >= quote_finish)
			done = 1;

		    if (!done && eol_p < (cp + rm))
		    {
			*eol_p = ' ';
			if (quoted && eol_p[1] == quoted)
			{
			    TY_int i = 0, oqc = quote_count, nqc;
			    nqc = 0;
			    while (eol_p[++i] && eol_p[i] == quoted)
				nqc++;
			    if (oqc != nqc)
			    {
				eol_p[0] = '\n';
				quote_count = nqc;
				cp = eol_p + 1;
			    }
			    else
			    {
				strcpy (eol_p + 1, eol_p + i);
			    }
			}
		    }
		    else
			filled = 1;

		}
	    }
	}
	else
	{
	    quoted = FALSE;
	    if (eol_p)
		cp = ++eol_p;
	    else
		cp = NULL;
	}
    }
}

#ifdef DO_TEST
void WW_getfile (char *filename, TY_int rm)
{
    FILE *f;
    if (f = fopen (filename, "r"))
    {
	TY_int sz = filesize (f);
	char *c;
	if (sz)
	{
	    c = malloc (sz);
	    if (c)
	    {
		sz = fread (c, 1, sz, f);
		c[sz] = 0;
		WW_wraptext (c, rm);
		printf ("%s:\n", c);
	    }

	}
    }
}
#endif

#ifdef M_AMIGA
static char *get_comment(char *filename)
{
    struct FileLock *lock = Lock (filename, ACCESS_READ);

    static struct FileInfoBlock fib;

    if (!lock)
	return (0);

if (!Examine (lock, &fib))
*fib.fib_Comment = 0;
	    UnLock (lock);
return (fib.fib_Comment);
}
#endif

/*
 * Zero indicates success, non-zero = failure
 */
static TY_int copy_or_move (char *old_name, char *new_name, TY_int move_it)
{
TY_int return_code=0;

    if (old_name && *old_name && new_name && *new_name)
    {
/*
 * If moving, firsly try to rename the file, if this fails then
 * use the copy/delete technique....
 */
if(move_it && rename_file (old_name, new_name))
return(0);

{
char *com;
	FILE *f = fopen (old_name, "r");
	if (f)
	{
	    FILE *of = fopen (new_name, "w");
	    TY_int sz = 4000, r_sz;
	    char *c;

	    if (!of)
	    {
                fclose (f);
		error_log ("Error opening %s for write", new_name);
return_code = 1;
	    }
            else
	    if (sz)
	    {
		c = malloc (sz);
		if (c)
		{
		    while (!feof (f))
		    {
			r_sz = fread (c, 1, sz, f);
			if (sz > 0)
			    fwrite (c, 1, r_sz, of);
		    }
		    free (c);
		}
	    }
if(of)
    fclose (of);

    fclose (f);

#ifdef AMIGA
com = get_comment(old_name);
if (com)
SetComment(new_name, com);
#endif
		    if (move_it)
			return_code = unlink (old_name);
	}
}
return (return_code);
    }
return (1);
}

TY_int move_file (char *old_name, char *new_name)
{
    return copy_or_move (old_name, new_name, TRUE);
}

TY_int copy_file (char *old_name, char *new_name)
{
    return copy_or_move (old_name, new_name, FALSE);
}

TY_int file_present (char *filename)
{
    struct stat st;

    if (stat (filename, &st))
    {
	return (FALSE);
    }
    else
	return (TRUE);
}

char *strmxcpy (char *s1, char *s2, TY_int maxlen)
{
    if (strlen (s2) < maxlen)
	return (strcpy (s1, s2));
    strncpy (s1, s2, maxlen - 1);
    s1[maxlen - 1] = 0;
    return (s2);
}


/*
 * Used for searching flists..
 */
TY_int file_match (char *line, char *file, TY_int anywhere_on_line)
{
    char *c, *cc, tstr[120];

strmxcpy(tstr,line,sizeof(tstr));
change_case_lower(tstr);

    if (c = strindex (tstr, file))
    {
	if (anywhere_on_line)
	    return (TRUE);
	else
	{
	    cc = tstr;
	    while (*cc && isspace (*cc))
		cc++;
	    if (cc == c)
	    {
		TY_int len = strlen (file);
		if (isspace (c[len]))
		    return (2);
		else
		    return (TRUE);
	    }
	}
    }
    return (FALSE);
}

FILE *create_message_save (char *file, TY_int print)
{
    char *open_mode="w";

    if (print)
    {
	file = PR_get_printer_device ();
        if (print == 2)
        {  if (strchr(file,':') 
              && !multi_do_it("Please make printer %s ready",file,"_Print|_Cancel"))
            return 0;
        }
    }

    if (print || (file && (open_mode=check_new_file (file))))
    {
	FILE *f = fopen (file, open_mode);
	if (f)
	{
	    return (f);
	}
	else
	    file_error (file);
    }
    return (0); /* aaaa */
}

void message_save (FILE * f, char *text)
{
    if (f)
    {
	fwrite (text, strlen (text), 1, f);
	fprintf (f, "\n");
    }
}

void finish_message_save (FILE * f, TY_int print)
{
    if (f)
    {
	fprintf (f, "\n");
	if (print && PR_add_formfeed ())
	    fprintf (f, "%c", 12);
	fclose (f);
    }
}

char *output_conf_name (char *conf, char *topic)
{
    static char ts[50];
    if (!conf)
	return (NULL);
    if (topic)
    {
	strcpy (ts, conf);
	strcat (ts, "/");
	strcat (ts, topic);
	return (ts);
    }
    if (conf)
	return (conf);
    return (0);
}

TY_int named_filesize (char *filename)
{
    struct stat st;

    if (stat (filename, &st) == 0)
    {
	return ((TY_int) st.st_size);
    }
    else
	return (0);
}

TY_int get_file_date (char *filename)
{
    struct stat st;

    if (stat (filename, &st) == 0)
    {
	return ((TY_int) st.st_mtime);
    }
    else
	return (0);
}

TY_int does_file_exist (char *filename)
{
    struct stat st;

    if (stat (filename, &st))
    {
	return (FALSE);
    }
    else
	return (TRUE);
}

time_t get_file_modtime (char *filename)
{
    struct stat buf;
    TY_int sr;

    sr = stat (filename, &buf);
/*
 *
 */
    if (sr >= 0)
    {
	return (buf.st_mtime);
    }
    return (0);
}


#ifdef M_AMIGA
char *FIB_get_name (struct FileInfoBlock *fs)
{
    return (fs->fib_FileName);
}

static TY_int breakflag;

void treewalk (filename, fn, arg)
char *filename;
void (*fn) ();
void *arg;
{
    char name[2000];
    strcpy (name, filename);
    walk (filename, fn, name, arg);
}

TY_int walk (filename, fn, path, arg)
char *filename;
void (*fn) ();
char *path, *arg;
{
    register struct FileInfoBlock *fib;
    char file[80], cur_path[120];
    struct FileLock *lock = Lock (filename, ACCESS_READ);

    if (!path)
	path = filename;

    if (!lock)
    {
	fprintf (stderr, "Unable to lock %s\n", filename);
	return (-1);
    }
#ifdef DEV
    printf ("walk(%s,%x,%s)\n", filename, fn, path);
#endif

    fib = (struct FileInfoBlock *) calloc (sizeof (struct FileInfoBlock), 1);
    if (fib == NULL)
    {
	fprintf (stderr, "can't allocate file info block\n");
	return (0);
    }
    strcpy (cur_path, path);
    /*
     * examine initial path name
     */

    if (Examine (lock, fib))
    {

	/*
	 * execute the expression on the inital path
	 */
	if (fib->fib_DirEntryType > 0)
	{
#ifdef FIONA

	    /*
	     * set up printable path name
	     */

	    strcat (cur_path, fib->fib_FileName);
	    strcat (cur_path, "/");
#endif
	}
	else
	{

	    /*
	     * if initial path name is not a directory then we just return
	     */
	    (*fn) (fib, path, arg);

	    free (fib);
	    UnLock (lock);
	    return (0);
	}

	/*
	 * examine directory contents
	 */

	while (ExNext (lock, fib))
	{
	    if (breakflag)
		break;

	    /*
	     * recurse if we have found a directory
	     */

	    if (fib->fib_DirEntryType > 0)
	    {
		char *old_file = malloc (strlen (filename) + 1);
		if (old_file)
		{
		    char cur_path[120];
		    strcpy (old_file, filename);

		    if (*path)
		    {
			strcpy (cur_path, path);
			strcat (cur_path, fib->fib_FileName);
			strcat (cur_path, "/");
		    }
		    else
		    {
			strcpy (cur_path, fib->fib_FileName);
			strcat (cur_path, "/");
		    }
		    strcpy (file, cur_path);

		    UnLock (lock);
		    walk (file, fn, cur_path, arg);
		    if (!(lock = Lock (old_file, ACCESS_READ)))
		    {
			fprintf (stderr, "locking error -:%s:\n", old_file);
			free (old_file);
			return (-1);
		    }
		    free (old_file);
		}
		else
		{
		    fprintf (stderr, "Unable to get memory\n");
		}
	    }
	    else
	    {
		(*fn) (fib, path, arg);
	    }

	    if (SetSignal (0L, 0L) & SIGBREAKF_CTRL_C)
	    {
		breakflag = TRUE;
		break;
	    }
	}
    }
    free (fib);
    UnLock (lock);
}
#endif

#ifdef WINDOWS
char *FIB_get_name (void *A)
{
    return (0);
}

void treewalk (filename, fn, arg)
char *filename;
void (*fn) ();
void *arg;
{
    tell_em ("treewalk(), not coded", 0);
}
#endif

#ifdef VMS
char *FIB_get_name (void *A)
{
    return (0);
}

void treewalk (filename, fn, arg)
char *filename;
void (*fn) ();
void *arg;
{
    tell_em ("treewalk(), not coded", 0);
}
#endif

void gsub (char *str, char old_c, char new_c)
{
    char *c;
    if (str)
	while (c = strchr (str, old_c))
	    *c = new_c;
}

char *make_text_from_open_file (FILE *f)
{
    char *text, *c, *t1;
    TY_int fsize;

    if (!f)
    {
	return (NULL);
    }
    fseek (f, 0, 2);		/*
				 * Seek to end of file
				 */
    fsize = ftell (f);		/*
				 * Figure out how long the file is
				 */
if(!fsize)
return (NULL);

    text = malloc (fsize + 10);
    if (!text)
    {
	error_log ("Unable to get %ld bytes of memory for reply", fsize + 1);
	return (NULL);
    }
    fseek (f, 0, 0);		/*
				 * Seek to start of file
				 */
    fread (text, fsize, 1, f);
    text[fsize] = 0;
    return (text);
}

char *make_text_from_file (char *filename)
{
    FILE *f;

    if (f = fopen (filename, "r"))
    {
char *text = make_text_from_open_file(f);
    fclose (f);
	return (text);
    }

    return (0);
}

char *describe_size(TY_int size)
{
STATIC_VAR(buf,char,50);

if(size < 1000)
sprintf(buf,"%5ld bytes",size);
else if(size < 1000000)
sprintf(buf,"%5.1fkb", (float) (size / 1024.0));
else
sprintf(buf,"%5.1fMb", (float) (size / 1024000.0));
return (buf);
}

TY_int free_mem_size(void)
{
#ifdef AMIGA
return (AvailMem(0));
#else
return (300000);
#endif
}

char *strcpy_field (TY_int field, char *output, char *input)
{
    char *ptr = input, *field_ptr, length = 0;

    while (ptr && field)
    {
	field--;
	field_ptr = ptr;
	ptr = strchr (ptr, ' ');
	if (ptr)
	{
	    length = ptr - field_ptr;
	    ptr = skip_spaces (ptr);
	}
    }

    if (!field && field_ptr)
    {
	if (length)
	{
	    strncpy (output, field_ptr, length);
	    output[length] = 0;
	}
	else
	    strcpy (output, field_ptr);
    }

    return field_ptr;
}

/*
 * Ensures that there is enough space in the string for the insertion
 * (if ptr is passed it is assumed that this is a pointer to a char
 *  within the string, and it will be modified to point to the same char
 *  in the new string)
 */
char *make_space(char *line, char **ptr, TY_int new_length, TY_int *line_len)
{
    TY_int ptr_offset;
    char *new_line;

    if (new_length >= *line_len)
    {
        if(ptr)
	    ptr_offset = *ptr - line;
	new_length += 20;
	new_line = calloc (new_length, 1);
	if (!new_line)
	    return line;
	strcpy (new_line, line);
	line = new_line;
	*line_len = new_length;
        if(ptr)
	    *ptr = line + ptr_offset;
    }
return line;
}

/*
 * Inserts the required amount of space into the string.
 * *NOTE* This assumes that there is sufficient space, so 
 *        call make space first if you're not sure...
 */
void insert_space (char *ptr, TY_int space)
{
    char *ip;

    if(!ptr || !*ptr) return;

    ip = ptr + strlen (ptr) + space;
    *ip-- = 0;
    while (ip >= ptr)
    {
	*ip = ip[-space];
	ip--;
    }
}

char *skip_sp(char *ptr)
{
 while(*ptr && isspace(*ptr))ptr++;
return ptr;
}

char *get_short_time (time_t ti)
{
    struct tm *tm;
    STATIC_VAR(temp, char, 10);

*temp = 0;
    if (ti)
    {
	tm = localtime(&ti);
	sprintf(temp, "%02d:%02d",tm->tm_hour, tm->tm_min);
    }
return temp;
}

void put_file_id (FILE * f, TY_int magic_no)
{
    fseek (f, 0, 0);
    write_int (f, magic_no);
}

TY_int get_file_id (FILE * f, char *filename)
{
    TY_int test_no;
    fseek (f, 0, 0);
    test_no = read_int (f);
    return (test_no);
}

TY_int check_file_id (FILE * f, char *filename, TY_int magic_no)
{
    if (get_file_id (f, filename) != magic_no)
    {
	error_log ("Bad magic number in file %s", filename);
	fseek (f, 0, 0);
	return (1);
    }
    else
	return (0);
}

FILE *smart_fopen (char *filename, char *mode, char *msg)
{
    int retry = TRUE;
FILE *f;

    do
    {
    if (!(f = fopen (filename, mode)))
    {
	if (!multi_do_it (msg, filename, "_Retry|_Cancel"))
	    retry = FALSE;
    }
    } while (!f && retry);
return f;
}

