/*---------------------------------------------------------------------------
 *
 *      Title:          NICOLA - Resource Database main module for Motif
 *
 *      File:           RE_motif.c
 *
 *      Date:           27-APR-1994
 *
 *      Description:
 *
 *      Copyright  1994 R.J.Harrison
 *
 *      Rev:    By              Done                                    Date
 *      1.00    R.J.Harrison    Implement                               27-APR-1994
 *
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <math.h>



#include <stdio.h>		/* For printf and so on. */
#include <Xm/Text.h>
#include <Mrm/MrmAppl.h>	/* Motif Toolkit and MRM */
#include <X11/X.h>

#define MODULE "res/re_main.c"
#include "OS_def.h"		/*
				 * Os specific definitions 
				 */
#include "MY_alloc.h"		/*
				 * Redefine allocation routines
				 */

#include "RE.h"
#include "win.h"
#include "prefs.h"

/*
 * Global data
 */
static MrmType class_id;	/* Place to keep class ID */
static MrmType *dummy_class;	/* and class variable. */
static char *db_filename_vec[] =/* Mrm.hierachy file list. */
{
    "Nicola:Resources.uid"
};

/* Motif Global variables */
static Display         *display;               /* Display variable */
static XtAppContext    app_context;            /* application context */
static Widget          toplevel_widget;        /* Root widget ID of application */
static Widget          frame; 
static MrmHierarchy    s_MrmHierarchy;         /* MRM database hierarchy ID */

static int db_filename_num =
(sizeof db_filename_vec / sizeof db_filename_vec[0]);
char *vuit_dummy_ident_value = "VUIT dummy identifier value";

static RE_resource *root; /* Used only during widget creation..*/

static void 
set_something(w, resource, value)
    Widget w;
    char *resource, *value;
{
    Arg al[1];

    XtSetArg(al[0], resource, value);
    XtSetValues(w, al, 1);
}


XmString make_xmstring (char *text)
{
long status, bc;
return DXmCvtFCtoCS(text, &bc, &status);
}

typedef struct WIDGET_LIST
{
Widget w;
RE_resource_entry *re;
struct WIDGET_LIST *next;
}WIDGET_LIST;

static WIDGET_LIST *widl=0;

RE_resource_entry *find_re_from_widget(Widget w)
{
WIDGET_LIST *wl;
wl=widl;
while(wl)
{
if(wl->w == w)
return (wl->re);
wl = wl->next;
}
return (0);
}

void associate_widget(Widget w, RE_resource_entry *re)
{
WIDGET_LIST *wl=calloc(1,sizeof(*wl));
wl->w = w;
wl->re = re;
wl->next = widl;
widl=wl;
}

void process_re(RE_resource_entry *re)
{
    if (re->process)
	(*re->process) (re, 0);

    RE_update_item (re, FALSE);
}
/*
 * Given a new resource structure, allocate and create a resource database
 * entry.
 */
static RE_resource_entry *create_resource (Widget w, long size, long type, char *name)
{
    RE_resource_entry *re;

    re = FW_calloc (root->window, 1, sizeof (*re));

if(!re)
return (0);

    re->name = name;
    re->g = w;

    re->read =  FW_calloc(root->window,1,size);
    re->write = re->read;

    re->last_value = FW_calloc (root->window, 1, size);
    re->next = NULL;
    re->item_size = size;
    re->type = type;

/*
    re->key_shortcut = tolower (key_shortcut);
    re->redisplay = FALSE;
    re->num_lines = num_lines;
    re->num_columns = num_columns;
    re->cur_pos = -1;
    re->vertical = vertical;
    re->id = id;
*/
    re->root = root;

    if(!root->first_resource)
	root->first_resource = re;
    else
    {
	re->next = root->first_resource;
	root->first_resource = re;
    }
    associate_widget(w,re);
    return (re);
}

/*
 * Simplified GET VALUE routine to use only when retrieving a single
 * attribute. If we need to retrieve more than one, all values should be put
 * into one arglist and we should make one XtGetValues call (which is MUCH
 * more efficient).
 */

static void 
get_something(w, resource, value)
    Widget w;
    char *resource, *value;
{
    Arg al[1];

    XtSetArg(al[0], resource, value);
    XtGetValues(w, al, 1);
}

void 
BU_create(Widget w,
	  char *name,
	  unsigned long *reason)
{
RE_resource_entry *re=create_resource (w, sizeof(long), BUTTON_KIND, name);
}

void 
BX_create(Widget w,
	  char *name,
	  unsigned long *reason)
{
RE_resource_entry *re=create_resource (w, sizeof(long), CHECKBOX_KIND, name);
}

void 
CO_create(Widget w,
	  char *name,
	  unsigned long *reason)
{
RE_resource_entry *re=create_resource (w, sizeof(long), PALETTE_KIND, name);
}

void 
LV_create(Widget w,
	  char *name,
	  unsigned long *reason)
{
RE_resource_entry *re=create_resource (w, sizeof(long), LISTVIEW_KIND, name);
}

void 
NI_create(Widget w,
	  char *name,
	  unsigned long *reason)
{
RE_resource_entry *re=create_resource (w, sizeof(long), INTEGER_KIND, name);
}

void 
SL_create(Widget w,
	  char *name,
	  unsigned long *reason)
{
RE_resource_entry *re=create_resource (w, sizeof(long), SLIDER_KIND, name);
}

void 
TE_create(Widget w,
	  char *name,
	  unsigned long *reason)
{
RE_resource_entry *re=create_resource (w, sizeof(long), TEXT_KIND, name);
}

void 
SI_create(Widget w,
	  char *name,
	  unsigned long *reason)
{
RE_resource_entry *re=create_resource (w, XmTextFieldGetMaxLength(w)+4, STRING_KIND, name);
}

static Widget sel_widget_list[100];
static int sel_widget_ind = 0;
static RE_resource_entry *cur_re;

void 
cy_item_create(Widget w,
	  char *name,
	  unsigned long *reason)
{
if (sel_widget_ind < 100)
sel_widget_list[sel_widget_ind++] = w;
}

void 
mx_item_create(Widget w,
	  char *name,
	  unsigned long *reason)
{
associate_widget(w, cur_re);
}

void 
MX_create(Widget w,
	  char *name,
	  unsigned long *reason)
{
long i;
RE_resource_entry *re=create_resource (w, sizeof(long), MX_KIND, name);
cur_re = re;
}

CY_create(Widget w,
	  char *name,
	  unsigned long *reason)
{
long i;
RE_resource_entry *re=create_resource (w, sizeof(long), CYCLE_KIND, name);
for (i=0;i<sel_widget_ind;i++)
associate_widget(sel_widget_list[i],re);
sel_widget_ind=0;
}

void 
BX_process(w, tag, toggle)
    Widget w;
    int *tag;
    XmToggleButtonCallbackStruct *toggle;
{
    RE_resource_entry *re = find_re_from_widget(w);
if(!re)
{
printf ("Can't locate widget %x\n",w);
return;
}
    if (toggle->set)
	RE_set_value (re, TRUE);
    else
	RE_set_value (re, FALSE);

    if (re_debug)
    {
	printf ("Checkbox '%s' %s selected (%d)\n",
		 re->name, toggle->set ? "is" : "is not", toggle->set);
    }
process_re(re);
}

void 
LV_process(w, tag, list)
    Widget w;
    int *tag;
    XmListCallbackStruct *list;
{
    RE_resource_entry *re = find_re_from_widget(w);
if(!re)
{
printf ("Can't locate widget %x\n",w);
return;
}
    if (re_debug)
    {
	printf ("Listbox %s, value %d\n", re->name, list->item_position);
    }
    RE_set_value (re, list->item_position);

process_re(re);
}

void 
CO_process(w, tag, list)
    Widget w;
    int *tag;
    void *list;
{
    RE_resource_entry *re = find_re_from_widget(w);
if(!re)
{
printf ("Can't locate widget %x\n",w);
return;
}
process_re(re);
}

void 
SL_process(w, tag, scale)
    Widget w;
    int *tag;
    XmScaleCallbackStruct *scale;
{
RE_resource_entry *re = find_re_from_widget(w);
if(!re)
{
printf ("Can't locate widget %x\n",w);
return;
}
    RE_set_value (re, scale->value);
    if (re_debug)
    printf("Scale (%s) %d\n", re->name, scale->value);
process_re(re);
}

void 
BU_process(w, tag, reason)
    Widget w;
    int *tag;
    char *reason;
{
RE_resource_entry *re = find_re_from_widget(w);
if(!re)
{
printf ("Can't locate widget %x\n",w);
return;
}
    if (re_debug)
	printf ("Button pushed '%s'.\n", re->name);
process_re(re);
}

void 
SI_process(w, tag, reason)
    Widget w;
    int *tag;
    char *reason;
{
    RE_resource_entry *re = find_re_from_widget(w);

if(!re)
{
printf ("Can't locate widget %x\n",w);
return;
}
    strcpy (re->write, XmTextFieldGetString(w));
    strcpy (re->last_value, re->write);
    if (re_debug)
    printf("String changed (%x) %s\n", w, XmTextFieldGetString(w));
process_re(re);
}

void 
NI_process(w, tag, reason)
    Widget w;
    int *tag;
    char *reason;
{
    RE_resource_entry *re = find_re_from_widget(w);
    char *buf, *ptr, *optr, err = 0;
    TY_int slen;

if(!re)
{
printf ("Can't locate widget %x\n",w);
return;
}
    buf = XmTextFieldGetString(w);
    slen = strlen(buf);

    if (slen >0 && *buf)
    {
	optr = ptr = buf;
	while (*ptr && slen--)
	{
	    if (isdigit (*ptr) || (*ptr == '.'))
	    {
		*optr++ = *ptr++;
	    }
	    else
	    {
		err = 1;
		*ptr++;
	    }
	}

	if (!err)
	{
	    TY_int new_value;
	    *ptr=0;
	    new_value = atol (buf); 
	    RE_set_value (re, new_value);
	    memcpy (re->last_value, re->write, re->item_size);
    if (re_debug)
	    printf("Number %s changed to %d\n", re->name, new_value);
	}
    }
    process_re(re);
}

void 
CY_process(w, tag, reason)
    Widget w;
    int *tag;
    char *reason;
{
RE_resource_entry *re = find_re_from_widget(w);

if(!re)
{
printf ("Can't locate widget %x\n",w);
return;
}

RE_set_value (re, *tag);
    if (re_debug)
    printf("CY %s value %d\n",re->name,*tag);
process_re(re);
}

void 
MX_process(w, tag, reason)
    Widget w;
    int *tag;
    char *reason;
{
RE_resource_entry *re = find_re_from_widget(w);

if(!re)
{
printf ("Can't locate widget %x\n",w);
return;
}
    RE_set_value (re, *tag);
    if (re_debug)
    printf("MX %s value %d\n",re->name,*tag);
process_re(re);
}


static MrmRegisterArg reglist[] = {
    {"BU_process", (caddr_t) BU_process},
    {"BU_create", (caddr_t) BU_create},
    {"BX_process", (caddr_t) BX_process},
    {"BX_create", (caddr_t) BX_create},
    {"CO_process", (caddr_t) CO_process},
    {"CO_create", (caddr_t) CO_create},
    {"CY_process", (caddr_t) CY_process},
    {"CY_create", (caddr_t) CY_create},
    {"CY_item_create", (caddr_t) cy_item_create},
    {"LV_process", (caddr_t) LV_process},
    {"LV_create", (caddr_t) LV_create},
    {"MX_process", (caddr_t) MX_process},
    {"MX_create", (caddr_t) MX_create},
    {"MX_item_create", (caddr_t) mx_item_create},
    {"NI_process", (caddr_t) NI_process},
    {"NI_create", (caddr_t) NI_create},
    {"SI_process", (caddr_t) SI_process},
    {"SI_create", (caddr_t) SI_create},
    {"SL_process", (caddr_t) SL_process},
    {"SL_create", (caddr_t) SL_create},
{"TE_create", (caddr_t) TE_create}};

static int reglist_num = (sizeof reglist / sizeof reglist[0]);

/*
 * This is called during startup and reads in the main resource file.
 * There is only one file, which is divided up into individual resource sections.
 */
TY_int RE_init_resource (char *filename, unsigned int argc, char *argv[])
{
    Arg arglist[2];
    int n;

    MrmInitialize();		/* Initialize MRM before initializing */
    /* the X Toolkit. */
    DXmInitialize();		/* Initialize additional DEC widgets */

    /*
     * Initialize the X Toolkit. We get back a top level shell widget.
     */
    XtToolkitInitialize();

    app_context = XtCreateApplicationContext();
    display = XtOpenDisplay(app_context, NULL, argv[0], "example",
			    NULL, 0, &argc, argv);
    if (display == NULL)
    {
	fprintf(stderr, "%s:  Can't open display\n", argv[0]);
	exit(1);
    }
    n = 0;
    XtSetArg(arglist[n], XmNallowShellResize, True);
    n++;
    toplevel_widget = XtAppCreateShell(argv[0], NULL, applicationShellWidgetClass,
				       display, arglist, n);

    /*
     * Open the UID files (the output of the UIL compiler) in the hierarchy
     */
    if (MrmOpenHierarchy(db_filename_num,	/* Number of files. */
			 db_filename_vec,	/* Array of file names.  */
			 NULL,	/* Default OS extenstion. */
			 &s_MrmHierarchy)	/* Pointer to returned MRM ID */
	!= MrmSUCCESS)
	error_log("can't open hierarchy");

    MrmRegisterNames(reglist, reglist_num);

    n = 0;
    XtSetArg(arglist[n], XmNwidth, 100);    n++;
    XtSetArg(arglist[n], XmNheight, 100);    n++;

    frame = XmCreateMainWindow (toplevel_widget, "_root_", 0, 0);
if(!frame)
error_log("Failed to create frame");
}

/*
 * Scan for the resource given, and create a resource structure for it. This can be
 * called as often as required, for each window of this type (usually).
 *
 * It is the resposibility of the window opening routine to open a window of the correct size,
 * and pass a pointer to here.
 * The routine RE_get_sizes may be used to find the X and Y size to use.
 */
RE_resource *RE_create_resources (ZPL_WINDOW * fw, char *resource_name)
{
    char *reptr, *ptr,*uil_name;
    MD_GADGET_P g;
    RE_new_gadget *nr;
    RE_resource *r;
    RE_resource_entry *re, *last_re;
    TY_int done,narg;
    Arg arglist[2];
    struct TextFont *font;
    static struct TextAttr *rf;
Widget id, par;

    if (!resource_name)
	return (NULL);

    r = FW_calloc (fw, 1, sizeof (*r));

    r->name = FW_savestr (fw, resource_name);
uil_name = savestr(resource_name);
gsub(uil_name,' ','_');
gsub(uil_name,'.','_');

    rf = PR_get_re_font ();

    if (rf && rf->ta_Name && *rf->ta_Name)
    {
	r->textattr = rf;
    }
    r->window = fw;

root=r;
id=0;
class_id = 0;
    narg = 0;
/*    XtSetArg(arglist[narg], XmNallowShellResize, True);     narg++;*/

printf ("Created %s\n",resource_name);
#ifdef AAA
    par = XmCreateDialogShell (frame, resource_name, arglist, narg);
if(!par)
{
error_log("Unable to create top level shell\n");
return 0;
}
XtManageChild(par);
#endif
done = MrmFetchWidget(s_MrmHierarchy, resource_name, frame, &id,&class_id);
if(done == MrmSUCCESS)
{
    XtManageChild(frame);
    XtManageChild(id);
    XtRealizeWidget(toplevel_widget);
}
else
{
free(r);
r=0;
error_log("Couldn't find resource %s %d\n",uil_name,done);
}
free(uil_name);
    return (r);
}

/*
 * Release memory used by a resource which is no longer in use
 */
TY_int RE_dispose (RE_resource * r)
{
    return (0);
}

void RE_add_resources (RE_resource * r, MD_WINDOW_P win)
{
}

TY_int RE_set_scrollbar (RE_resource_entry * re, TY_int top, TY_int total, TY_int win_size)
{
    if (re && re->g)
    {
XmScrollBarSetValues(re->g, top, win_size, 1, win_size-1, 0);
set_something(re->g, XmNmaximum, total);
    }
    return (1);
}

RE_resource *RE_add_scrollbar (RE_resource * r, MD_WINDOW_P win, ZPL_WINDOW *fw)
{
}

/*
 * Frees an entiry resource structure, and the data items associated with it.
 * Call this after any 'update_lists' (or anything else using the data items)
 * have been destroyed
 */
TY_int RE_free_resource (RE_resource * resource)
{
    RE_resource_entry *re, *nre;
    if (!resource)
	return (FALSE);

    re = resource->first_resource;

    while (re)
    {
	nre = re->next;
	free_resource_entry (re);
	re = nre;
    }
    return (TRUE);
}

void RE_delete_resources (RE_resource * r, MD_WINDOW_P win)
{
#ifdef AMIGA
    FW_free (r->window, r);
#endif
}

void RE_change_gadget_text (RE_resource_entry *re, char *newtext)
{
}

/*
 * From a resource structure, find the given resource entry. This is usually used to
 * find the root Glist, or to get the individual data item adresseses.
 * --exact matches only, please--
 */
RE_resource_entry *RE_find_resource (char *name, RE_resource * resource)
{
    RE_resource_entry *re;
    if (!resource)
    {
	error_log ("RE_find_resource(): Given NULL resource to search");
	return (FALSE);
    }
    re = resource->first_resource;
    if (!re)
	error_log ("RE_find_resource(): No resource entries");
    while (re)
    {
	if (!strcmp (name, re->name))
	    return (re);
	re = re->next;
    }
    return (NULL);
}

RE_resource_entry *RE_locate_resource_id (int id, RE_resource * resource)
{
    RE_resource_entry *re;
    if (!resource)
    {
	error_log ("RE_find_resource(): Given NULL resource to search");
	return (FALSE);
    }
    re = resource->first_resource;
    if (!re)
	error_log ("RE_find_resource(): No resource entries");
    while (re)
    {
	if (re->id == id)
	    return (re);
	re = re->next;
    }
    return (NULL);
}

void RE_set_process (RE_resource_entry * re, void (*process) (), void *param)
{
    if (re)
    {
	re->process = process;
	re->process_param = param;
    }
}

void *RE_remove_lv (RE_resource_entry * re)
{
XmListDeleteAllItems(re->g);
    return (re->read);
}

void RE_add_lv (RE_resource_entry * re, void *ptr)
{
    struct Node *node;
    struct List *lis = (struct List *) ptr;

    re->size = NG_lv_size (ptr);

    if (re->size < re->cur_pos)
	re->cur_pos = re->size;

    RE_remove_lv(re);
    if (lis && lis->lh_Head)
    {
	node = lis->lh_Head;
	while (node->ln_Succ)
	{
	    XmListAddItem(re->g, make_xmstring(node->ln_Name), 0);
	    node = node->ln_Succ;
	}
    }
    if(re->cur_pos <= re->size)
	RE_select_lv (re, re->cur_pos);
    else
	RE_select_lv (re, 0);
    re->read = ptr;
}

void RE_select_lv (RE_resource_entry * re, TY_int index)
{
    re->cur_pos = index;
    RE_set_value (re, re->cur_pos);
    XmListSelectPos(re->g, index, 0);
}

void RE_set_top_lv (RE_resource_entry * re, TY_int index)
{
    re->cur_pos = index;
    XmListSetPos(re->g, index);
}

void RE_scroll_lv (RE_resource_entry * re, TY_int up)
{
    if (re->type == LISTVIEW_KIND)
    {
	if (up && re->cur_pos > 0)
	{
	    RE_set_top_lv (re, re->cur_pos - 1);
	    RE_select_lv (re, re->cur_pos);
	}
	else if (!up && re->cur_pos < (re->size - 1))
	{
	    RE_set_top_lv (re, re->cur_pos + 1);
	    RE_select_lv (re, re->cur_pos);
	}
    }
}

void RE_page_scroll_lv (RE_resource_entry * re, TY_int up)
{
    if (re->type == LISTVIEW_KIND)
    {
	TY_int nl = re->num_lines - 1;
	if (up)
	{
	    if (re->cur_pos - nl >= 0)
		RE_set_top_lv (re, re->cur_pos - nl);
	    else
		RE_set_top_lv (re, 0);
	    RE_select_lv (re, re->cur_pos);
	}
	else
	{
	    if (re->cur_pos < re->size - nl)
		RE_set_top_lv (re, re->cur_pos + nl);
	    else
		RE_set_top_lv (re, re->size - 1);
	    RE_select_lv (re, re->cur_pos);
	}
    }
}

void *RE_detach_lv (RE_resource_entry * re)
{
    return (re->read);
}

void RE_reattach_lv (RE_resource_entry * re)
{
}

TY_int RE_get_value (RE_resource_entry * re)
{
    switch (re->item_size)
    {
    case 1:
	return (TY_int) (*(char *) re->write);
    case 2:
	return (TY_int) (*(unsigned short *) re->write);
    default:
	return (*(TY_int *) re->write);
    }
}

TY_int RE_get_named_value (char *name, RE_resource * r)
{
    RE_resource_entry *re = RE_find_resource (name, r);
    if (re)
    {
	return (RE_get_value (re));
    }
    return (0);
}

void RE_set_value (RE_resource_entry * re, TY_int value)
{
    switch (re->item_size)
    {
    case 1:
	(*(char *) re->write) = (char) value;
	break;
    case 2:
	(*(unsigned short *) re->write) = (unsigned short) value;
	break;
    default:
	(*(TY_int *) re->write) = value;
    }
}

void RE_set_named_value (char *name, TY_int value, RE_resource * r)
{
    RE_resource_entry *re = RE_find_resource (name, r);
    if (re)
    {
	RE_set_value (re, value);
    }
}

void RE_update_item (RE_resource_entry * re, TY_int force)
{
    unsigned TY_int *iptr = re->write;

    if (re->update_in_progress)
	return;
    if (force || memcmp (re->write, re->last_value, re->item_size))
    {
	re->update_in_progress = 1;
	switch (re->type)
	{
	case INTEGER_KIND:
{
char buf[30];
		sprintf (buf, "%ld", RE_get_value (re));
		XmTextFieldSetString(re->g, buf);
}
	    break;

	case TEXT_KIND:
	case STRING_KIND:
		XmTextFieldSetString(re->g, re->read);
	    break;

	case CHECKBOX_KIND:
		XmToggleButtonSetState(re->g, RE_get_value (re), 0);
	    break;

	case CYCLE_KIND:
/*RE_get_value (re)*/
	    break;

	case PALETTE_KIND:
	    break;

	case MX_KIND:
	    break;

	case SLIDER_KIND:
XmScaleSetValue(re->g,RE_get_value (re));
	    break;

	case SCROLLER_KIND:
	    break;

	case LISTVIEW_KIND:
	    break;
	}
	memcpy (re->last_value, re->write, re->item_size);
	re->update_in_progress = 0;
    }
}

void RE_set_item (RE_resource * r, char *name, void *ptr, TY_int size)
{
    RE_resource_entry *re = RE_find_resource (name, r);
    if (!re)
	return;

    re->read = re->write = ptr;
    re->item_size = size;
    FW_free (r->window, re->last_value);
    re->last_value = FW_calloc (r->window, 1, re->item_size);
    RE_update_item (re, TRUE);
}

void RE_update (RE_resource * resource)
{
    RE_resource_entry *re;
    if (!resource)
    {
	error_log ("RE_update(): Given NULL resource to update");
	return;
    }
    re = resource->first_resource;
    while (re)
    {
	RE_update_item (re, FALSE);
	re = re->next;
    }
}

void RE_set_max (RE_resource_entry * re, TY_int max_val)
{
set_something(re->g, XmNmaximum, max_val);
}

void RE_disable_gadget (RE_resource_entry * re)
{
}

void RE_enable_gadget (RE_resource_entry * re)
{
}

process_events()
{
XEvent event;
/*if(XtAppPending(app_context))*/
{
XtAppNextEvent(app_context, &event);
XtDispatchEvent(&event);
}

}

