/*---------------------------------------------------------------------------
 *
 *      Title:          NICOLA - Resource Database Gadget Creation
 *
 *      File:           RE_ng.c
 *
 *      Date:           8 June 1992
 *
 *      Description:
 *              This module is responsible for creating new gadgets. It also
 *      provides routines for ListView handling.
 *
 *      Copyright  1993 R.J.Harrison
 *
 *      Rev:    By              Done                                    Date
 *      1.00    R.J.Harrison    Implement                            8  June  92
 *
 *---------------------------------------------------------------------------*/


#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#ifdef AMIGA
#include <exec/types.h>
#include <exec/memory.h>
#include <exec/libraries.h>
#include <intuition/intuition.h>
#include <intuition/gadgetclass.h>
#include <libraries/gadtools.h>
#include <graphics/displayinfo.h>

#ifdef NO_INLINE
#include <clib/exec_protos.h>
#include <clib/intuition_protos.h>
#include <clib/gadtools_protos.h>
#else
#include <proto/exec.h>
#include <proto/intuition.h>
#include <proto/gadtools.h>
#endif
#endif

#include "zpl.h"

#ifdef AMIGA
#include "intuisup.h"
#endif

/*
 * Scaling factors.
 */
TY_int x_offset = 0, y_offset = 0, FontX = 8, FontY = 8, re_debug;

extern TY_int RE_allow_scale_y, using_v36 ();

#define SCALE_Y

static void do_cursor (RE_new_gadget * nr)
{
    char *e = strchr (nr->ng->ng_GadgetText, '|');

    if (e)
    {
	if (!e[1])
	    *e = 0;
	else
	    strcpy (e, e + 1);
	nr->key_shortcut = RE_scroll_lv_id;
    }
}

static TY_int do_underscore (RE_new_gadget * nr)
{
#ifdef WINDOWS
    char shortcut='&';
#else
    char shortcut='_';
#endif
    char *p = strchr (nr->ng->ng_GadgetText, shortcut);

    if (!p)
	return (NULL);
    nr->key_shortcut = p[1];
    return (shortcut);
}

#ifdef AMIGA
struct GadgetData *make_gadget_data (MD_NEWGADGET_P ng, unsigned type)
{
    struct GadgetData *gd = calloc (1, sizeof (*gd));

    if (gd)
    {
	gd->gd_Flags = 0;

	if (ng->ng_GadgetText)
	{
	    char *e = strchr (ng->ng_GadgetText, '|');

	    if (e)
	    {
		strcpy (e, e + 1);
	    }
	    if (strchr (ng->ng_GadgetText, '_'))
		gd->gd_Flags |= GADGET_DATA_FLAG_HOTKEY;

	}
	gd->gd_Type = type;

	if (ng->ng_Flags & PLACETEXT_LEFT)
	    gd->gd_Flags |= GADGET_DATA_FLAG_TEXT_LEFT;
	if (ng->ng_Flags & PLACETEXT_RIGHT)
	    gd->gd_Flags |= GADGET_DATA_FLAG_TEXT_RIGHT;
	if (ng->ng_Flags & PLACETEXT_ABOVE)
	    gd->gd_Flags |= GADGET_DATA_FLAG_TEXT_ABOVE;
	if (ng->ng_Flags & PLACETEXT_BELOW)
	    gd->gd_Flags |= GADGET_DATA_FLAG_TEXT_BELOW;

	gd->gd_LeftEdge = ng->ng_LeftEdge;
	gd->gd_TopEdge = ng->ng_TopEdge;
	gd->gd_Width = ng->ng_Width;
	gd->gd_Height = ng->ng_Height;
	gd->gd_Text = ng->ng_GadgetText;
	gd->gd_TextAttr = ng->ng_TextAttr;
    }
    return (gd);
}
#endif

/*
 * Add a text descriptive label.
 */

#ifdef WINDOWS
TY_int NG_strlen (char *str)
{
    TY_int count = 0;
    while (str && *str)
    {
	if (*str != '_')
	    count++;
	str++;
    }
    return (count);
}

static void add_label (RE_new_gadget * nr)
{
if(nr->ng->ng_GadgetText && *nr->ng->ng_GadgetText)
{
   MD_GADGET_P g;
    int xp, yp;
int xsize=FW_get_string_xsize(FW_get_dialog_font(),nr->ng->ng_GadgetText);

    if (nr->ng->ng_Flags & PLACETEXT_ABOVE)
    {
	yp = nr->ng->ng_TopEdge - SCFY_rel (1);
	xp = (nr->ng->ng_LeftEdge + nr->ng->ng_Width / 2) - (xsize/2);
    }
    else
    {
	yp = nr->ng->ng_TopEdge;

	if (nr->ng->ng_Flags & PLACETEXT_LEFT)
{
xp = nr->ng->ng_LeftEdge - xsize;
nr->ng->ng_LeftEdge += SCFX_rel(2);
}
	else
	    xp = nr->ng->ng_LeftEdge + nr->ng->ng_Width + SCFX_rel (2);
    }

    g = CreateWindow ("STATIC", nr->ng->ng_GadgetText, WS_CHILD,

		      xp, yp,
		      xsize, SCFY_rel(1), //nr->ng->ng_Height+4,
		      nr->window->w, -1, nr->instance, NULL);
SendMessage(g, WM_SETFONT,
                    (WPARAM) FW_get_dialog_font(), 0);    ShowWindow (g, SW_SHOWNA);
}
}
#endif

/*
 * The gadget specific 'finishing' off new gadget routines.
 */

MD_GADGET_P NG_cycle (MD_GADGET_P g, RE_new_gadget * nr, char **names)
{
    extern TY_int FontY;
    TY_int sf;

    nr->type = CYCLE_KIND;
#ifdef AMIGA
    sf = 0;
    if (FontY < 10)
	sf = 10 - FontY;
    nr->ng->ng_Height += 2 + sf;

    if (using_v36 ())
    {
	g = CreateGadget (CYCLE_KIND, g, nr->ng,
			  GTCY_Labels, names,
			  GT_Underscore, do_underscore (nr),
			  TAG_DONE);
    }
    else
    {
	struct GadgetData *ng;
	ng = make_gadget_data (nr->ng, GADGET_DATA_TYPE_CYCLE);
	ng->gd_SpecialData.gd_CycleData.gd_CycleActive = 0;
	ng->gd_SpecialData.gd_CycleData.gd_CycleTextArray = names;
	g = (MD_GADGET_P) ng;
    }
#ifdef SCALE_Y
    if (RE_allow_scale_y)
	y_offset += 6;
#endif
#endif
#ifdef WINDOWS
    {
	TY_int i, ls = 0,xs, new_height;

xs=FW_get_string_xsize(FW_get_dialog_font(),nr->ng->ng_GadgetText);
nr->ng->ng_Flags |= PLACETEXT_LEFT;
nr->ng->ng_LeftEdge += xs;
	add_label (nr);

	for (i = 0; names[i]; i++)
	{
	    int x = strlen (names[i]);
	    if (x > ls)
		ls = x;
	}
	ls += 5;
	nr->ng->ng_Width = SCFX_rel (ls);
	i = i + 2;
	i = min (i, 12);
	new_height = SCFY_rel (i);

do_underscore (nr);
	g = CreateWindow ("COMBOBOX", NULL, WS_CHILD | CBS_DROPDOWNLIST |	/*
										 * CBS_SORT|
			   */ WS_VSCROLL | WS_TABSTOP,
			  nr->ng->ng_LeftEdge, nr->ng->ng_TopEdge,
			  nr->ng->ng_Width, new_height,
			  nr->window->w, nr->id, nr->instance, NULL);

	for (i = 0; names[i]; i++)
	{
	    LPARAM str;
	    str = (LPARAM) names[i];
	    SendMessage (g, CB_ADDSTRING, 0, str);
	}
	SendMessage (g, CB_SETCURSEL, 0, 0);
    }
#ifdef SCALE_Y
    if (RE_allow_scale_y)
	y_offset += SCFY_rel (1);
#endif
#endif
/*
 * y_offset += sf;
 */
    return (g);
}

MD_GADGET_P NG_mx (MD_GADGET_P g, RE_new_gadget * nr, char **names, TY_int use_underscore, TY_int active)
{
    nr->type = MX_KIND;
#ifdef AMIGA
    if (using_v36 ())
    {
	g = CreateGadget (MX_KIND, g, nr->ng,
			  GTMX_Labels, names,
			  GT_Underscore, do_underscore (nr),
			  GTMX_Active, active,
			  TAG_DONE);
    }
    else
    {
	struct GadgetData *ng;
	ng = make_gadget_data (nr->ng, GADGET_DATA_TYPE_MX);
	ng->gd_Flags |= GADGET_DATA_FLAG_NO_BORDER;
	ng->gd_SpecialData.gd_MXData.gd_MXSpacing = 1;
	ng->gd_SpecialData.gd_MXData.gd_MXActiveEntry = active;
	ng->gd_SpecialData.gd_MXData.gd_MXTextArray = names;
	ng->gd_Text = "";
	g = (MD_GADGET_P) ng;
    }

#ifdef SCALE_Y
    {
	char **c = names;
	TY_int n = 0;
	while (*c++)
	    n++;
	y_offset += SCFY_rel (n + 1);
    }
#endif
#endif
#ifdef WINDOWS
    {
	TY_int i, ls = 0,xs, new_height;

xs=FW_get_string_xsize(FW_get_dialog_font(),nr->ng->ng_GadgetText);
nr->ng->ng_Flags |= PLACETEXT_LEFT;
nr->ng->ng_LeftEdge += xs;
	add_label (nr);

	for (i = 0; names[i]; i++)
	{
	    int x = strlen (names[i]);
	    if (x > ls)
		ls = x;
	}
	ls += 5;
	nr->ng->ng_Width = SCFX_rel (ls);
	i = i + 2;
	i = min (i, 12);
	new_height = SCFY_rel (i);

do_underscore (nr);
	g = CreateWindow ("COMBOBOX", NULL, WS_CHILD | CBS_DROPDOWNLIST |	/*
										 * CBS_SORT|
			   */ WS_VSCROLL | WS_TABSTOP,
			  nr->ng->ng_LeftEdge, nr->ng->ng_TopEdge,
			  nr->ng->ng_Width, new_height,
			  nr->window->w, nr->id, nr->instance, NULL);

	for (i = 0; names[i]; i++)
	{
	    LPARAM str;
	    str = (LPARAM) names[i];
	    SendMessage (g, CB_ADDSTRING, 0, str);
	}
	SendMessage (g, CB_SETCURSEL, 0, 0);
    }
#ifdef SCALE_Y
    if (RE_allow_scale_y)
	y_offset += SCFY_rel (1);
#endif
#endif
    return (g);
}

MD_GADGET_P NG_box (MD_GADGET_P g, RE_new_gadget * nr, TY_int on, TY_int us)
{
    nr->type = CHECKBOX_KIND;
#ifdef AMIGA
    nr->ng->ng_Height += 2;

    if (using_v36 ())
    {
	g = CreateGadget (CHECKBOX_KIND, g, nr->ng,
			  GTCB_Checked, on,
			  GT_Underscore, do_underscore (nr),
			  TAG_DONE);
    }
    else
    {
	struct GadgetData *ng;
	ng = make_gadget_data (nr->ng, GADGET_DATA_TYPE_CHECK);
	ng->gd_SpecialData.gd_CheckData.gd_CheckSelected = on;
	g = (MD_GADGET_P) ng;
    }
#endif
#ifdef WINDOWS
do_underscore (nr);
    g = CreateWindow ("BUTTON", nr->ng->ng_GadgetText, WS_CHILD | BS_AUTOCHECKBOX,
		      nr->ng->ng_LeftEdge, nr->ng->ng_TopEdge,
		      nr->ng->ng_Width, nr->ng->ng_Height,
		      nr->window->w, nr->id, nr->instance, NULL);
#endif
#ifdef SCALE_Y
    if (RE_allow_scale_y)
	y_offset += 6;
#endif
    return (g);
}

MD_GADGET_P NG_button (MD_GADGET_P g, RE_new_gadget * nr)
{
    nr->type = BUTTON_KIND;
#ifdef AMIGA
    nr->ng->ng_Height += 2;
    if (using_v36 ())
    {
	g = CreateGadget (BUTTON_KIND, g, nr->ng, GT_Underscore, do_underscore (nr), TAG_DONE);
    }
    else
    {
	struct GadgetData *ng;
	ng = make_gadget_data (nr->ng, GADGET_DATA_TYPE_BUTTON);
	g = (MD_GADGET_P) ng;
    }
#endif
#ifdef WINDOWS
do_underscore (nr);
    g = CreateWindow ("BUTTON", nr->ng->ng_GadgetText, WS_CHILD | WS_TABSTOP,
		      nr->ng->ng_LeftEdge, nr->ng->ng_TopEdge,
		      nr->ng->ng_Width, nr->ng->ng_Height+5,
		      nr->window->w, nr->id, nr->instance, NULL);
    if (RE_allow_scale_y)
	y_offset += 5;
#endif
#ifdef SCALE_Y
    if (RE_allow_scale_y)
	y_offset += 6;
#endif
    return (g);
}

MD_GADGET_P NG_numeric (MD_GADGET_P g, RE_new_gadget * nr, TY_int maxlen, TY_int output)
{
    if (maxlen <= 0)
	maxlen = 4;
    nr->type = INTEGER_KIND;
#ifdef AMIGA

    if (using_v36 ())
    {
	g = CreateGadget (INTEGER_KIND, g, nr->ng, GTIN_Number, output,
			  GT_Underscore, do_underscore (nr),
			  GTIN_MaxChars, maxlen, TAG_DONE);
    }
    else
    {
	struct GadgetData *ng;
	ng = make_gadget_data (nr->ng, GADGET_DATA_TYPE_INTEGER);
	ng->gd_SpecialData.gd_InputData.gd_InputLen = maxlen + 1;
	ng->gd_SpecialData.gd_InputData.gd_InputDefault = (BYTE *) output;
	g = (MD_GADGET_P) ng;
    }
#endif
#ifdef WINDOWS
// Label Text...
    add_label (nr);

do_underscore (nr);
nr->ng->ng_Height += 4;
   g = CreateWindow ("EDIT", NULL,
		      WS_CHILD | ES_AUTOHSCROLL | WS_BORDER | WS_TABSTOP,
		      nr->ng->ng_LeftEdge, nr->ng->ng_TopEdge,
		      nr->ng->ng_Width, nr->ng->ng_Height,
		      nr->window->w, nr->id, nr->instance, NULL);
    if (RE_allow_scale_y)
	y_offset += 4;
#endif

#ifdef SCALE_Y
    if (RE_allow_scale_y)
	y_offset += 8;
#endif
    return (g);
}
MD_GADGET_P NG_string (MD_GADGET_P g, RE_new_gadget * nr, TY_int maxlen, char *output)
{
    nr->type = STRING_KIND;
#ifdef AMIGA

    if (using_v36 ())
    {
	extern struct Library *GadToolsBase;
	g = CreateGadget (STRING_KIND, g, nr->ng, GTST_MaxChars, maxlen,
			  GT_Underscore, do_underscore (nr),
			  GA_Immediate, TRUE,
			  TAG_DONE);

	if (GadToolsBase->lib_Version == 37 && g)
	{
	    g->Activation |= GACT_IMMEDIATE;
	}

    }
    else
    {
	struct GadgetData *ng;
	ng = make_gadget_data (nr->ng, GADGET_DATA_TYPE_STRING);
	ng->gd_SpecialData.gd_InputData.gd_InputLen = maxlen + 1;
	ng->gd_SpecialData.gd_InputData.gd_InputDefault = output;
	g = (MD_GADGET_P) ng;
    }
#endif
#ifdef WINDOWS
// Label Text...
    add_label (nr);

do_underscore (nr);
nr->ng->ng_Height += 4;
    g = CreateWindow ("EDIT", NULL,
		      WS_CHILD | ES_AUTOHSCROLL | WS_BORDER | WS_TABSTOP,
		      nr->ng->ng_LeftEdge, nr->ng->ng_TopEdge,
		      nr->ng->ng_Width, nr->ng->ng_Height,
		      nr->window->w, nr->id, nr->instance, NULL);
    if (RE_allow_scale_y)
	y_offset += 4;
#endif
#ifdef SCALE_Y
    if (RE_allow_scale_y)
	y_offset += 8;
#endif
    return (g);
}

MD_GADGET_P NG_lv (MD_GADGET_P g, RE_new_gadget * nr, struct List * lis, TY_int add_sg)
{
    nr->type = LISTVIEW_KIND;

#ifdef AMIGA
    if (using_v36 ())
    {
int read_only = add_sg == 2;
	do_cursor (nr);
	nr->ng->ng_TopEdge += 4;
	if ((add_sg == 1) || using_v38())
	    g = CreateGadget (LISTVIEW_KIND, g, nr->ng,
			      GTLV_ReadOnly, read_only,
			      GTLV_Labels, lis, GTLV_Top, 0,
		     LAYOUTA_SPACING, 1, GTLV_ShowSelected, NULL, TAG_DONE);
	else
	    g = CreateGadget (LISTVIEW_KIND, g, nr->ng, GTLV_Labels, lis, GTLV_Top, 0,
			      GTLV_ReadOnly, read_only,
			      LAYOUTA_SPACING, 1, TAG_DONE);
    }
    else
    {
	struct GadgetData *ng;
	ng = make_gadget_data (nr->ng, GADGET_DATA_TYPE_LISTVIEW);
	if (add_sg == 1)
	    ng->gd_Flags |= GADGET_DATA_FLAG_LISTVIEW_SHOW_SELECTED;
	ng->gd_SpecialData.gd_ListViewData.gd_ListViewSpacing = 1;
	ng->gd_SpecialData.gd_ListViewData.gd_ListViewTop = 0;
	ng->gd_SpecialData.gd_ListViewData.gd_ListViewList = lis;
	g = (MD_GADGET_P) ng;
    }
#endif
#ifdef WINDOWS
    do_cursor (nr);
    add_label (nr);

do_underscore (nr);
    g = CreateWindow ("LISTBOX", nr->ng->ng_GadgetText,
		      WS_VSCROLL | WS_BORDER | WS_CHILD | LBS_NOTIFY |	/*
									 * LBS_DISABLENOSCROLL|
		       */ WS_TABSTOP,
		      nr->ng->ng_LeftEdge, nr->ng->ng_TopEdge,
		      nr->ng->ng_Width, nr->ng->ng_Height,
		      nr->window->w, nr->id, nr->instance, NULL);
#endif
#ifdef SCALE_Y
    if (RE_allow_scale_y)
	y_offset += nr->ng->ng_Height + SCFY_rel (1);
/*
 * y_offset += 12;
 */
#endif
    return (g);
}


/*
 * Scrollbar routines
 */
MD_GADGET_P NG_sb (MD_GADGET_P g, RE_new_gadget * nr, TY_int vis, TY_int arrow, TY_int vertical, TY_int rv)
{
    nr->type = SCROLLER_KIND;
    nr->vertical = vertical;
#ifdef AMIGA
    if (using_v36 ())
    {
	g = CreateGadget (SCROLLER_KIND, g, nr->ng, GTSC_Visible, vis,
			  GTSC_Arrows, arrow, PGA_Freedom, vertical ? LORIENT_VERT : LORIENT_HORIZ,
			  GA_RelVerify, rv, TAG_DONE);
    }
    else
    {
	struct GadgetData *ng;

	ng = make_gadget_data (nr->ng, GADGET_DATA_TYPE_SCROLLER);

	ng->gd_Flags |= GADGET_DATA_FLAG_TEXT_ABOVE;
	if (vertical)
	    ng->gd_Flags |= GADGET_DATA_FLAG_ORIENTATION_VERT;

	ng->gd_SpecialData.gd_ScrollerData.gd_ScrollerVisible = vis;
	ng->gd_SpecialData.gd_ScrollerData.gd_ScrollerVisible = 1;
	ng->gd_SpecialData.gd_ScrollerData.gd_ScrollerTotal = 1;	/*
									 * total entries 
									 */
	ng->gd_SpecialData.gd_ScrollerData.gd_ScrollerTop = 1;	/*
								 * current top entry 
								 */
	g = (MD_GADGET_P) ng;
    }
#endif
#ifdef WINDOWS
do_underscore (nr);
    g = CreateWindow ("SCROLLBAR", NULL,
		   WS_CHILD | WS_TABSTOP | (vertical ? SBS_VERT : SBS_HORZ),
		      nr->ng->ng_LeftEdge, nr->ng->ng_TopEdge,
		      nr->ng->ng_Width, nr->ng->ng_Height,
		      nr->window->w, nr->id, nr->instance, NULL);
#endif
    return (g);
}

MD_GADGET_P NG_text (MD_GADGET_P g, RE_new_gadget * nr, TY_int box)
{
    nr->type = TEXT_KIND;
#ifdef AMIGA
    if (using_v36 ())
    {
	nr->ng->ng_Height += 2;
	nr->ng->ng_LeftEdge += 2;
	g = CreateGadget (nr->type, g, nr->ng, GTTX_Border, box, TAG_DONE);
    }
    else
    {
	struct GadgetData *ng;
	ng = make_gadget_data (nr->ng, GADGET_DATA_TYPE_BUTTON);
	ng->gd_Flags |= GADGET_DATA_FLAG_NO_BORDER;
	g = (MD_GADGET_P) ng;
    }
#endif
#ifdef WINDOWS
    g = CreateWindow ("STATIC", nr->ng->ng_GadgetText, WS_CHILD,
		      nr->ng->ng_LeftEdge, nr->ng->ng_TopEdge,
		      nr->ng->ng_Width, nr->ng->ng_Height,
		      nr->window->w, nr->id, nr->instance, NULL);
#endif
#ifdef SCALE_Y
    if (RE_allow_scale_y)
	y_offset += 4;
#endif
    return (g);
}

/*
 * Scrollbar routines
 */
MD_GADGET_P NG_slider (MD_GADGET_P g, RE_new_gadget * nr, TY_int min, TY_int max)
{
    nr->type = SLIDER_KIND;
#ifdef AMIGA
    if (using_v36 ())
    {
	short w_min = min, w_max = max;
	g = CreateGadget (SLIDER_KIND, g, nr->ng,
			  GTSL_Min, w_min, GTSL_Max, w_max,
			  GTSL_LevelFormat, "%5ld",
			  GTSL_MaxLevelLen, 5,
			  GA_RelVerify, TRUE,
			  GT_Underscore, do_underscore (nr),
			  GTSL_LevelPlace, PLACETEXT_RIGHT,
			  TAG_DONE);
    }
    else
    {
	struct GadgetData *ng;
	ng = make_gadget_data (nr->ng, GADGET_DATA_TYPE_SLIDER);
/*
 * minimal level 
 */
	ng->gd_SpecialData.gd_SliderData.gd_SliderMin = min;
	ng->gd_SpecialData.gd_SliderData.gd_SliderMax = max;
	g = (MD_GADGET_P) ng;
    }
#endif
#ifdef WINDOWS
    add_label (nr);
do_underscore (nr);
    g = CreateWindow ("SCROLLBAR", NULL,
		      WS_CHILD | SBS_HORZ | WS_TABSTOP,
		      nr->ng->ng_LeftEdge, nr->ng->ng_TopEdge,
		      nr->ng->ng_Width, nr->ng->ng_Height,
		      nr->window->w, nr->id, nr->instance, NULL);
#endif
#ifdef SCALE_Y
    if (RE_allow_scale_y)
	y_offset += 4;
#endif
    return (g);
}

MD_GADGET_P NG_palette (MD_GADGET_P g, RE_new_gadget * nr, TY_int depth, TY_int iw)
{
    nr->type = PALETTE_KIND;
#ifdef AMIGA
    depth = min (depth, 3);
    if (using_v36 ())
    {
	nr->ng->ng_Width = iw + depth * 30;
	g = CreateGadget (PALETTE_KIND, g, nr->ng,
			  GTPA_Depth, depth, GTPA_IndicatorWidth, iw,
			  GT_Underscore, do_underscore (nr),
			  TAG_DONE);
    }
    else
    {
	struct GadgetData *ng;
	nr->ng->ng_TopEdge += FontY + 2;
	nr->ng->ng_Flags = PLACETEXT_ABOVE;
	ng = make_gadget_data (nr->ng, GADGET_DATA_TYPE_PALETTE);
	ng->gd_SpecialData.gd_PaletteData.gd_PaletteDepth = depth;
	ng->gd_Height = max (16, ng->gd_Height);
	y_offset += FontY + 2;
	g = (MD_GADGET_P) ng;
    }
#endif
#ifdef WINDOWS
    {
char *names[] = {"Colour 1","Colour 2","Colour 3","Colour 4","Colour 5","Colour 6","Colour 7","Colour 8",0};
	TY_int i, ls = 0,new_height;

	add_label (nr);

	for (i = 0; names[i]; i++)
	{
	    int x = strlen (names[i]);
	    if (x > ls)
		ls = x;
	}
	ls += 5;
	nr->ng->ng_Width = SCFX_rel (ls);
	i = i + 2;
	i = min (i, 12);
	new_height = SCFY_rel (i);

/*
 * nr->ng->ng_GadgetText 
 */
do_underscore (nr);
	g = CreateWindow ("COMBOBOX", NULL, WS_CHILD | CBS_DROPDOWNLIST		/*
										 * |CBS_SORT
			   */  | WS_VSCROLL | WS_TABSTOP,
			  nr->ng->ng_LeftEdge, nr->ng->ng_TopEdge,
			  nr->ng->ng_Width, new_height,
			  nr->window->w, nr->id, nr->instance, NULL);

	for (i = 0; names[i]; i++)
	{
	    LPARAM str;
	    str = (LPARAM) names[i];
	    SendMessage (g, CB_ADDSTRING, 0, str);
	}
	SendMessage (g, CB_SETCURSEL, 0, 0);
}
#endif
#ifdef SCALE_Y
    if (RE_allow_scale_y)
	y_offset += 8;
#endif
    return (g);
}
