/*---------------------------------------------------------------------------
 *
 *	Title:		NICOLA - Requester Handling
 *
 *	File:		strinp.c
 *
 *	Date:		August 1992
 *
 *	Description:
 *           This module provides the requesters as used for files, palettes etc...
 *
 *      $Log:$
 *
 *	Copyright  1993 R.J.Harrison
 *---------------------------------------------------------------------------*/
static char rcsid[] = "$Id:$";

#include <stdio.h>
#include <string.h>
#include <math.h>

#include <sys/types.h>
#include <sys/stat.h>

#include <libraries/reqtools.h>
#include <proto/reqtools.h>

#include "OS_def.h"		/*
				 * Os specific definitions 
				 */
#include "win.h"
#include "prefs.h"

#include "MY_alloc.h" /* Redefine allocation routines*/

static ULONG wp_tags[] =
{RT_WaitPointer, TRUE, TAG_END};

void convert_attr_to_font (struct TextAttr *attr, TEXT_FONT * tf)
{
    memset (tf, 0, sizeof (*tf));
    strcpy (tf->name, attr->ta_Name);
    tf->y_size = attr->ta_YSize;
    tf->style = attr->ta_Style;
    tf->flags = attr->ta_Flags;
}

void convert_font_to_attr (TEXT_FONT * tf, struct TextAttr *attr)
{
    memset (attr, 0, sizeof (*attr));
    attr->ta_Name = tf->name;
    attr->ta_YSize = tf->y_size;
    attr->ta_Style = tf->style;
    attr->ta_Flags = tf->flags;
}

void get_colour_tv ()
{
    rtPaletteRequest ("Select Colour", NULL, RT_WaitPointer, TRUE, TAG_END);
    save_palette ();
    load_palette ();
}

void tell_em (char *string, char *str1)
{
if(!is_interactive())
{
error_log (string,str1);
return ;
}
    rtEZRequest (string, "Continue", NULL, (struct TagItem *) wp_tags, str1);
}


void *get_font (char *title, TY_int fixed, void * cur_font)
{
    static TEXT_FONT tf, *rv=0;
    struct rtFontRequester *fontreq = NULL;

if(!is_interactive())
{
error_log ("Font selection only applicable in interactive mode");
return NULL;
}

    memset (&tf, 0, sizeof (tf));
    if (fontreq = rtAllocRequestA (RT_FONTREQ, NULL))
    {

	struct TextFont *DefaultFont;
	if (fixed)
	    fontreq->Flags = FREQF_STYLE | FREQF_FIXEDWIDTH;

	if (rtFontRequest (fontreq, title, RT_WaitPointer, TRUE, TAG_END))
	{
            convert_attr_to_font (&fontreq->Attr, &tf);
  	    rv = &tf;
            tell_em ("The font change will only affect new windows",0);
        }
	rtFreeRequest (fontreq);
    }
    else
	rtEZRequest ("Out of memory!", "Oh for virtual memory", NULL, NULL);

    return (rv);
}

TY_int strinput (ZPL_WINDOW * fw, char *curval, char *title, TY_int maxlen)
{
    return ((TY_int) rtGetString (curval, min (70, maxlen), title, NULL,
			       RT_WaitPointer, TRUE,
			       TAG_END));
}


TY_int passinput (ZPL_WINDOW * fw, char *password, char *title, TY_int maxlen, TY_int change)
{
if(rtGetString (password, min (70, maxlen), title, NULL,
			       RT_WaitPointer, TRUE,
RTGS_Invisible,TRUE,
			       TAG_END))
{
if(change)
{
char *second_password=calloc(maxlen,1);
if(rtGetString (second_password, min (70, maxlen), "Enter password again for validation", NULL,
			       RT_WaitPointer, TRUE,
RTGS_Invisible,TRUE,
			       TAG_END))
{
if (strcmp(second_password, password))
{
tell_em("Password not entered the same twice",0);
return 0;
}
return 1;

}
else
return 0;

}
return(1);
}
else
return (0);
}

TY_int ranged_numinput (ZPL_WINDOW * fw, TY_int *curval, char *title, TY_int minval, TY_int maxval)
{
    long longnum = *curval;

    if (!title)
	title = "Numeric input";

if(!is_interactive())
return 0;

if (minval > maxval)
minval = maxval;

    if (rtGetLong (&longnum, title, NULL,
/*		   RTGL_Width, 300,*/
                   RTGL_Min, minval,
                   RTGL_Max, maxval,
		   RT_WaitPointer, TRUE,
		   RTGL_ShowDefault, FALSE, TAG_END))
{
*curval = longnum;
	return (1);
}
return(0);
}

TY_int numinput (ZPL_WINDOW * fw, TY_int *curval, char *title)
{
    long longnum = *curval;

    if (!title)
	title = "Numeric input";

if(!is_interactive())
return 0;

    if (rtGetLong (&longnum, title, NULL,
/*		   RTGL_Width, 300,*/
		   RT_WaitPointer, TRUE,
		   RTGL_ShowDefault, FALSE, TAG_END))
{
*curval = longnum;
	return (1);
}
return(0);
}

void file_error (char *fullfilename)
{
    rtEZRequest ("Cannot open %s", "Sorry", NULL, (struct TagItem *) wp_tags, fullfilename);
}

TY_int do_it (char *string, char *str1)
{
    static ULONG tags[] =
    {RT_Underscore, '_',
     RTEZ_DefaultResponse, 1,
     RT_WaitPointer, TRUE,
     TAG_END, /*RTEZ_ReqTitle, (ULONG)"mytitle"*/ };

if(!is_interactive())
return 1;

    return (TY_int) rtEZRequest (string, "_Yes|_No", NULL,
			      (struct TagItem *) tags, str1);
}

TY_int do_it_n (char *string, char *str1)
{
    static ULONG tags[] =
    {RT_Underscore, '_',
     RTEZ_DefaultResponse, 0,
     RT_WaitPointer, TRUE,
     TAG_END, /*RTEZ_ReqTitle, (ULONG)"mytitle"*/ };

if(!is_interactive())
return 0;

    return (TY_int) rtEZRequest (string, "_Yes|_No", NULL,
			      (struct TagItem *) tags, str1);
}


char *check_new_file (char *filename)
{
    struct stat st;
    static ULONG wp_tags[] =
    {RT_Underscore, '_',
     RTEZ_DefaultResponse, 2,
     RT_WaitPointer, TRUE,
     TAG_END};

    if (stat (filename, &st) == 0)
    {
if(!is_interactive())
return "a";
switch (rtEZRequest ("%s already exists", "_Overwrite|_Append|_Cancel", NULL,
				   (struct TagItem *) wp_tags, filename))
{
case 0:
return 0;
case 1:
return "w";
case 2:
return "a";
default:
return 0;
}
return 0;
    }
    else
	return ("w");
}

TY_int file_exists (char *filename)
{
    struct stat st;

    if (stat (filename, &st))
    {
if(is_interactive())
	rtEZRequest ("Can't read from %s", "Oh Dear|How Sad|Never Mind", NULL, (struct TagItem *) wp_tags, filename);
	return (FALSE);
    }
    else
	return (TRUE);
}

TY_int multi_do_it (char *prompt, char *str1, char *opts)
{
    struct stat st;
    static ULONG tags[] =
    {RT_Underscore, '_',
     RTEZ_DefaultResponse, 0,
     RT_WaitPointer, TRUE,
     TAG_END, /*RTEZ_ReqTitle, (ULONG)"mytitle"*/ };

if(!is_interactive())
return 0;

    return (TY_int) rtEZRequest (prompt, opts, NULL,
			      (struct TagItem *) tags, str1);
}

TY_int multi_do_it_n (char *prompt, char *str1, char *opts)
{
    struct stat st;
    static ULONG tags[] =
    {RT_Underscore, '_',
     RTEZ_DefaultResponse, 1,
     RT_WaitPointer, TRUE,
     TAG_END, /*RTEZ_ReqTitle, (ULONG)"mytitle"*/ };

if(!is_interactive())
return 1;

    return (TY_int) rtEZRequest (prompt, opts, NULL,
			      (struct TagItem *) tags, str1);
}

char *file_request_path (char *prompt, char *path)
{
    char filename[64];
    static char fullfilename[256];
    struct rtFileRequester *filereq = NULL;

if(!is_interactive())
return 0;
    if (filereq = rtAllocRequestA (RT_FILEREQ, NULL))
    {
	if (!path || !*path)
	    path = PR_get_default_path ();
	if (path)
	    rtChangeReqAttr (filereq,
			     RTFI_Dir, path,
			     NULL);
	*filename = 0;
	if (rtFileRequest (filereq, filename, prompt, RT_WaitPointer, TRUE, TAG_END))
	{
	    TY_int n = strlen (filereq->Dir);
	    if (path)
		strcpy (path, filereq->Dir);
	    if (n == 0 || filereq->Dir[n - 1] == ':')
		sprintf (fullfilename, "%s%s", filereq->Dir, filename);
	    else
		sprintf (fullfilename, "%s/%s", filereq->Dir, filename);
	}
	else
	{
	    rtFreeRequest (filereq);
	    return (NULL);
	}
	rtFreeRequest (filereq);
    }
    else
    {
	rtEZRequest ("Out of memory!", "I'll buy some!", NULL, (struct TagItem *) wp_tags);
	return (NULL);
    }

    return (fullfilename);

}

char *file_request_path_save (char *prompt, char *path)
{
return file_request_path (prompt, path);
}

char *file_request_nopath (char *prompt, char *path)
{
    char filename[64];
    static char fullfilename[256];
    struct rtFileRequester *filereq = NULL;

if(!is_interactive())
return 0;

    if (filereq = rtAllocRequestA (RT_FILEREQ, NULL))
    {
	if (!path || !*path)
	    path = PR_get_default_path ();
	rtChangeReqAttr (filereq,
			 RTFI_Dir, path, NULL);
	*filename = 0;
	if (rtFileRequest (filereq, filename, prompt, RT_WaitPointer, TRUE, TAG_END))
	{
	    strcpy (fullfilename, filename);
	}
	else
	{
	    rtFreeRequest (filereq);
	    return (NULL);
	}
	rtFreeRequest (filereq);
    }
    else
    {
	rtEZRequest ("Out of memory!", "I'll buy some!", NULL, (struct TagItem *) wp_tags);
	return (NULL);
    }
    return (fullfilename);
}

char *file_request (char *prompt)
{
    static char path[100];
    return (file_request_path (prompt, path));
}

char *dir_request (char *prompt, char *working_directory)
{
    char *rv = 0;
    struct rtFileRequester *filereq = NULL;

if(!is_interactive())
return 0;

		if (filereq = rtAllocRequestA (RT_FILEREQ, NULL))
		{
		    if (rtFileRequest (filereq, working_directory, prompt,
				       RTFI_Flags, FREQF_NOFILES, TAG_END))
		    {
		        rv = working_directory;
			strcpy (rv, filereq->Dir);
		    }
		    rtFreeRequest (filereq);
		}
		else
		    tell_em("Out of memory!", 0);

return rv;
}
