#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "OS_def.h"
#include "win.h"

#define ALLOC_PROTOS_ONLY
#include "MY_alloc.h"

#define PROPER_WALLOC
#ifdef PROPER_WALLOC
/*
 * Alloc sanity checking routines
 */
typedef struct ALLOC_TRACK
{
    void *data;
    struct ALLOC_TRACK *next, *prev;
} ALLOC_TRACK;

static void *FW_mem_track (ZPL_WINDOW * fw, ALLOC_TRACK * ck)
{
    if (ck)
    {
	void *ptr = (char *) ck + sizeof (ALLOC_TRACK);
	ck->data = ptr;
	if (!fw->alloc_track)
	    fw->alloc_track = ck;
	else
	{
            ck->prev = 0;
	    ck->next = fw->alloc_track;
	    fw->alloc_track->prev = ck;
	    fw->alloc_track = ck;
	}
	return (ptr);
    }
    return (0);
}

static TY_int FW_mem_remove (ZPL_WINDOW * fw, void *data, char *module, TY_int line)
{
    ALLOC_TRACK *ck, *lck = NULL;
    char *td = data;
    td = td - sizeof (ALLOC_TRACK);
    ck = (ALLOC_TRACK *) td;

    if (ck->data == data)
    {
	lck = ck->prev;
	if (lck)
	{
/*
 * Fix the backwards link, and the forwards pointer.
 */
	    if (ck->next)
		ck->next->prev = lck;
	    lck->next = ck->next;
	}
	else
	{
	    if (ck->next)
		ck->next->prev = NULL;
	    fw->alloc_track = ck->next;
if(fw->alloc_track && fw->alloc_track->prev)
	    fw->alloc_track->prev = 0;
	}
	ck->data = 0;
	MY_free (ck, module, line);
    }
else
printf ("Walloc: data %x corrupted\n",data);
    return (0);
}

void *_FW_malloc (ZPL_WINDOW * fw, unsigned TY_int sz, char *module, TY_int line)
{
    void *ptr;
    if (sz)
    {
	TY_int size = sizeof (ALLOC_TRACK) + sz + sizeof (TY_int);

	if (!fw)
	    return MY_malloc (sz, module, line);

	ptr = (void *) MY_malloc (size, module, line);
	if (ptr)
	{
	    memset (ptr, 0, sizeof (ALLOC_TRACK));
	    return (FW_mem_track (fw, ptr));
	}
    }
    return (NULL);
}

void *_FW_calloc (ZPL_WINDOW * fw, unsigned TY_int ne, unsigned TY_int sz, char *module, TY_int line)
{
    return (_FW_malloc (fw, ne * sz, module, line));
}

void _FW_free (ZPL_WINDOW * fw, void *a, char *module, TY_int line)
{
    FW_mem_remove (fw, a, module, line);
}

void FW_free_all (ZPL_WINDOW * fw)
{
    ALLOC_TRACK *ck = fw->alloc_track, *lck;
    char id[80];

    sprintf (id, "wm(%ld,%x,%x,%x)", fw->window_id, fw->w, fw->type, fw);
    while (ck)
    {
	lck = ck->next;
	MY_free (ck, id, 0);
	ck = lck;
    }
    fw->alloc_track = NULL;
}

void *_FW_savestr (ZPL_WINDOW * fw, char *p, char *module, TY_int line)
{
    char *c = _FW_malloc (fw, strlen (p) + 3, module, line);
    strcpy (c, p);
    return (c);
}
#else
void *_FW_malloc (ZPL_WINDOW * fw, unsigned TY_int sz, char *module, TY_int line)
{
return MY_calloc(sz,1,module,line);
}

void *_FW_calloc (ZPL_WINDOW * fw, unsigned TY_int ne, unsigned TY_int sz, char *module, TY_int line)
{
    return (MY_calloc (ne, sz, module, line));
}

void _FW_free (ZPL_WINDOW * fw, void *a, char *module, TY_int line)
{
MY_free(a,module,line);
}

void FW_free_all (ZPL_WINDOW * fw)
{
return;
}

void *_FW_savestr (ZPL_WINDOW * fw, char *p, char *module, TY_int line)
{
    char *c = MY_malloc (strlen (p) + 3, module, line);
    strcpy (c, p);
    return (c);
}
#endif
