/*---------------------------------------------------------------------------
 *
 *	Title:		NICOLA Window Input/Output handling
 *
 *	File:		wop.c
 *
 *	Date:		1991
 *
 *	Description:
 *           This module contains the 'nice' functions to output ztext
 *      and stuff to a ZPL_WINdow. The type of the FW will
 *      depend on what we do. Interactive windows have slightly
 *      magical scrolling characteristics
 *
 *      Data sources:  RKM libraries/Devices
 *
 *	Rev: 1.0
 *	1.0	R.J.Harrison	Implement		             1991
 *
 *	Copyright  1991-3 R.J.Harrison
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <ctype.h>

#ifdef AMIGA
#include <intuition/intuition.h>
#include <graphics/text.h>
#include <graphics/rastport.h>
#include <string.h>
#include <clib/graphics_protos.h>
#include <clib/intuition_protos.h>
#include <clib/exec_protos.h>
#endif
#ifdef WINDOWS
#include "ms_win.h"
static HFONT font_handle, fixed_font_handle;
#endif

#include <stdarg.h>

#include "zpl.h"

#include "keys.h"

#ifdef WIN_BUF
#define MY_GetDC(fw) fw->dc
#define MY_ReleaseDC(win,b) InvalidateRect(win, NULL, 0);
#else
#define MY_GetDC(fw) GetDC(fw->w)
#define MY_ReleaseDC(win,b) ReleaseDC(win,b)
#endif

#ifdef AMIGA

#define POINTER_DATA 64
#define POINTER_SIZE (long)(POINTER_DATA*2)

TY_int FW_do_more(ZPL_WINDOW *, TY_int);

static USHORT chip zz_pointer_data[POINTER_DATA] =
{
    /* Plane 0 */
    0x0400, 0x07C0,
    0x0000, 0x07C0,
    0x0100, 0x0380,
    0x0000, 0x07E0,
    0x07C0, 0x1FF8,
    0x1FF0, 0x3FEC,
    0x3FF8, 0x7FDE,
    0x3FF8, 0x7FBE,
    0x7FFC, 0xFF7F,
    0x7EFC, 0xFFFF,
    0x7FFC, 0xFFFF,
    0x3FF8, 0x7FFE,
    0x3FF8, 0x7FFE,
    0x1FF0, 0x3FFC,
    0x07C0, 0x1FF8,
    0x0000, 0x07E0,
    0x0000, 0x0000,
    0x0000, 0x0000,
    0x0000, 0x0000,
    0x0000, 0x0000,
    0x0000, 0x0000,
    0x0000, 0x0000,
    0x0000, 0x0000,
    0x0000, 0x0000,
    0x0000, 0x0000,
    0x0000, 0x0000,
    0x0000, 0x0000,
    0x0000, 0x0000,
    0x0000, 0x0000,
    0x0000, 0x0000,
    0x0000, 0x0000,
    0x0000, 0x0000
};

static USHORT *zz_pointer = zz_pointer_data;	/* GLOBAL */

#define CSI_COLOUR "\233\60;%dm"
#define CSI_INVERSE "\233\67m"
#define CSI_NORMAL "\233\60m"
#define CSI_CURSOR_ON "\233 p"
#define CSI_CURSOR_OFF "\233\60 p"
#define CSI_GET_CPOS "\233\x36\x6e"
#define CSI_CLS "\233\x4a"
#define CSI_SET_PAGE "\233%dt"
#define CSI_RESET_PAGE "\233t"
#define CSI_ERASE_EOL "\233K"
#define CSI_SCROLL_DOWN "\233%dS"
#define CSI_SCROLL_UP "\233%dT"
#define WINDOW_BOUNDS   "\x9b\x30 q"
#define UNDERLINE_START "\233\64m"
#define UNDERLINE_END "\233\62\64m"
#define ITALIC_START "\233\63m"
#define ITALIC_END "\233\62\63m"
#define BOLD_START "\233\61m"
#define BOLD_END "\233\62\62;%dm"
#define REVERSE_START "\233\67m"
#define REVERSE_END "\233\62\67m"
#endif

#ifdef WINDOWS
static HBRUSH *brush;
static COLORREF *palette;

void
FW_init(void)
{
    palette = calloc(16, sizeof(*palette));
    brush = calloc(16, sizeof(*brush));

    palette[0] = RGB(255, 255, 255);
    palette[1] = RGB(0, 0, 0);
    palette[2] = RGB(60, 200, 200);
    palette[3] = RGB(255, 0, 0);
    palette[4] = RGB(155, 0, 155);
    palette[5] = RGB(0, 155, 155);
    palette[6] = RGB(155, 0, 155);
    palette[7] = RGB(155, 128, 255);
    palette[8] = RGB(128, 200, 200);

    brush[0] = CreateSolidBrush(palette[0]);
    brush[1] = CreateSolidBrush(palette[1]);
    brush[2] = CreateSolidBrush(palette[2]);
    brush[3] = CreateSolidBrush(palette[3]);
    brush[4] = CreateSolidBrush(palette[4]);
    brush[5] = CreateSolidBrush(palette[5]);
    brush[6] = CreateSolidBrush(palette[6]);
    brush[7] = CreateSolidBrush(palette[7]);
    brush[8] = CreateSolidBrush(palette[8]);

    {
	LOGFONT font;
	HDC hDC;

	memset(&font, 0, sizeof(font));
	hDC = GetDC(0);
	font.lfHeight = -MulDiv(9, GetDeviceCaps(hDC, LOGPIXELSY), 72);;
#ifdef xxa
	font.lfWidth = 0;
	font.lfEscapement = 0;
	font.lfOrientation = 0;
	font.lfWeight = 0;
	font.lfItalic = 0;
	font.lfUnderline = 0;
	font.lfStrikeOut = 0;
	font.lfCharSet = OEM_CHARSET;
	font.lfOutPrecision = OUT_DEFAULT_PRECIS;
	font.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	font.lfQuality = DEFAULT_QUALITY;
	font.lfPitchAndFamily = FIXED_PITCH | FF_MODERN;
	font.lfCharSet = DEFAULT_CHARSET;
#endif
	lstrcpy(font.lfFaceName, "System");

	/* if (font_handle) */
	/* DeleteObject( font_handle) ; */
	font_handle = CreateFontIndirect(&font);
	if (!font_handle)
	    error_log("Unable to open font %s, using default", font.lfFaceName);

	lstrcpy(font.lfFaceName, "Courier New");
	font.lfHeight = -12;
	fixed_font_handle = CreateFontIndirect(&font);
	if (!fixed_font_handle)
	    error_log("Unable to open fixed font %s, using default", font.lfFaceName);
	ReleaseDC(0, hDC);
    }
}

HFONT
FW_get_fixed_font()
{
    return fixed_font_handle;
}

HFONT
FW_get_dialog_font()
{
    return font_handle;
}
COLORREF
FW_get_palette(ZPL_WINDOW * fw, TY_int col)
{
    if (col > 7)
	col = 7;
    if (col < 0)
	col = 0;
    return palette[col];
}

HBRUSH
FW_get_brush(ZPL_WINDOW * fw, TY_int col)
{
    if (col > 7)
	col = 7;
    if (col < 0)
	col = 0;
    return brush[col];
}

TY_int
FW_get_font_y_size(HFONT font_handle)
{
    HDC hDC = GetDC(NULL);
    TEXTMETRIC tm;
    HFONT ofh = NULL;

    if (font_handle)
	ofh = SelectObject(hDC, font_handle);
    GetTextMetrics(hDC, &tm);
    if (ofh)
	SelectObject(hDC, ofh);
    ReleaseDC(NULL, hDC);
    return tm.tmHeight + tm.tmExternalLeading;
}

TY_int
FW_get_font_x_size(HFONT font_handle)
{
    HDC hDC = GetDC(NULL);
    TEXTMETRIC tm;
    HFONT ofh = NULL;

    if (font_handle)
	ofh = SelectObject(hDC, font_handle);
    GetTextMetrics(hDC, &tm);
    if (ofh)
	SelectObject(hDC, ofh);
    ReleaseDC(NULL, hDC);
    return tm.tmAveCharWidth;
}

TY_int
FW_get_screen_width(void)
{
    return GetSystemMetrics(SM_CXSCREEN);
}

TY_int
FW_get_screen_height(void)
{
    return GetSystemMetrics(SM_CYSCREEN);
}
#endif

#ifdef AMIGA
static char *change_rendition (char **ptr, char *start, char *end, char *line, TY_int *line_len, TY_int replace)
{
    TY_int new_length = strlen (line) + strlen (start) + strlen (end);
    char marker = **ptr, *end_ptr;

    line = make_space(line, ptr, new_length, line_len);
    insert_space (*ptr, strlen (start) - replace);
    memcpy (*ptr, start, strlen (start));

    if (marker > 127)
    {
	end_ptr = *ptr + strlen (start);
	while (*end_ptr && *end_ptr > 127)
	{
	    *end_ptr = *end_ptr & 0x7f;
	    end_ptr++;
	}
    }
    else
	end_ptr = strchr (*ptr, marker);
    insert_space (end_ptr, strlen (end) - replace);
    memcpy (end_ptr, end, strlen (end));
    if (marker > 127)
	*ptr = end_ptr + strlen (end);
    return line;
}

static char *convert_markers (char *new_line, char marker, char *start, char *end, TY_int *line_len)
{
    char *ptr, *ptr_end=0;
    ptr = strchr (new_line, marker);

    while (ptr)
    {
        if(ptr[1] == marker)
        {
            while (*ptr && (*ptr == marker)) ptr++;
            if(!*ptr)
                ptr =0;
            else
		ptr = strchr (ptr, marker);
        }

        if (ptr)
{
        if(*ptr && (!ptr_end || !ptr_end[1] || isalpha (ptr[1]))
          && ((ptr == new_line) || isspace(ptr[-1])) )
	{
	    ptr_end = strchr (ptr + 1, marker);
	    if (ptr_end &&
#ifdef PARANOID
                (!ptr_end[1] || ptr_end[1] == ' ' || ptr_end[1] == '\n' 
                 || ptr_end[1] == '\t' || ptr_end[1] == ')' || ptr_end[1] == ']'
                 || ptr_end[1] == '_'  || ptr_end[1] == '*' || ptr_end[1] == '/' 
                 || ptr_end[1] == '.'  || ptr_end[1] == ',' || ptr_end[1] == '?' 
                 || ptr_end[1] == '!'  || ptr_end[1] == ';' || ptr_end[1] == ':' ))
#else
                (!ptr_end[1] || !isalpha(ptr_end[1]) )
                && (!ptr_end[1] || 
                    (marker != '/' ) || strchr(" '\t])-;.,?:!",ptr_end[1])))
#endif
	    {
		new_line = change_rendition (&ptr, start, end, new_line, line_len, 1);
		ptr = strchr (ptr, marker);
	    }
	    else
		ptr = 0;
	}
	else
	    ptr = strchr (ptr + 1, marker);
    }
}
    return new_line;
}

static char *convert_high_chars (char *new_line, char *start, char *end, TY_int *line_len)
{
    char *ptr = new_line;

    while (*ptr)
    {
	if (*ptr > 127)
	{
	    new_line = change_rendition (&ptr, start, end, new_line, line_len, 0);
	}
	ptr++;
    }
    return new_line;
}

TY_int locate_high (char *ptr)
{
    while (*ptr)
	if (*ptr > 127)
	    return 1;
	else
	    ptr++;
    return 0;
}
#endif

char *do_line_highlighting (TY_int colour, char *line)
{
#ifdef AMIGA
    static char *new_line = 0;
    static TY_int line_len = 0;
    char bold_end[30];

    if (strchr (line, '_') || strchr (line, '/') || strchr (line, '*') || strchr (line, '~')) 
/* || locate_high (line))*/
    {

	if (strlen (line) > line_len)
	{
	    if (new_line)
		free (new_line);
	    line_len = strlen (line);
	    if (line_len < 100)
		line_len = 100;
            line_len += 10;
	    new_line = calloc (line_len, 1);
	    if (!new_line)
		return line;
	    strcpy (new_line, line);
	}
	else
	    strcpy (new_line, line);

        sprintf (bold_end,BOLD_END, colour+30);

/*new_line = convert_high_chars (new_line, REVERSE_START, REVERSE_END, &line_len);*/
	new_line = convert_markers (new_line, '_', UNDERLINE_START, UNDERLINE_END, &line_len);
	new_line = convert_markers (new_line, '/', ITALIC_START, ITALIC_END, &line_len);
	new_line = convert_markers (new_line, '*', BOLD_START, bold_end, &line_len);
	new_line = convert_markers (new_line, '~', REVERSE_START, REVERSE_END, &line_len);
	return new_line;
    }
#endif
    return line;
}

void
FW_scroll(ZPL_WINDOW * fw, TY_int num_lines)
{
#ifdef AMIGA
    char tmp[20];
    FW_set_scroll(fw);
    sprintf(tmp, CSI_SCROLL_DOWN, num_lines);
    FW_puts(fw, tmp);

#endif
#ifdef WINDOWS
#ifndef WIN_BUF
 if(!BIT_SET(fw->type, FW_NO_SCROLL))
    ScrollWindow(fw->w, 0, -fw->font_y_size * num_lines, &fw->scroll_rect, NULL);
#endif
#endif
}

void
FW_scroll_up(ZPL_WINDOW * fw, TY_int num_lines)
{
#ifdef AMIGA
    char tmp[20];
    FW_set_scroll(fw);
    sprintf(tmp, CSI_SCROLL_UP, num_lines);
    FW_puts(fw, tmp);

#endif
#ifdef WINDOWS
#ifndef WIN_BUF
    ScrollWindow(fw->w, 0, fw->font_y_size * num_lines, &fw->scroll_rect, NULL);
#endif
#endif
}

/*
 * This routine is the low level output text to window. it willl scroll
 * (after output) if required.
 */
static void
w_write_text(ZPL_WINDOW * fw, UBYTE * string, TY_int size, TY_int lf)
{
    if (string && size && fw->type & FW_TYPE_IO)
    {
#ifdef AMIGA
	fw->write_req->io_Command = CMD_WRITE;
	fw->write_req->io_Data = (APTR) string;
	fw->write_req->io_Length = size;
	DoIO(GET_IO_REQUEST(fw->write_req));
#endif
#ifdef WINDOWS
	{
	    RECT rect;
	    HDC hdc;
	    int yp = fw->current_text_y * fw->font_y_size;
	    int xp = fw->current_text_x * fw->font_x_size;
	    char *c = string;

	    /* We have to Zap the Cr's */
	    if (c = strchr(c, '\n'))
		size = c - string;

	    hdc = MY_GetDC(fw);
	    if (fixed_font_handle)
		SelectObject(hdc, fixed_font_handle);

	    SetTextColor(hdc, FW_get_palette(fw, fw->colour));
	    SetBkColor(hdc, GetSysColor(COLOR_WINDOW));

	    rect.top = yp;
	    rect.bottom = yp + fw->font_y_size;
	    rect.left = xp;
	    rect.right = fw->width;
	    SetBkMode(hdc, OPAQUE);
	    ExtTextOut(hdc, xp, yp,
		       (lf ? ETO_OPAQUE : 0) | ETO_CLIPPED, &rect,
		       string, size, NULL);

	    MY_ReleaseDC(fw->w, hdc);

	    if (lf)
	    {
		fw->current_text_y++;
		fw->current_text_x = 0;

		if (fw->current_text_y >= fw->scroll_y)
		{
		    fw->current_text_y = fw->scroll_y - 1;
                    if (!BIT_SET (fw->type, FW_SCROLLBACK))
		        FW_scroll(fw, 1);
		}
	    }
            else
		fw->current_text_x += size;
	}
#endif
    }
}

void
FW_write(ZPL_WINDOW * fw, UBYTE * string, TY_int size)
{
    if (fw)
    {
	fw->current_text_y = fw->scroll_y - 1;
	w_write_text(fw, string, size, 1);
    }
}
/*
 * Output a NULL-terminated string of characters to a console
 */
void
FW_puts(ZPL_WINDOW * fw, UBYTE * string)
{
    w_write_text(fw, string, strlen(string), 1);
}

/*
 * Output a NULL-terminated string of characters to a console using
 * highlighting rules..
 */
void
FW_puts_hl (ZPL_WINDOW * fw, UBYTE * string)
{
    w_write_text(fw, string, strlen(string), 1);
}

static TY_int
to_front(ZPL_WINDOW * fw)
{
#ifdef AMIGA
    WindowToFront(fw->w);
#endif
#ifdef WINDOWS
#endif
    return (0);
}

static TY_int
to_back (ZPL_WINDOW * fw)
{
#ifdef AMIGA
    WindowToBack (fw->w);
#endif
#ifdef WINDOWS
#endif
    return (0);
}

void
FW_activate(ZPL_WINDOW * fw)
{
#ifdef AMIGA
    ActivateWindow(fw->w);
#endif
#ifdef WINDOWS
#endif
}

void
FW_to_front(ZPL_WINDOW * fw, TY_int do_parent)
{
if(do_parent)
    to_front(fw);
    do_child_windows(fw, to_front, 0);
#ifdef AMIGA
    if (fw->w->IDCMPFlags & RAWKEY)
	FW_activate(fw);
#endif
#ifdef WINDOWS
#endif
}

void
FW_to_back (ZPL_WINDOW * fw, TY_int do_parent)
{
if(do_parent)
    to_back(fw);
    do_child_windows(fw, to_back, 0);
#ifdef AMIGA
    if (fw->w->IDCMPFlags & RAWKEY)
	FW_activate(fw);
#endif
#ifdef WINDOWS
#endif
}

char
FW_getc(ZPL_WINDOW * fw)
{
#ifdef AMIGA
    if (fw->type & FW_TYPE_IO)
    {
	char ch = 0;
	fw->read_req->io_Command = CMD_READ;
	fw->read_req->io_Data = &ch;
	fw->read_req->io_Length = 1;	/* means prTY_int till terminating
					 * null */
	DoIO(GET_IO_REQUEST(fw->read_req));
	return (ch);
    }
#endif
#ifdef WINDOWS
    return (0);
#endif
}

void
FW_set_busy_pointer(ZPL_WINDOW * fw)
{
#ifdef AMIGA
    if (fw->w)
	SetPointer(fw->w, zz_pointer, 25L, 16L, 5L, 5L);
#endif
#ifdef WINDOWS
#endif
}

void
FW_clear_busy_pointer(ZPL_WINDOW * fw)
{
#ifdef AMIGA
    if (fw->w)
	ClearPointer(fw->w);
#endif
#ifdef WINDOWS
#endif
}

void
FW_set_title(ZPL_WINDOW * fw, char *title)
{
#ifdef AMIGA
    if (fw->w)
	SetWindowTitles(fw->w, title, (char *) -1);
#endif
#ifdef WINDOWS
    SetWindowText(fw->w, title);
#endif
}

TY_int
FW_scale_text_x(ZPL_WINDOW * fw, TY_int x)
{
    return (x * fw->font_x_size);
}

TY_int
FW_scale_text_y(ZPL_WINDOW * fw, TY_int y)
{
    return (y * fw->font_y_size);
}

void
FW_line_r(ZPL_WINDOW * fw, TY_int xe, TY_int ye)
{
#ifdef AMIGA
    Draw(fw->rp, xe, ye);
#endif
#ifdef WINDOWS
    HDC hdc;
    hdc = MY_GetDC(fw);
#ifdef WIN32
    MoveToEx(hdc, fw->gx, fw->gy, NULL);
#else
    MoveTo(hdc, fw->gx, fw->gy);
#endif
    LineTo(hdc, xe, ye);
    MY_ReleaseDC(fw->w, hdc);
    fw->gx = xe;
    fw->gy = ye;
#endif
}

void
FW_line(ZPL_WINDOW * fw, TY_int xs, TY_int ys, TY_int xe, TY_int ye)
{
#ifdef AMIGA
    Move(fw->rp, xs, ys);
    Draw(fw->rp, xe, ye);
#endif
#ifdef WINDOWS
    HDC hdc;
    hdc = MY_GetDC(fw);
#ifdef WIN32
    MoveToEx(hdc, xs, ys, NULL);
#else
    MoveTo(hdc, xs, ys);
#endif
    LineTo(hdc, xe, ye);
    MY_ReleaseDC(fw->w, hdc);
    fw->gx = xe;
    fw->gy = ye;
#endif
}

void
FW_fill_r(ZPL_WINDOW * fw, TY_int xs, TY_int ys, TY_int xe, TY_int ye)
{
#ifdef AMIGA
    RectFill(fw->rp, xs, ys, xe, ye);
#endif
#ifdef WINDOWS
    HDC hdc;

    hdc = MY_GetDC(fw);

    SelectObject(hdc, FW_get_brush(fw, fw->gcolour));

    Rectangle(hdc, xs, ys, xe, ye);
    MY_ReleaseDC(fw->w, hdc);

    fw->gx = xe;
    fw->gy = ye;
#endif
}

void
FW_move(ZPL_WINDOW * fw, TY_int x, TY_int y)
{
#ifdef AMIGA
    static char tmp[20];

    sprintf(tmp, "\233%d;%dH", y + 1, x + 1);
    FW_puts(fw, tmp);
#endif
#ifdef WINDOWS
    fw->current_text_x = x;
    fw->current_text_y = y;
#endif
}

void
FW_move_s(ZPL_WINDOW * fw, TY_int x, TY_int y)
{
    FW_move(fw, x, y);

    if (y < fw->scroll_y && y >= 0)
	fw->current_text_y = y;
    else
	fw->current_text_y = fw->scroll_y - 1;
    fw->current_text_x = x;
}

void
FW_reset_scroll(ZPL_WINDOW * fw)
{
#ifdef AMIGA
    char tmp[20];
    sprintf(tmp, CSI_RESET_PAGE, 255);
    FW_puts(fw, tmp);
#endif
#ifdef WINDOWS
#endif
}

void
FW_set_scroll(ZPL_WINDOW * fw)
{
#ifdef AMIGA
    char tmp[20];
    if (fw->w->IDCMPFlags & RAWKEY)
    {
	FW_move(fw, fw->current_text_x, min(fw->current_text_y, fw->scroll_y - 1));
	sprintf(tmp, CSI_SET_PAGE, fw->scroll_y);
	FW_puts(fw, tmp);
    }
#endif
#ifdef WINDOWS
#endif
}

void
FW_erase_textline(ZPL_WINDOW * fw, TY_int x, TY_int y)
{
#ifdef AMIGA
    FW_reset_scroll(fw);
    FW_move(fw, x, y);
    FW_puts(fw, CSI_ERASE_EOL);
    FW_set_scroll(fw);
#endif
#ifdef WINDOWS
    /* FW_move(fw,x,y); */
    /* w_write_text(fw," ",1,FALSE); */
#endif
}

void
FW_erase_to_eol(ZPL_WINDOW * fw)
{
#ifdef AMIGA
    FW_puts(fw, CSI_ERASE_EOL);
#endif
#ifdef WINDOWS
    w_write_text(fw, " ", 1, FALSE);
#endif
}

void
FW_cls(ZPL_WINDOW * fw)
{
#ifdef AMIGA
    FW_reset_scroll(fw);
    FW_move(fw, 0, 0);
    FW_puts(fw, CSI_CLS);
    FW_set_scroll(fw);
#endif
#ifdef WINDOWS
    /* FW_scroll(fw,fw->scroll_y+3); */
#endif
}

void
FW_colour(ZPL_WINDOW * fw, TY_int col)
{
    if (fw->colour != col)
    {
#ifdef AMIGA
	static char tb[40];
	sprintf(tb, CSI_COLOUR, col + 30);
	FW_puts(fw, tb);
#endif
#ifdef WINDOWS
#endif
	fw->colour = col;
    }
}

/*
 * Set the graphics colour
 */
void
FW_gcolour(ZPL_WINDOW * fw, TY_int col)
{
#ifdef AMIGA
    TY_int num_colours = 1 << get_screen_depth ();
    SetAPen(fw->rp, min(col,num_colours));
#endif
#ifdef WINDOWS
    if (col != fw->gcolour)
    {
	fw->gcolour = col;
    }
#endif
}

void
FW_gtext (ZPL_WINDOW * fw, TY_int x, TY_int y, char *text)
{
#ifdef AMIGA
     SetDrMd(fw->rp, JAM2);
     Move(fw->rp, x, y);
     Text(fw->rp, text, strlen(text));
#endif
#ifdef WINDOWS
    fw->current_text_y = y / fw->font_y_size;
    fw->current_text_x = x / fw->font_x_size;
    w_write_text(fw, text, strlen(text), 0);
#endif
}

void
FW_prompt(ZPL_WINDOW * fw)
{
    TY_int cur_col = fw->colour;
    COLOUR_PREFS *cp = PR_get_colour_prefs();

    if (fw->prompt)
    {
#ifdef AMIGA
	FW_puts(fw, CSI_RESET_PAGE);
#endif
	FW_move(fw, 0, fw->input_y);
	FW_colour(fw, cp->prompt);
	FW_puts(fw, fw->prompt);
	FW_colour(fw, cur_col);
	FW_move(fw, fw->input_x + fw->input_pos, fw->input_y);
	/* ~    FW_set_scroll (fw); */
    }
}

void
FW_erase_prompt(ZPL_WINDOW * fw)
{
#ifdef AMIGA
    if (fw->prompt)
    {
	FW_puts(fw, CSI_RESET_PAGE);
	FW_move(fw, 0, fw->input_y - 1);
	FW_puts(fw, CSI_ERASE_EOL);
	FW_move(fw, fw->input_x + fw->input_pos, fw->input_y);
    }
#endif
#ifdef WINDOWS
    FW_erase_textline(fw, 0, fw->input_y);
#endif
}

void
FW_set_prompt(ZPL_WINDOW * fw, char *prompt)
{
    if (fw->prompt)
	FW_free(fw, fw->prompt);
    fw->prompt = FW_savestr(fw, prompt);
    fw->input_x = strlen(prompt);
    FW_prompt(fw);
}

#ifdef AMIGA
static TY_int more_action(ZPL_WINDOW *, UWORD, struct IntuiMessage *);
#else
static TY_int more_action(ZPL_WINDOW *, UWORD, EVENT_MESSAGE *);
#endif

TY_int
FW_do_more(ZPL_WINDOW * fw, TY_int start)
{
    COLOUR_PREFS *cp = PR_get_colour_prefs();
    TY_int cur_col = fw->colour;

    if (start)
    {
	if (BIT_SET(fw->type, FW_WAITING_FOR_MORE))
	    return (0);

	fw->lines_printed = -2;

	fw->process = more_action;

#ifdef AMIGA
	FW_puts(fw, CSI_CURSOR_ON);
	FW_puts(fw, CSI_RESET_PAGE);
	FW_move(fw, 0, fw->input_y);
	FW_colour(fw, PR_default_colour(cp->prompt));
	FW_puts(fw, CSI_INVERSE);
	FW_puts(fw, "More ?");
	FW_puts(fw, CSI_NORMAL);
	fw->colour = -1;
	FW_colour(fw, cur_col);
#endif
#ifdef WINDOWS
	FW_move(fw, 0, fw->input_y);
	FW_colour(fw, PR_default_colour(cp->prompt));
	FW_puts(fw, "More ?");
	fw->colour = -1;
	FW_colour(fw, cur_col);
#endif
	SET_BIT(fw->type, FW_WAITING_FOR_MORE);
    } else
    {
	fw->process = 0;
	FW_erase_textline(fw, 0, fw->input_y);
#ifdef AMIGA
	FW_puts(fw, CSI_CURSOR_OFF);
	FW_puts(fw, CSI_NORMAL);
#endif
#ifdef WINDOWS
#endif
	FW_set_scroll(fw);
	FW_prompt(fw);
	STRIP_BIT(fw->type, FW_WAITING_FOR_MORE);
	return (fw->lines_printed >= 0);
    }
    return (0);
}

#ifdef AMIGA
static TY_int
more_action(ZPL_WINDOW * fw, UWORD msg_class, struct IntuiMessage * msg)
{
    if (msg_class == CLOSEWINDOW)
    {
	FW_do_more(fw, FALSE);
	return (1);
    } else if (msg_class == RAWKEY)
    {
	TY_int i, numchars;
	char buffer[20];

	numchars = rawkey_convert(msg, buffer, sizeof(buffer), 0L);

	if (numchars > 0 && numchars < sizeof(buffer))
	{
	    for (i = 0; i < numchars; i++)
	    {
		switch (toupper(buffer[i]))
		{
		case RETURN:
		case ' ':
		case 'Y':
		    if (FW_output_sb(fw, TRUE))
			FW_do_more(fw, FALSE);
		    break;
		    /* Else stop more, as it's finished */
		case 'N':
		    fw->lines_printed = -2;
		    FW_remove_sb_after_end(fw);
		    FW_do_more(fw, FALSE);
		    break;
		}
	    }
	}
    }
    return (0);
}
#endif

#ifdef WINDOWS
static TY_int
more_action(ZPL_WINDOW * fw, UWORD msg_class, EVENT_MESSAGE * em)
{
    if (em->message == WM_CHAR)
    {
	switch (toupper(em->keycode))
	{
	    case RETURN:
	    case ' ':
	    case 'Y':
	    if (FW_output_sb(fw, TRUE))
		FW_do_more(fw, FALSE);
	    break;
	    /* Else stop more, as it's finished */
	case 'N':
	    fw->lines_printed = -2;
	    FW_remove_sb_after_end(fw);
	    FW_do_more(fw, FALSE);
	    break;
	}
    }
    return (0);
}
#endif

void
FW_output_more(ZPL_WINDOW * pfw)
{
    if (FW_output_sb(pfw, TRUE))
	FW_do_more(pfw, FALSE);
}

void
FW_output_all_text(ZPL_WINDOW * pfw)
{
    FW_output_all_sb(pfw);
    FW_do_more(pfw, FALSE);
}

/*
 * Draw text onto a non-scrolling 'graphics display' window
 */
void
FW_gr_text(ZPL_WINDOW * fw, TY_int x, TY_int y, char *text)
{
FW_gcolour(fw, 1);
#ifdef AMIGA
FW_gtext(fw, 2+fw->w->BorderLeft+(x+1)*fw->font_x_size, y*fw->font_y_size+fw->w->BorderTop+2, text);
#else
FW_move_s(fw,x,y);
FW_puts(fw, text);
#endif
}


void
FW_text(ZPL_WINDOW * fw, TY_int x, TY_int y, char *text)
{
    char *lf = NULL, tc = 0;
    /* TY_int             ys = fw->w->IFont->tf_YSize; */
    TY_int sb = BIT_SET(fw->type, FW_SCROLLBACK);

    if (fw->lines_printed < 0)
	return;

    if (!sb)
    {
	FW_move_s(fw, x, y);
	FW_set_scroll(fw);
    }
    do
    {
	lf = strchr(text, '\n');

	/*
	 * Handle line feeds by splitting the line at the \n and outputting
	 * the portion (or adding the portion to the scrollback ready to be
	 * output)
	 */
	if (lf)
	{
	    tc = lf[1];
	    lf[1] = 0;
	} else
	    fw->current_text_x += strlen(text);

	/*
	 * Ensure that the line doesn't wrap.
	 */
	if (strlen(text) >= fw->t_width)
	    text[fw->t_width - 1] = 0;

	if (sb)
	    FW_add_sb(fw, text);
	else
	{
	    FW_puts(fw, text);

	    if (lf)
	    {
		if (++fw->current_text_y >= fw->scroll_y)
		{
		    fw->current_text_y = fw->scroll_y;
		}
		fw->current_text_x = 0;
		y++;
	    }
	}
	if (lf)
	{
	    lf[1] = tc;
	    text = ++lf;
	}
    }
    while (lf);
    if (sb)
	FW_set_sb(fw);
}

TY_int
FW_output_disabled(ZPL_WINDOW * fw)
{
    return (fw->lines_printed < 0);
}

void
FW_clear_more(ZPL_WINDOW * fw)
{
    fw->lines_printed = 0;
}

void
FW_printf(ZPL_WINDOW * fw, char *fmt,...)
{
    STATIC_VAR(local_buf, char, 3000);
    va_list ap;
    void *a, *b, *c, *d, *e, *f, *g, *h, *i, *j, *k, *l, *m, *n, *o, *p, *q, *r, *s, *t, *u, *v, *w, *x, *y, *z;

    if (!fw)
	return;

    va_start(ap, fmt);
    a = va_arg(ap, void *);
    b = va_arg(ap, void *);
    c = va_arg(ap, void *);
    d = va_arg(ap, void *);
    e = va_arg(ap, void *);
    f = va_arg(ap, void *);
    g = va_arg(ap, void *);
    h = va_arg(ap, void *);
    i = va_arg(ap, void *);
    j = va_arg(ap, void *);
    k = va_arg(ap, void *);
    l = va_arg(ap, void *);
    l = va_arg(ap, void *);
    m = va_arg(ap, void *);
    n = va_arg(ap, void *);
    o = va_arg(ap, void *);
    p = va_arg(ap, void *);
    q = va_arg(ap, void *);
    r = va_arg(ap, void *);
    s = va_arg(ap, void *);
    t = va_arg(ap, void *);
    u = va_arg(ap, void *);
    v = va_arg(ap, void *);
    w = va_arg(ap, void *);
    x = va_arg(ap, void *);
    y = va_arg(ap, void *);
    z = va_arg(ap, void *);
    va_end(ap);

    sprintf(local_buf, fmt, a, b, c, d, e, f, g, h, i, j, k, l, m, n, o, p, q, r, s, t, u, v, w, x, y, z);
    if (strlen(local_buf) > 3000)
	tell_em("FW_printf(): buffer len exceeded", NULL);
    if (fw->type & FW_TYPE_IO)
	FW_text(fw, fw->current_text_x, fw->current_text_y, local_buf);
    else if (*local_buf)
    {
	char *eol = strrchr(local_buf, '\n');
	if (eol && !eol[1])
	    *eol = 0;
	if (*local_buf)
	    tell_em(local_buf, NULL);
    }
}

void
FW_putc(ZPL_WINDOW * fw,
	char c,
	TEXT_COORDINATE x,
	TEXT_COORDINATE y)
{
    FW_move(fw, x, y);
    w_write_text(fw, &c, 1, 0);
}

void
FW_remove_cursor(ZPL_WINDOW * fw)
{
#ifdef AMIGA
    FW_puts(fw, CSI_CURSOR_OFF);
#endif
#ifdef WINDOWS
#endif
}

void
FW_draw_cursor(ZPL_WINDOW * fw, TY_int x, TY_int y)
{
    fw->cursor_x = x;
    fw->cursor_y = y;
#ifdef AMIGA
    FW_puts(fw, CSI_CURSOR_ON);
    FW_move(fw, fw->cursor_x, fw->cursor_y);
#endif
#ifdef WINDOWS
#endif
}

void
FW_cursor_move(ZPL_WINDOW * fw)
{
#ifdef AMIGA
    FW_puts(fw, CSI_CURSOR_ON);
    FW_reset_scroll(fw);
    FW_move(fw, fw->input_pos + fw->input_x, fw->input_y);
#endif
#ifdef WINDOWS
#endif
}

void
error_log(char *fmt,...)
{
    static ZPL_WINDOW *fw = 0;
    char temp[500];

    va_list ap;
    {
	STATIC_VAR(buf, char, 2500);
	va_start(ap, fmt);
	vsprintf(buf, fmt, ap);
	va_end(ap);
	if (strlen(buf) > 2500)
	    tell_em("error_log(): buffer len exceeded", 0);
	tell_em(buf, NULL);
    }
}

void
log_message(char *fmt,...)
{
    static ZPL_WINDOW *fw = 0;
    STATIC_VAR (temp,char,2500);
    va_list ap;
    void *a, *b, *c, *d, *e, *f, *g, *h, *i, *j, *k, *l;

    va_start(ap, fmt);
    a = va_arg(ap, void *);
    b = va_arg(ap, void *);
    c = va_arg(ap, void *);
    d = va_arg(ap, void *);
    e = va_arg(ap, void *);
    f = va_arg(ap, void *);
    g = va_arg(ap, void *);
    h = va_arg(ap, void *);
    i = va_arg(ap, void *);
    j = va_arg(ap, void *);
    k = va_arg(ap, void *);
    l = va_arg(ap, void *);
    l = va_arg(ap, void *);
    va_end(ap);

	sprintf(temp, fmt, a, b, c, d, e, f, g, h, i, j, k, l);
	if (strlen(temp) > 2500)
	    tell_em("log_message: buffer len exceeded", 0);
	tell_em(temp, NULL);
}

void
FW_error(ZPL_WINDOW * fw, char *err)
{
    FW_printf(fw, "Error:%s\n", err);
}

TY_int
FW_fp_type_file(ZPL_WINDOW * fw, FILE * f)
{
    char line[180];
    if (f)
    {
	FW_colour(fw, PR_get_colour(FOREGROUND_COLOUR));
	while (read_line(line, sizeof(line) - 1, f) > 0)
	{
	    FW_printf(fw, "%s\n", line);
	}
	return (TRUE);
    }
    return (FALSE);
}

TY_int
FW_type_file(ZPL_WINDOW * fw, char *filename)
{
    FILE *f;
    if (f = fopen(filename, "r"))
    {
	FW_fp_type_file(fw, f);
	fclose(f);
	return (TRUE);
    }
    return (FALSE);
}

TY_int
FW_get_size(ZPL_WINDOW * fw)
{
#ifdef AMIGA
    TY_int done = FALSE, i, rows;
    char buffer[100], *p = buffer, ch;
    FW_puts(fw, WINDOW_BOUNDS);

    if (fw->type & FW_TYPE_IO)
    {
	while (!done)
	{
	    ch = FW_getc(fw);
	    *p++ = ch;
	    if (ch == 'r')
		done = TRUE;
	}
	*p = 0;
	sscanf(buffer + 1, "%d;%d;%d", &i, &i, &rows);
	return (rows);
    }
#endif
    return (0);			/* This routine only works on the amiga
				 * console device... */
}

/*
 * This function is called whenever the window is resized.
 */
void
change_window_size(ZPL_WINDOW * fw)
{
    TY_int cys = fw->height, cxs = fw->width;
#ifdef AMIGA
    TY_int ys = fw->w->IFont->tf_YSize, ba;

    FW_erase_prompt(fw);
    ba = fw->w->IFont->tf_Baseline;
    fw->height = fw->w->Height - (fw->w->BorderTop - fw->w->BorderBottom);
    fw->t_width = (fw->w->Width - (fw->w->BorderRight - fw->w->BorderLeft)) / fw->w->IFont->tf_XSize;
    fw->width = fw->w->Width;

    if (!(fw->input_y = FW_get_size(fw)))
	fw->input_y = (fw->height / ys);
    fw->t_height = fw->input_y - 2;
    fw->scroll_y = fw->input_y - 1;

    fw->font_x_size = fw->w->IFont->tf_XSize;
    fw->font_y_size = fw->w->IFont->tf_YSize;

    FW_prompt(fw);
    if (fw->w->IDCMPFlags & RAWKEY)
	FW_set_scroll(fw);
    else
    {
	fw->scroll_y = fw->input_y - 1;
	fw->t_height = fw->input_y - 1;
    }

    if ((cxs != fw->width || cys != fw->height) &&
	BIT_SET(fw->type, FW_SCROLLBACK))
	FW_change_sb_size(fw);
#endif
#ifdef WINDOWS
    HDC hDC = GetDC(fw->w);
    TEXTMETRIC tm;
    RECT rcWindow;

    if (fixed_font_handle)
	SelectObject(hDC, fixed_font_handle);
    GetTextMetrics(hDC, &tm);
    ReleaseDC(fw->w, hDC);

    GetClientRect(fw->w, &rcWindow);

    fw->font_x_size = tm.tmAveCharWidth;
    fw->font_y_size = tm.tmHeight + tm.tmExternalLeading;

    FW_erase_prompt(fw);

    fw->height = rcWindow.bottom - rcWindow.top;
    fw->width = rcWindow.right - rcWindow.left;
    fw->t_width = fw->width / fw->font_x_size;

    fw->input_y = (fw->height / fw->font_y_size);

    if (((fw->input_y + 1) * fw->font_y_size) >= (fw->height - 3))
	fw->input_y--;

    fw->t_height = fw->input_y - 2;
    fw->scroll_y = fw->input_y - 1;

    fw->scroll_rect = rcWindow;
    fw->scroll_rect.bottom = fw->scroll_y*fw->font_y_size;
    /* fw->scroll_rect.left=0; */
    /* fw->scroll_rect.top=0; */

    FW_prompt(fw);

    if (((*fw->sub_window->hdr.process) (fw->sub_window, WM_GET_WINDOW_TYPE, 0, 0) & FW_TYPE_IO))
	FW_set_scroll(fw);
    else
    {
	fw->scroll_y = fw->input_y - 1;
	fw->t_height = fw->input_y - 1;
    }

    if ((cxs != fw->width || cys != fw->height) &&
	BIT_SET(fw->type, FW_SCROLLBACK))
	FW_change_sb_size(fw);
    /* a slimy hack to force the scroll position, region to */
    /* be recalculated based on the new character sizes */

#ifdef RESIZE
    GetClientRect(fw->w, &rcWindow);
    SendMessage(fw->w, WM_SIZE, SIZENORMAL,
		(LPARAM) MAKELONG(rcWindow.right - rcWindow.left,
				  rcWindow.bottom - rcWindow.top));
#endif

#endif
}

void
FW_redraw_frame(ZPL_WINDOW * fw)
{
#ifdef AMIGA
    RefreshWindowFrame(fw->w);
#endif
#ifdef WINDOWS
#endif
}

TY_int
FW_get_left_edge(ZPL_WINDOW * fw)
{
#ifdef AMIGA
    return (fw->w->LeftEdge);
#endif
#ifdef WINDOWS
    RECT rect;

    GetWindowRect(fw->w, &rect);
    return (rect.left);
#endif
}

TY_int
FW_get_top_edge(ZPL_WINDOW * fw)
{
#ifdef AMIGA
    return (fw->w->TopEdge);
#endif
#ifdef WINDOWS
    RECT rect;

    GetWindowRect(fw->w, &rect);
    return (rect.top);
#endif
}

TY_int
FW_get_height(ZPL_WINDOW * fw)
{
#ifdef AMIGA
    return (fw->w->Height);
#endif
#ifdef WINDOWS
    RECT rect;

    GetWindowRect(fw->w, &rect);
    return (rect.bottom - rect.top);
#endif
}

TY_int
FW_get_width(ZPL_WINDOW * fw)
{
#ifdef AMIGA
    return (fw->w->Width);
#endif
#ifdef WINDOWS
    RECT rect;

    GetWindowRect(fw->w, &rect);
    return (rect.right - rect.left);
#endif
}

TY_int
FW_get_border_top(ZPL_WINDOW * fw)
{
#ifdef AMIGA
    return (fw->w->BorderTop);
#endif
#ifdef WINDOWS
    return (0);
#endif
}

char *
FW_get_title(ZPL_WINDOW * fw)
{
#ifdef AMIGA
    return (fw->w->Title);
#endif
#ifdef WINDOWS
    STATIC_VAR(title, char, 80);
    GetWindowText(fw->w, title, 80);
    return (title);
#endif
}

void
draw_complete_line(ZPL_WINDOW * fw, TY_int percent_complete, TY_int sy, TY_int sx)
{
    TY_int ex;

    if (percent_complete < 0)
	return;

    percent_complete = min(100, percent_complete);

#ifdef AMIGA
    sy = ((1 + (SHORT) fw->w->RPort->Font->tf_YSize) * sy) + 5;
    sx = ((SHORT) fw->w->RPort->Font->tf_XSize * sx);
    sy = sy + fw->w->RPort->Font->tf_YSize/2;
#endif
#ifdef WINDOWS
    sy = fw->font_y_size * sy + (fw->font_y_size/2);
    sx = fw->font_x_size * sx;
    sy = sy + fw->font_y_size/2;
#endif
    ex = sx + percent_complete * (((FW_get_width(fw) - sx) -12) / 100);

     	if (!percent_complete)
	{
            TY_int ex = sx + 100 * (((FW_get_width(fw) - sx) -12) / 100);
	    FW_gcolour(fw, 0);
	    FW_line(fw, sx, sy, ex, sy);
	}

    if (percent_complete)
    {
	FW_gcolour(fw, 1);
	FW_line(fw, sx, sy, ex, sy);
    }
}

void
draw_bevel_box(ZPL_WINDOW * fw, TY_int sx, TY_int sy, TY_int ex, TY_int ey)
{
	FW_gcolour(fw, 1);
	FW_line(fw, sx - 1, sy - 1, ex + 1, sy - 1);
	FW_gcolour(fw, 2);
	FW_line_r(fw, ex + 1, ey + 1);
	FW_line_r(fw, sx - 1, ey + 1);
	FW_gcolour(fw, 1);
	FW_line_r(fw, sx - 1, sy - 1);
}

void
draw_complete_box(ZPL_WINDOW * fw, TY_int draw_outline, TY_int percent_complete, TY_int sy, TY_int sx)
{
    TY_int ex, ey;
char pc_buf[20];
static TY_int init=FALSE, filly_bar_colour, mono=FALSE, gx;

if(!init)
{
#ifdef AMIGA
    TY_int num_colours = 1 << get_screen_depth ();
#else
    TY_int num_colours = 8;
#endif
if(num_colours < 3)
{
mono=TRUE;
filly_bar_colour=1;
}
else
filly_bar_colour=3;
init = TRUE;
}

    if (percent_complete < 0)
	return;
    percent_complete = min(100, percent_complete);

#ifdef AMIGA
    sy = ((1 + (SHORT) fw->w->RPort->Font->tf_YSize) * sy) + 5;
    sx = ((SHORT) fw->w->RPort->Font->tf_XSize * sx);
    ey = sy + fw->w->RPort->Font->tf_YSize;
#endif
#ifdef WINDOWS
    sy = fw->font_y_size * sy;
    sx = fw->font_x_size * sx;
    ey = sy + fw->font_y_size;
#endif

    ex = sx + percent_complete * (((FW_get_width(fw) - sx) -12) / 100);

    if (draw_outline || !percent_complete)
    {
	TY_int ex = sx + 100 * (((FW_get_width(fw) - sx) -12) / 100);
gx = ex - fw->font_x_size*3;

	FW_gcolour(fw, 1);
	FW_line(fw, sx - 1, sy - 1, ex + 1, sy - 1);
	FW_gcolour(fw, 2);
	FW_line_r(fw, ex + 1, ey + 1);
	FW_line_r(fw, sx - 1, ey + 1);
	FW_gcolour(fw, 1);
	FW_line_r(fw, sx - 1, sy - 1);
	if (!percent_complete)
	{
	    FW_gcolour(fw, 0);
	    FW_fill_r(fw, sx, sy, ex, ey);
	}
    }
    if (percent_complete)
    {
	FW_gcolour(fw, filly_bar_colour);
	FW_fill_r(fw, sx, sy, ex, ey);
    }
    sprintf (pc_buf,"%3d%%",percent_complete);
    FW_gcolour(fw, 1);
    FW_gtext(fw, gx, sy-5, pc_buf);
}

void
calculate_window_size(TY_int * height, TY_int num_lines, TY_int * width, TY_int num_columns)
{
#ifdef AMIGA
    struct TextFont *font = get_console_font();	/* Font to use */


    if (font || (font = OpenFont(get_window_font())))
    {
	*height = ((1 + (SHORT) font->tf_YSize) * num_lines);
	*width = (max(7, (SHORT) font->tf_XSize) * num_columns) + 20;
	CloseFont(font);	/* Close the Font */
    } else
	printf("Couldn't get default font\n");
#endif
#ifdef WINDOWS
    *height = num_lines * FW_get_font_y_size(fixed_font_handle) + 20;
    *width = num_columns * FW_get_font_x_size(fixed_font_handle);
#endif
}


#ifdef WINDOWS

UBYTE
get_screen_depth()
{
    return (8);			/* 8 Planes deep ... (a fudge) */
}

HWND
OpenWindow(struct NewWindow * nw)
{
    HWND hWnd;

#define MDI
#ifdef MDI
    MDICREATESTRUCT mcs;

    mcs.szTitle = nw->Title;

    mcs.szClass = get_root_class();
    mcs.hOwner = get_instance();

    /* Use the default size for the window */
	mcs.x = nw->LeftEdge;
	mcs.y = nw->TopEdge;
	mcs.cx = nw->Width;
	mcs.cy = nw->Height;
//    mcs.x = mcs.cx = CW_USEDEFAULT;
//    mcs.y = mcs.cy = CW_USEDEFAULT;

    /* Set the style DWORD of the window to default */
    mcs.style = 0;

    /* tell the MDI Client to create the child */
    hWnd = (HWND) SendMessage(get_mdi_client(),
			      WM_MDICREATE,
			      0,
			      (LONG) (LPMDICREATESTRUCT) & mcs);

#else
    hWnd = CreateWindow(get_root_class(),	/* See RegisterClass() call. */
			nw->Title,	/* Text for window title bar. */
    WS_THICKFRAME | WS_CAPTION | WS_BORDER | WS_POPUPWINDOW /* WS_CHILD| */ ,
    /* WS_CHILD|WS_BORDER|WS_CAPTION, */
			nw->LeftEdge, nw->TopEdge, nw->Width, nw->Height,	/* Use default
										 * positioning */
			get_real_root_window(),	/* parent. */
			NULL,
			get_instance(),	/* This instance owns this window. */
			NULL	/* We don't use any data in our WM_CREATE */
	);
#endif

    /* If window could not be created, return "failure" */
    if (!hWnd)
    {
	error_log("Failed to open window '%s'", nw->Title);
	return (FALSE);
    }
    /* MoveWindow (hWnd, 0,0, 600,200, TRUE); */
    /* SetScrollRange(hWnd, SB_VERT, 0, 10,TRUE); */
    /* ShowWindow (hWnd, SW_SHOWNORMAL);/* Show the window */
    /* UpdateWindow (hWnd);	/* Sends WM_PAINT message */
    return (hWnd);
}
#endif


#ifdef __AA__
WS_OVERLAPPEDWINDOW,		/* Window style. */
CW_USEDEFAULT, 0, CW_USEDEFAULT, 0,	/* Use default positioning */
NULL,
#endif

/*
 * This function is responsible for intialising everything about the window.
 * It is called after the window has been created.
 */
void
FW_init_window(ZPL_WINDOW * fw)
{
#ifdef WINDOWS
    HDC hdc;

    hdc = MY_GetDC(fw);
    if (fixed_font_handle)
	SelectObject(hdc, fixed_font_handle);
    MY_ReleaseDC(fw->w, hdc);

#endif

    if (fw->type & FW_TYPE_IO)
    {
	if (fw->type & FW_TYPE_OUTPUT)
	{
	    FW_remove_cursor(fw);
	    FW_reset_scroll(fw);
	    FW_set_scroll(fw);
	} else
	{
	    FW_draw_cursor(fw, 0, 0);
	    FW_reset_scroll(fw);
	    FW_move(fw, fw->input_pos + fw->input_x, fw->input_y);
	}
	FW_colour(fw, 1);
    }
    /* FW_set_scroll(fw); */
    fw->highest_pl = 0;//HISTORY_SIZE-1;
    memset(fw->previous_lines, 0, sizeof(fw->previous_lines));
    FW_clear_busy_pointer(fw);
}

#ifdef WINDOWS
TY_int
FW_get_string_xsize(HFONT fon, char *str)
{
    char *newstr = savestr(str), *ptr = newstr, *optr = ptr;
    HDC dc = GetDC(0);
SIZE te;

    while (ptr && *ptr)
    {
	if ((*ptr != '_') && (*ptr != '&'))
	    *optr++ = *ptr++;
	else
	    ptr++;
    }

    if (ptr != optr)
	*optr = 0;

#ifdef WIN32
    GetTextExtentPoint32 (dc, newstr, strlen(newstr),&te);
#else
    GetTextExtentPoint (dc, newstr, strlen(newstr),&te);
#endif
//    SelectObject(dc, ofh);
    ReleaseDC(0, dc);

    free(newstr);
    return (te.cx);
}
#endif
